/***********************************************************
 *                   Acknowledge                           *
 * This file contains the code for the HotLine Acknowledge *
 * Dialog Box.                                             *
 *                                                         *
 *                      History                            *
 * 19/05/2006  Initial Version                             *
 * 23/05/2006  HotLine Version 4.0                         *
 * 26/05/2006  Create TextNode to wrap text in Alert       *
 * 06/06/2006  Remove display of XUL use the Abstract      *
 * 15/06/2006  Add Tag to Acknowledgement Dialog           *
 ***********************************************************/

/***********************************************************
 *                 T_Acknowledge                           *
 *  This type handles the Acknowledge Dialog.              *
 ***********************************************************/

//  T_Acknowledge Properties
function T_Acknowledge(){
}// of T_Acknowledge

//  T_Acknowledge Methods
T_Acknowledge.prototype.Initialize = function (){
  var Element;
  var AlwaysUseIFrame=false;
  //  Called when the load event happens
  this.Acknowledgement=window.arguments[0];
  var Acknowledgement=this.Acknowledgement;
  //  Setup the Dialog for this Acknowledgement
  document.title=T_HotLineConstant.ExtensionTitle + ' ' + Acknowledgement.AckType;
  Element=document.getElementById('Acknowledge-Dialog');
  if (!Acknowledgement.ItsAnAlert){
    //  If its a Status Item, remove the buttons. The ondialogcancel
    //  event fires when the window is closed.
    var Button=Element.getButton('accept');
    Button.parentNode.removeChild(Button);
    var Button=Element.getButton('cancel');
    Button.label='Cancel';  //  Need a StringBundle for this
  };
  //  Add the StyleSheet for the Host
  var Skin=T_HotLineConstant.ChromeSkin(
                    T_HotLineConstant.StyleSheetFileName(Acknowledgement.HostName));
  var StyleSheet=document.createProcessingInstruction("xml-stylesheet",
                                                       "href='" + Skin + "' type='text/css'");
  document.insertBefore(StyleSheet, document.documentElement); 
  //  Show the Details
  Element=document.getElementById('Ack-Content-Caption');
  Element.setAttribute('class',T_HotLineConstant.AttributeNameClass(Acknowledgement.HostName));
  Element.label=Acknowledgement.AckType;

  Element=document.getElementById('Ack-Details-Caption');
  Element.setAttribute('class',T_HotLineConstant.AttributeNameClass(Acknowledgement.HostName));

  Element=document.getElementById('Ack-HostName-Label');
  Element.setAttribute('class',T_HotLineConstant.AttributeNameClass(Acknowledgement.HostName));
  Element=document.getElementById('Ack-HostName');
  Element.setAttribute('class',T_HotLineConstant.HostClass(Acknowledgement.HostName));
  Element.value=Acknowledgement.HostName;

  Element=document.getElementById('Ack-LocalTime-Label');
  Element.setAttribute('class',T_HotLineConstant.AttributeNameClass(Acknowledgement.HostName));
  Element=document.getElementById('Ack-LocalTime');
  Element.setAttribute('class',T_HotLineConstant.HostClass(Acknowledgement.HostName));
  Element.value=Acknowledgement.LocalTime.toLocaleString();

  Element=document.getElementById('Ack-Sender-Label');
  Element.setAttribute('class',T_HotLineConstant.AttributeNameClass(Acknowledgement.HostName));
  Element=document.getElementById('Ack-Sender');
  Element.setAttribute('class',T_HotLineConstant.HostClass(Acknowledgement.HostName));
  Element.value=Acknowledgement.Opal;

  Element=document.getElementById('Ack-TimeStamp-Label');
  Element.setAttribute('class',T_HotLineConstant.AttributeNameClass(Acknowledgement.HostName));
  Element=document.getElementById('Ack-TimeStamp');
  Element.setAttribute('class',T_HotLineConstant.HostClass(Acknowledgement.HostName));
  Element.value=Acknowledgement.TimeStamp.toLocaleString();

  if (Acknowledgement.Tag.length==0){
    Element=document.getElementById('Ack-Tag-Row');
    Element.collapsed=true;
  }
  else{
    Element=document.getElementById('Ack-Tag-Label');
    Element.setAttribute('class',T_HotLineConstant.AttributeNameClass(Acknowledgement.HostName));
    Element=document.getElementById('Ack-Tag');
    Element.setAttribute('class',T_HotLineConstant.HostClass(Acknowledgement.HostName));
    Element.value=Acknowledgement.Tag;
  };

  //  Show the Content

  Element=document.getElementById('Ack-Content-GroupBox');

  var Description=document.createElement('description');
  Description.setAttribute('class',T_HotLineConstant.HostClass(Acknowledgement.HostName));
  var TextLine = document.createTextNode(Acknowledgement.Content);
  Description.appendChild(TextLine);
  Description.flex='1';
  Element.appendChild(Description);

}// of  T_Acknowledge.prototype.Initialize

T_Acknowledge.prototype.Finalize = function (){
  //  Called when the unload event happens
}// of  T_Acknowledge.prototype.Finalize

T_Acknowledge.prototype.DoAckNow = function (){
  //  Called when the ondialogaccept event happens
  this.Acknowledgement.Acknowledged=true;
}// of  T_Acknowledge.prototype.DoAckNow

T_Acknowledge.prototype.DoAckLater = function (){
  //  Called when the ondialogcancel event happens
  this.Acknowledgement.Acknowledged=false;
}// of  T_Acknowledge.prototype.DoAckLater


/***********************************************************
 *                 T_Acknowledgement                       *
 *  This type defines the parameters and the result of     *
 *  the acknowledgement and is provided to the Acknowledge *
 *  Dialog as a window argument.                           *
 ***********************************************************/

//  T_Acknowledgement Properties
function T_Acknowledgement(){

  //  If any error occurs, the ErrorReason is set to the reason,
  //  and the Error method returns True.
  this.ErrorReason='';

  //  This property is set to True if the user presses the
  //  'Acknowledge Now' Key. If the 'Acknowledge Later' key is
  //  pressed, the property is set to False.
  this.Acknowledged=false;

  //  These properties are set in Initialize
  this.HostName='';

}// of T_Acknowledgement

T_Acknowledgement.prototype.Initialize = function (AckType,ItsAnAlert,HostName,LocalTime,ClassName, 
                                                     Opal,TimeStamp,Content,Tag,Reason){
  //  LocalTime is a Date Object
  this.AckType=AckType;
  this.ItsAnAlert=ItsAnAlert;
  this.HostName=HostName;
  this.LocalTime=LocalTime;
  this.ClassName=ClassName;
  this.Opal=Opal;
  this.TimeStamp=TimeStamp;
  this.Content=Content;
  this.Tag=Tag;
  return true;
}// of  T_Acknowledgement.prototype.Initialize


T_Acknowledgement.prototype.Error = function (Reason){
  return (!Empty(Reason.Text=this.ErrorReason));
}// of  T_Acknowledgement.prototype.Error


/***********************************************************
 *                 DoAcknowledge                           *
 * To Acknowledge a HotLine Alert or Status message        *
 * create a T_Acknowledgement object and pass it to        *
 * the Acknowledge function below. Check the Acknowledged  *
 * property if the function returns true. If the function  *
 * returns false, an error occurred and it is described    *
 * in Reason.                                              *
 ***********************************************************/

var Acknowledge = new T_Acknowledge();

function DoAcknowledge(Acknowledgement,Reason){
  window.openDialog(T_HotLineConstant.ChromeContent('acknowledge.xul'),
                    "",
                    "chrome,modal,dialog,resizable",
                    Acknowledgement);
  return !Acknowledgement.Error(Reason);
}// of  DoAcknowledge


