/***********************************************************
 *                   BridgeProtocol                        *
 * This file contains JavaScript types and functions which *
 * implement the CopyWrite Bridge Protocol used to connect *
 * an MCP system to another Operating System.              *
 *                                                         *
 * T_Rq  - Common request constants and functions and is   *
 *         the base type for the CopyWrite Transport       *
 *         requests.                                       *
 *                                                         *
 * T_ConnectRq - A connect request packed which contains   *
 *               the service to be used, the user name     *
 *               and other proerties for the session.      *
 * T_ConnectResp - The response to a connect request       *
 *                 indicating success or failure and       *
 *                 a reason.                               *
 * T_DataRq - A data packet sent to or received from       *
 *            an MCP system.                               *
 * T_MotifyRq - A notification message sent to or received *
 *              from an MCP system.                        *
 *                                                         *
 * Uses                                                    *
 *   generalsupport.js                                     *
 *   mcpsupport.js                                         *
 *                                                         *
 *                      History                            *
 * 16/02/2006  Initial Version                             *
 * 23/05/2006  HotLine Version 4.0                         *
 ***********************************************************/

//**********************************************************
//                T_Rq                                     *
//**********************************************************

//  T_Rq Properties
function T_Rq(){
}
//  T_Rq Methods

//  T_Rq Class Items (Header)

T_Rq.MSGLENGTHV         = 4;     // 4 Bytes
T_Rq.MSGTYPEV           = 1;     // 1 Byte
  T_Rq.CONNECTRQV       = 1;
  T_Rq.CONNECTRESPONSEV = 2;
  T_Rq.DATARQV          = 3;
  T_Rq.NOTIFYRQV        = 4;
T_Rq.MSGVERSIONV        = 1;     // 1 Byte
  T_Rq.CURRENTVERSIONV  = 2;
T_Rq.MSGTAGV            = 6;     // 6 Byte 
T_Rq.MSGFIXEDSIZEV      = (T_Rq.MSGLENGTHV +
                           T_Rq.MSGTYPEV +
                           T_Rq.MSGVERSIONV +
                           T_Rq.MSGTAGV);

//  T_Rq Class Items (Attributes)

T_Rq.ATTNOSIZEV          = 1;     // 1 Byte
  T_Rq.USERNAMEV         = 1;
  T_Rq.SERVICENAMEV      = 2;
  T_Rq.LIBACCESSV        = 3;
    T_Rq.BYTITLEV        = 1;
    T_Rq.BYFUNCTIONV     = 2;
    T_Rq.BYINITIATORV    = 3;
  T_Rq.FUNCTIONNAMEV     = 4;
  T_Rq.TITLEV            = 5;
  T_Rq.REASONV           = 6;
  T_Rq.PARAMETERV        = 7;
  T_Rq.HOSTNAMEV         = 8;
T_Rq.ATTLENGTHSIZEV      = 4;     // 4 Bytes
T_Rq.ATTHEADERSIZEV      = 5;    

T_Rq.INTEGERVALSIZEV     = 4;
T_Rq.WORDVALSIZEV        = 6;
T_Rq.StringAttSize = function(L){return (T_Rq.ATTHEADERSIZEV+L)}
T_Rq.IntegerAttSize = function(){return (T_Rq.ATTHEADERSIZEV+T_Rq.INTEGERVALSIZEV)}

T_Rq.PutStringAtt = function(which,value){
  return String.fromCharCode(which) +
          Int32ToString(value.length) +
          value;
}// of T_Rq.PutStringAtt

T_Rq.PutIntegerAtt = function(which,value){
  return String.fromCharCode(which) +
          Int32ToString(T_Rq.INTEGERVALSIZEV) +
          Int32ToString(value);
}// of T_Rq.PutIntegerAtt

T_Rq.BuildHeader = function(MsgLength,MsgType,MsgTag){
  return Int32ToString(MsgLength) +
           ByteToString(MsgType) +
           ByteToString(T_Rq.CURRENTVERSIONV) +
           Int48ToString(MsgTag);
}// of T_Rq.BuildHeader

T_Rq.GetInt32 = function(Msg,Index){
  var value=0;
  //  Extract the Network Order Integer32 at the given Index and
  //  return a Number
  MustBeString('T_Rq.GetInt32',Msg);
  for(var j=0;j<4;j++){
    value+=Msg.charCodeAt(Index+j) << ((3-j)*8);
  };
  return value;
}// of T_Rq.GetInt32

T_Rq.GetInt16 = function(Msg,Index){
  //  Extract the Network Order Integer16 at the given Index and
  //  return a Number
  MustBeString('T_Rq.GetInt16',Msg);
  return (Msg.charCodeAt(Index) << 8) + Msg.charCodeAt(Index+1);
}// of T_Rq.GetInt16

T_Rq.GetByte = function(Msg,Index){
  //  Extract the Byte at the given Index and return a Number
  MustBeString('T_Rq.GetByte',Msg);
  return (Msg.charCodeAt(Index));
}// of T_Rq.GetByte

T_Rq.GetBoolean = function(Msg,Index){
  //  Extract the Boolean at the given Index
  MustBeString('T_Rq.GetBoolean',Msg);
  return (Msg.charCodeAt(Index)!=0);
}// of T_Rq.GetBoolean

T_Rq.GetMsgLength = function(Msg){
  //  Extract the MsgLength from the Msg
  return T_Rq.GetInt32(Msg,0);
}// of T_Rq.GetMsgLength

T_Rq.GetMsgType = function(Msg){
  //  Extract the MsgType from the Msg
  return T_Rq.GetByte(Msg,T_Rq.MSGLENGTHV);
}// of T_Rq.GetMsgType

T_Rq.GetMsgVersion = function(Msg){
  //  Extract the MsgVersion from the Msg
  return T_Rq.GetByte(Msg,(T_Rq.MSGLENGTHV+T_Rq.MSGTYPEV));
}// of T_Rq.GetMsgVersion

T_Rq.GetMsgTag = function(Msg){
  //  Extract the MsgTag from the Msg (ignore 2 leading bytes)
  return T_Rq.GetInt32(Msg,(T_Rq.MSGLENGTHV+T_Rq.MSGTYPEV+T_Rq.MSGVERSIONV+2));
}// of T_Rq.GetMsgLength

T_Rq.GetAttSize = function(Msg,Index){
  //  Return the total size of the Attribute at the given Index
  return T_Rq.ATTHEADERSIZEV+T_Rq.GetInt32(Msg,Index+T_Rq.ATTNOSIZEV);
}// of T_Rq.GetAttSize

//**********************************************************
//                T_ConnectRq                              *
//**********************************************************

//  T_ConnectRq Properties
function T_ConnectRq(){
}
//  Inherit from T_Rq
T_ConnectRq.prototype=new T_Rq();
T_ConnectRq.prototype.constructor=T_ConnectRq;

//  T_ConnectRq Methods

//  T_ConnectRq Class Items

T_ConnectRq.Build = function(MsgTag,UserName,ServiceName,LibAccess,
                               FunctionName,Title,Parameter,HostName){
  var MsgLength = T_Rq.MSGFIXEDSIZEV +
                  T_Rq.StringAttSize(UserName.length) +
                  T_Rq.StringAttSize(ServiceName.length) +
                  T_Rq.IntegerAttSize() +  // LibAccess
                  T_Rq.StringAttSize(FunctionName.length) +
                  T_Rq.StringAttSize(Title.length) +
                  T_Rq.StringAttSize(Parameter.length) +
                  T_Rq.StringAttSize(HostName.length);
  var ConnectRq = T_Rq.BuildHeader(MsgLength,T_Rq.CONNECTRQV,MsgTag) +
                                   T_Rq.PutStringAtt(T_Rq.USERNAMEV,
                                                     ASCIIToEBCDIC.Translate(UserName)) +
                                   T_Rq.PutStringAtt(T_Rq.SERVICENAMEV,
                                                     ASCIIToEBCDIC.Translate(ServiceName)) +
                                   T_Rq.PutIntegerAtt(T_Rq.LIBACCESSV,LibAccess) +
                                   T_Rq.PutStringAtt(T_Rq.FUNCTIONNAMEV,
                                                     ASCIIToEBCDIC.Translate(FunctionName)) +
                                   T_Rq.PutStringAtt(T_Rq.TITLEV,
                                                     ASCIIToEBCDIC.Translate(Title)) +
                                   T_Rq.PutStringAtt(T_Rq.PARAMETERV,Parameter) +
                                   T_Rq.PutStringAtt(T_Rq.HOSTNAMEV,
                                                     ASCIIToEBCDIC.Translate(HostName));
  return ConnectRq;
}// of T_ConnectRq.Build

//**********************************************************
//                T_ConnectResp                            *
//**********************************************************

//  T_ConnectResp Properties
function T_ConnectResp(){
  this.Reason='';
  this.Accepted=false;
}
//  Inherit from T_Rq
T_ConnectResp.prototype=new T_Rq();
T_ConnectResp.prototype.constructor=T_ConnectResp;

//  T_ConnectResp Methods

T_ConnectResp.prototype.Load = function(Msg){
  var j,k;
  var sMsg = T_Rq.GetMsgLength(Msg);
  var iMsg = T_Rq.MSGFIXEDSIZEV;
  //  Loads a validated Connect Response

  //  Accepted
  this.Accepted=(T_Rq.GetMsgTag(Msg)==1);

  //  Attributes
  while (iMsg < sMsg){
    var AttNo=T_Rq.GetByte(Msg,iMsg);
    switch(AttNo){
       //  Reason
  case T_Rq.REASONV:
      j=iMsg+T_Rq.ATTNOSIZEV;
      k=T_Rq.GetInt32(Msg,j);
      j+=T_Rq.ATTLENGTHSIZEV;
      this.Reason=EBCDICToASCII.Translate(Msg.substr(j,k));
      break;
  default:
      break;
    };
    iMsg+=T_Rq.GetAttSize(Msg,iMsg);
  }
}// of T_ConnectResp.prototype.Load

var ConnectResp = new T_ConnectResp();

//  T_ConnectResp Class Items

//**********************************************************
//                T_DataRq                                 *
//**********************************************************

//  T_DataRq Properties
function T_DataRq(){
}
//  Inherit from T_Rq
T_DataRq.prototype=new T_Rq();
T_DataRq.prototype.constructor=T_DataRq;

//  T_DataRq Methods

T_DataRq.prototype.Load = function(Msg){
  //  Loads a validated Data Request
  this.Msg=Msg;
}// of T_DataRq.prototype.Load

T_DataRq.prototype.Data = function(){
  var DataLength = T_Rq.GetMsgLength(this.Msg) - T_Rq.MSGFIXEDSIZEV;
  //  Returns the Packet PayLoad
  return this.Msg.slice(T_Rq.MSGFIXEDSIZEV,T_Rq.MSGFIXEDSIZEV+DataLength);
}// of T_DataRq.prototype.Data

//  T_DataRq Class Items

T_DataRq.Build = function(MsgTag,Data){
  var MsgLength = T_Rq.MSGFIXEDSIZEV + Data.length;
  var DataRq = T_Rq.BuildHeader(MsgLength,T_Rq.DATARQV,MsgTag) + Data;
  return DataRq;
}// of T_DataRq.Build

//**********************************************************
//                T_NotifyRq                               *
//**********************************************************

//  T_NotifyRq Properties
function T_NotifyRq(){
}
//  Inherit from T_Rq
T_NotifyRq.prototype=new T_Rq();
T_NotifyRq.prototype.constructor=T_NotifyRq;

//  T_NotifyRq Methods
T_NotifyRq.prototype.Load = function(Msg){
  //  Loads a validated Notify Request
  this.Msg=Msg;
}// of T_NotifyRq.prototype.Load

T_NotifyRq.prototype.Text = function(){
  var DataLength = T_Rq.GetMsgLength(this.Msg) - T_Rq.MSGFIXEDSIZEV;
  //  Returns the Packet PayLoad
  return EBCDICToASCII.Translate(this.Msg.slice(T_Rq.MSGFIXEDSIZEV,T_Rq.MSGFIXEDSIZEV+DataLength));
}// of T_NotifyRq.prototype.Text

//  T_NotifyRq Class Items

T_NotifyRq.Build = function(TextData){
  var MsgLength = T_Rq.MSGFIXEDSIZEV + TextData.length;
  var NotifyRq = T_Rq.BuildHeader(MsgLength,T_Rq.NOTIFYRQV,0) + 
                  ASCIIToEBCDIC.Translate(TextData);
  return NotifyRq;
}// of T_NotifyRq.Build


//**********************************************************
//                T_Dispatcher                             *
//  Requests have a MsgLength as the first field of the    *
//  Message. The T_Dispatcher accumulates Bytes received   *
//  on the TCP session, and returns complete messages      *
//  to GetMsg.                                             *
//**********************************************************

//  T_Dispatcher Properties

function T_Dispatcher(){
  this.Buffer='';
}// of T_Dispatcher

//  T_Dispatcher Methods

T_Dispatcher.prototype.Receive = function(Bytes){
  this.Buffer+=Bytes;
  return true;
}// of T_Dispatcher.prototype.Receive

T_Dispatcher.prototype.GetMsg = function(){
  if (this.Buffer.length <  T_Rq.MSGLENGTHV)
    return '';
  var MsgLength = T_Rq.GetMsgLength(this.Buffer);
  if (this.Buffer.length < MsgLength)
    return '';
  var Msg = this.Buffer.slice(0,MsgLength);
  this.Buffer = this.Buffer.slice(MsgLength,this.Buffer.length);
//  alert('Msg='+HexString(Msg)+'<');
//  alert('Buffer='+HexString(this.Buffer)+'<');
  return Msg;
}// of T_Dispatcher.prototype.GetMsg

