/***********************************************************
 *                   Configurator                          *
 * This file supports the configuration of HotLine.        *
 *                                                         *
 * Uses                                                    *
 *   generalsupport.js                                     *
 *   mcpsupport.js                                         *
 *                                                         *
 *                      History                            *
 * 27/03/2006  Initial Version                             *
 * 23/05/2006  HotLine Version 4.0                         *
 *             Implement HoursToKeep attribute             *
 * 30/05/2006  Implement Save and Restore Configuration    *
 * 09/06/2006  Invert Colour for Stale <description>       *
 ***********************************************************/

//**********************************************************
//                T_Configurator                           *
//  This type is used to manage the hosts.rdf file.        *
//**********************************************************

//  T_Configurator Properties
function T_Configurator(){
}// of T_Configurator

T_Configurator.ItsSet            = 'yes';
T_Configurator.ItsReset          = 'no';

//  T_Status Methods
T_Configurator.prototype.Initialize = function (HotLineConstant,Status,Reason){
  var HostList;
  //  When Verbose is set there are additional messages
  //  sent to the Status panel. It could be a Host
  //  configuration value some time.
  this.Verbose=true;
  //  Initialize Status Messages
  this.Status=Status;
  //  Store a reference to the HotLine Constants
  this.HotConstant=HotLineConstant;

  //  Recover from Backup Configuration if Reinstallation

  if (!this.Recovery(Reason))
    return false;

  //  Load the Host Configuration RDF

  this.HostConfigFile=new T_File(this.BuildDataPath(T_HotLineConstant.HostsConfigFile));
  this.RDF=new T_RDF(T_HotLineConstant.RDFNameSpace);
  this.Display('','Loading ' + this.HostConfigFile.Path.ToURI());
  if (!this.RDF.Initialize(this.HostConfigFile.Path.ToURI(),true,Reason))
    return false;
  //  Subject URIs 
  this.HostsSubjectURI = this.RDF.SubjectURI('hosts');
  //  Predicate URIs 
  this.AttName_Predicate_URI = this.RDF.PredicateURI('attname');
  this.AttValue_Predicate_URI = this.RDF.PredicateURI('attvalue');
  //  These predicates are used with the TypeNames
  this.MnemValue_Predicate_URI = this.RDF.PredicateURI('mnemvalue');
  this.RecordFilterPredicateURI = this.RDF.PredicateURI(T_HotLineConstant.RecordFilter);
  this.AlertFilterPredicateURI = this.RDF.PredicateURI(T_HotLineConstant.AlertFilter);
  this.StatusFilterPredicateURI = this.RDF.PredicateURI(T_HotLineConstant.StatusFilter);

  //  Ensure there is a Hosts Container for the Template to function

  HostList=new T_RDFContainer(this.RDF,this.HostsSubjectURI);
  if (!(HostList.IsContainer(Reason) ? HostList.Open(Reason) : HostList.Create(Reason)))
    return false;

  this.RDF.Save();

  //  This is disabled until building the HostNames-MenuList
  //  using an RDF Template works without crashing FireFox
  //  this.AddToDataSources("HostNames-MenuList",true);

  this.AddToDataSources("Hosts-Tree",true);

  this.AddToDataSources("TT-REC-NAMES-ArrowScrollBox",true);
  this.AddToDataSources("Record-Filter-ArrowScrollBox",true);
  this.AddToDataSources("Alert-Filter-ArrowScrollBox",true);
  this.AddToDataSources("Status-Filter-ArrowScrollBox",true);
  
  return true;
}// of T_Configurator.prototype.Initialize

T_Configurator.prototype.Recovery = function (Reason){
  //  Check if the Host Configuration File is Resident
  var HostConfigFile=new T_File(this.BuildDataPath(T_HotLineConstant.HostsConfigFile));
  if (HostConfigFile.Resident())
    return true;
  this.Display('',HostConfigFile.Path.ToURI() + ' Not Resident');
  //  Check if the Backup Directory exists. If it does not, there
  //  is nothing to Recover
  var ProfileDirectory=new T_Directory(new T_ProfileDirectory(T_HotLineConstant.BackupDirectory));
  this.Display('','Backup Directory is ' + ProfileDirectory.Name());
  if (!ProfileDirectory.Resident())
    return true;
  //  Check if there is a Backup Host Configuration.
  var HostConfigFile=new T_File(ProfileDirectory.Path);
  HostConfigFile.Path.Append(T_HotLineConstant.HostsConfigFile);
  this.Display('','Searching for ' + HostConfigFile.Path.ToURI());
  if (!HostConfigFile.Resident())
    return true;
  //  Recover the previous configuration
  return this.RestoreConfiguration(Reason);
}// of  T_Configurator.prototype.Recovery

T_Configurator.prototype.Finalize = function (Reason){
  return this.SaveConfiguration(Reason);
}// of  T_Configurator.prototype.Finalize

T_Configurator.prototype.Activated = function (Active){
  //  Configurator has become Visible
if (Active){
  //  Check if there is a Host Selected
}
else{
};
}// of T_Configurator.prototype.Activated

T_Configurator.prototype.Display = function (HostName,Text){
  this.Status.Display(HostName,Text);
}// of T_Configurator.prototype.Display

T_Configurator.prototype.Flush = function (){
  this.RDF.Save();
}// of T_Configurator.prototype.Flush

T_Configurator.prototype.BuildPath = function (FileName){
  //  Build and return a T_Path object to the FileName
  //  in the Profile Directory.
  return new T_ExtensionDirectory(T_HotLineConstant.ExtensionID,
                                    'content',
                                    FileName);
}// of T_Configurator.prototype.BuildPath

T_Configurator.prototype.BuildDataPath = function (FileName){
  //  Build and return a T_Path object to the FileName
  //  in the DataPath Directory.
  return new T_LocalDirectory(this.HotConstant.DataPath.path,
                              FileName);
}// of T_Configurator.prototype.BuildDataPath

T_Configurator.prototype.GetDataPath = function (){
  //  Build a T_Path object to Data Directory
  return new T_LocalDirectory(this.HotConstant.DataPath.path);
}// of T_Configurator.prototype.GetDataPath

T_Configurator.prototype.ConfigurationFileName = function (){
  return this.HostConfigFile.Path.ToDisplay();
}// of T_Configurator.prototype.ConfigurationFileName

T_Configurator.prototype.AddToDataSources = function (ElementName,Rebuild){
  this.RDF.AddToDataSources(ElementName,Rebuild);
}// of T_Configurator.prototype.AddToDataSources

T_Configurator.prototype.RemoveFromDataSources = function (ElementName,Rebuild){
  this.RDF.RemoveFromDataSources(ElementName,Rebuild);
}// of T_Configurator.prototype.RemoveFromDataSources

T_Configurator.prototype.ResetDataSources = function (ElementName,Rebuild){
  this.RDF.ResetDataSources(ElementName,Rebuild);
}// of T_Configurator.prototype.ResetDataSources

T_Configurator.prototype.BuildTypeFaceList = function (){
  //  This function builds an inmemory datasource
  //  containing the typeface names.


}// of T_Configurator.prototype.BuildTypeFaceList

//**********************************************************
//              Edit Functions                             *
//**********************************************************

T_Configurator.prototype.ValidHostName = function (Text,HostName,Reason){
  //  This function removes leading and trailing spaces
  //  from the text and then extracts the leading characters
  //  which comprise a valid HostName. If the HostName 
  //  extracted matches the trimmed text, the function
  //  returns true, otherwise it returns false and a reason.
  //  The characters that comprise a valid HostName are
  // returned in the Value property of the T_ByReference
  // HostName variable.
  Text=Trim(Text);
  var Result=Text.match(/[a-zA-Z][-_[a-zA-Z0-9]*/);
  if (Result == null){
    Reason.Text='Invalid HostName. Scanning =>' + Text + '<';
    HostName.Value='';
    return false;
  };
  HostName.Value=Result[0];
  if (HostName.Value == Text)
    return true;
  Reason.Text='Invalid Character for HostName. Scanning =>' + 
                      Text.substring(HostName.Value.length,Text.length) + '<';
}// of T_Configurator.prototype.ValidHostName

T_Configurator.prototype.ValidTypeName = function (Text,TypeName,Reason){
  //  See ValidHostName comment..
  Text=Trim(Text);
  var Result=Text.match(/[a-zA-Z][-_[a-zA-Z0-9]*/);
  if (Result == null){
    Reason.Text='Invalid TypeName. Scanning =>' + Text + '<';
    TypeName.Value='';
    return false;
  };
  TypeName.Value=Result[0];
  if (TypeName.Value == Text)
    return true;
  Reason.Text='Invalid Character for TypeName. Scanning =>' + 
                      Text.substring(TypeName.Value.length,Text.length) + '<';
}// of T_Configurator.prototype.ValidTypeName

T_Configurator.prototype.ValidTypeValue = function (Text,TypeValue,Reason){
  //  See ValidHostName comment..
  Text=Trim(Text);
  var Result=Text.match(/[0-9]+/);
  if (Result == null){
    Reason.Text='Invalid TypeValue. Scanning =>' + Text + '<';
    TypeValue.Value='';
    return false;
  };
  TypeValue.Value=Result[0];
  if (TypeValue.Value == Text)
    return true;
  Reason.Text='Invalid Character for TypeValue. Scanning =>' + 
                      Text.substring(TypeValue.Value.length,Text.length) + '<';
}// of T_Configurator.prototype.ValidTypeValue

T_Configurator.prototype.ParseTypeNameAndValueList = function (Caller,Text,HandleTypeNameAndValue,Reason){
  var Items=Text.match(/( *[-_a-zA-Z0-9]+ *= *[0-9]+ *(,|$))/g); 
  if ((Items == null) || (Items.length == 0)){
    Reason.Text='No <name>=<value> list in "' + Text + '"';
    return false;
  };
  //  Call HandleTypeNameAndValue for each <name>=<value> pair
  for (var j=0;j<Items.length;j++){
    var NameAndValue=Items[j].match(/ *([-_a-zA-Z0-9]+) *= *([0-9]+) *,? */);
    if (NameAndValue.length != 3){
      Reason.Text='Invalid <name>=<value> pair in ' + Items[j];
      return false;
    };
    HandleTypeNameAndValue.call(Caller,NameAndValue[1],NameAndValue[2]);
  };
  return true;
}// of T_Configurator.prototype.ParseTypeNameAndValueList

//**********************************************************
//              Host Functions                             *
//**********************************************************

T_Configurator.prototype.FindHost = function (HostName,Reason){
  var Literal=new T_ByReference();
  if (!this.RDF.GetLiteral(this.RDF.SubjectURI(HostName),
                           this.AttValue_Predicate_URI,
                           Literal,
                           Reason))
    return false;
  return true;
}// of  T_Configurator.prototype.FindHost

T_Configurator.prototype.FindSimilarHost = function (Name,HostName,Reason){
  //  Returns true if a similar HostName already exists,
  //  differeing only in case, returned in HostName T_ByReference
  var HostNames=this.GetHosts(Reason);
  if (HostNames==null)
    return false;
  Name=Name.toLowerCase();
  for (var j=0;j<HostNames.length;j++){
    if (HostNames[j].toLowerCase()==Name){
      HostName.Value=HostNames[j];
      return true;
    };
  };
  Reason.Text=Name + ' is not configured';
  return false;
}// of  T_Configurator.prototype.FindSimilarHost


T_Configurator.prototype.CreateHost = function (HostName,Reason){
  //  Create the HostList Container if Not Present
  var HostList=new T_RDFContainer(this.RDF,this.HostsSubjectURI);
  if (!(HostList.IsContainer(Reason) ? HostList.Open(Reason) : HostList.Create(Reason)))
    return false;

  //  Create a Host Resource
  var HostURI=this.RDF.SubjectURI(HostName);
  var Host=new T_RDFDescription(this.RDF);
  if (!Host.Initialize(HostURI,Reason))
    return false;
  //  Create the HostName Attribute
  var AttName=new T_RDFAttribute(this.RDF,this.AttName_Predicate_URI);
  AttName.SetToLiteral('HostName');
  Host.Insert(AttName);
  //  Create the Label Attribute 
  var AttValue=new T_RDFAttribute(this.RDF,this.AttValue_Predicate_URI);
  AttValue.SetToLiteral(HostName);
  Host.Insert(AttValue);

  //  Create the Host Attributes Container
  var HostAttributes=new T_RDFContainer(this.RDF,HostURI);
  if (!HostAttributes.Create(Reason))
    return false;
  //  Insert the Container Reference into the HostList List
  HostList.Insert(HostAttributes);

  //  Save the Configuration File
  this.Flush();

  return true;
}// of  T_Configurator.prototype.CreateHost


T_Configurator.prototype.DeleteHost = function (HostName,Reason){
  //  Open the List of Hosts as a Container
  var HostList=new T_RDFContainer(this.RDF,this.RDF.SubjectURI('hosts'));
  if (!HostList.Open(Reason))
     return false;
  //  Destroy the Host
  var Host=new T_RDFContainer(this.RDF,this.RDF.SubjectURI(HostName));
  if (!Host.Open(Reason))
     return false;
  if (!Host.Destroy(Reason))
     return false;
  //  Remove the Host from the list of Hosts
  HostList.Remove(Host);
  //  Remove the Log File from the DataSources
  this.Display(HostName,'Removing ' + HostName + ' from the DataSources');
  this.RemoveFromDataSources("Hosts-Tree",true);
  this.RemoveFromDataSources("TT-REC-NAMES-ArrowScrollBox",true);
  this.RemoveFromDataSources("Record-Filter-ArrowScrollBox",true);
  this.RemoveFromDataSources("Alert-Filter-ArrowScrollBox",true);
  this.RemoveFromDataSources("Status-Filter-ArrowScrollBox",true);
  //  This HostName may not have all these attributes, so
  //  Display the Reason rather than failing the Delete
  this.Display(HostName,'Deleting ' + HostName + ' from the Configuration');

  if (!this.DeleteUserName(HostName,Reason))
    this.Display(HostName,Reason.Text);
  if (!this.DeleteIPAddress(HostName,Reason))
    this.Display(HostName,Reason.Text);
  if (!this.DeleteComment(HostName,Reason))
    this.Display(HostName,Reason.Text);
  if (!this.DeleteColour(HostName,Reason))
    this.Display(HostName,Reason.Text);
  if (!this.DeleteFontFamily(HostName,Reason))
    this.Display(HostName,Reason.Text);
  if (!this.DeleteTextSize(HostName,Reason))
    this.Display(HostName,Reason.Text);
  if (!this.DeleteConnect(HostName,Reason))
    this.Display(HostName,Reason.Text);
  if (!this.DeleteDisconnect(HostName,Reason))
    this.Display(HostName,Reason.Text);
  if (!this.DeleteConnected(HostName,Reason))
    this.Display(HostName,Reason.Text);
  if (!this.DeleteAutoConnect(HostName,Reason))
    this.Display(HostName,Reason.Text);
  if (!this.DeleteSelectAll(HostName,Reason))
    this.Display(HostName,Reason.Text);
  if (!this.DeleteTypeMask(HostName,Reason))
    this.Display(HostName,Reason.Text);
  if (!this.DeleteHoursToKeep(HostName,Reason))
    this.Display(HostName,Reason.Text);

  //  Delete the Attributes

  this.Display(HostName,'Deleting Attributes');
  if (!this.DeleteAttributes(HostName,Reason))
    this.Display(HostName,Reason.Text);


  //  Delete the TypeNames 

  this.Display(HostName,'Deleting TypeNames');
  if (!this.DeleteTypeNames(HostName,Reason))
    this.Display(HostName,Reason.Text);

  //  Remove the Log File for the Host

  
  //  Save the Configuration File
  this.Flush();
  this.Display(HostName,'Deleted ' + HostName);
  return true;
}// of  T_Configurator.prototype.DeleteHost

T_Configurator.prototype.GetHosts = function (Reason){
  //  This function returns an array of the HostNames
  //  that are defined in the RDF configuration file.
  //  If there are no hosts, the function returns null 
  //  and a Reason, otherwise true and the array of HostNames.
  //  Note:  See also this.RestoreConfiguration
  var Index;
  var Hosts = new Array();
  var HostList=new T_RDFContainer(this.RDF,this.HostsSubjectURI);
  //  Create the HostList Container if Not Present
  if (!(HostList.IsContainer(Reason) ? HostList.Open(Reason) : HostList.Create(Reason)))
    return null;
  var URIList=HostList.GetURIList(Reason);
  if (URIList==null)
    return null;
  for(var j=0;j<URIList.length;j++)
    //  Check for the HotLine NameSpace
    if ((Index=URIList[j].search(T_HotLineConstant.RDFNameSpace + '/')) >= 0)
      Hosts.push(URIList[j].substring(Index+T_HotLineConstant.RDFNameSpace.length+1,URIList[j].length));
  if (Hosts.length == 0){
    Reason.Text='No Hosts Configured';
    return null;
  };
  return Hosts;
}// of  T_Configurator.prototype.GetHosts

T_Configurator.prototype.SaveConfiguration = function (Reason){
  //  Saves the Configuration files to the Profile Directory

  //  Remove the old backup directory
  var OldDirectory=new T_Directory(new T_ProfileDirectory(T_HotLineConstant.OldBackupDirectory));
  if (OldDirectory.Resident()){
    if (!OldDirectory.Remove(Reason))
      return false;
    this.Display('','Removed ' + OldDirectory.Name());
  };
  //  Rename the current backup directory to old backup directory
  var Directory=new T_Directory(new T_ProfileDirectory(T_HotLineConstant.BackupDirectory));
  if (Directory.Resident()){
    if (!Directory.Rename(T_HotLineConstant.OldBackupDirectory,Reason))
      return false;
    this.Display('','Renamed ' + Directory.Name() + ' to ' + T_HotLineConstant.OldBackupDirectory);
  };
  if (false){
    //  Create a new backup directory
    var Directory=new T_Directory(new T_ProfileDirectory(T_HotLineConstant.BackupDirectory));
    if (!Directory.Make(T_HotLineConstant.DirPermissions,Reason))
       return false;
    this.Display('','Saving Configuration Files to ' + Directory.Name());
    //  Copy the Hosts Configuration
    this.Display('','Copying ' + this.HostConfigFile.Path.ToDisplay());
    if (!this.HostConfigFile.CopyTo(Directory,Reason))
      return false;
    //  Copy the Configuration Files for each Host
    var HostNames=this.GetHosts(Reason);
    if (HostNames != null){
      for (var j=0;j<HostNames.length;j++){
        //  Copy the StyleSheet
        var StyleSheet=new T_File(this.StyleSheetPath(HostNames[j]));
        this.Display('','Copying ' + StyleSheet.Path.ToDisplay());
        if (!StyleSheet.CopyTo(Directory,Reason))
          this.Display('',Reason.Text);
  
        //  Copy the Log
        var Log=new T_File(this.LogFilePath(HostNames[j]));
        this.Display('','Copying ' + Log.Path.ToDisplay());
        if (!Log.CopyTo(Directory,Reason))
          this.Display('',Reason.Text);
      };  
    };
    this.Display('','Saved Configuration Files to ' + Directory.Name());
  };
  return true;
}// of  T_Configurator.prototype.SaveConfiguration

T_Configurator.prototype.RestoreConfiguration = function (Reason){
  //  Restores the Configuration files from the Profile Directory

  //  Check that the Configurations Files are in the Profile Directory
  var ProfileDirectory=new T_Directory(new T_ProfileDirectory(T_HotLineConstant.BackupDirectory));
  if (!ProfileDirectory.Resident()){
    Reason.Text='No Directory ' + ProfileDirectory.Path.ToDisplay();
    return false;
  };
  //  Destination Directories
  var ContentDirectory=new T_Directory(this.GetDataPath());
  var SkinDirectory=new T_Directory(this.GetDataPath());

  this.Display('','Restoring Configuration Files from ' + ProfileDirectory.Name() + ' to ' +
                   ContentDirectory.Name());

  //  Copy the Hosts Configuration
  var HostConfigFile=new T_File(ProfileDirectory.Path);
  HostConfigFile.Path.Append(T_HotLineConstant.HostsConfigFile);
  this.Display('','Copying ' + HostConfigFile.Path.ToDisplay() + ' to ' +
                   ContentDirectory.Name());
  if (!HostConfigFile.CopyTo(ContentDirectory,Reason))
    return false;
  var HostConfigFile=new T_File(this.BuildDataPath(T_HotLineConstant.HostsConfigFile));

  //  Load the Host Configuration File as RDF

  var RDF=new T_RDF(T_HotLineConstant.RDFNameSpace);
  this.Display('','Loading ' + HostConfigFile.Path.ToURI());
  if (!RDF.Initialize(HostConfigFile.Path.ToURI(),true,Reason))
    return false;

  //  Build the HostName List

  var HostNames = new Array();
  var HostsSubjectURI = RDF.SubjectURI('hosts');
  var HostList=new T_RDFContainer(RDF,HostsSubjectURI);
  //  Create the HostList Container if Not Present
  if (HostList.IsContainer(Reason) && HostList.Open(Reason)){
    var URIList=HostList.GetURIList(Reason);
    if (URIList!=null){
      for(var j=0;j<URIList.length;j++)
        //  Check for the HotLine NameSpace
        if ((Index=URIList[j].search(T_HotLineConstant.RDFNameSpace + '/')) >= 0)
          HostNames.push(URIList[j].substring(Index+T_HotLineConstant.RDFNameSpace.length+1,URIList[j].length));
    };
  };
  
  //  Copy the Configuration Files for each Host

  if (HostNames.length > 0){
    for (var j=0;j<HostNames.length;j++){
    //  Copy the StyleSheet
      var ProfileDirectory=new T_Directory(new T_ProfileDirectory(T_HotLineConstant.BackupDirectory));
      var StyleSheet=new T_File(ProfileDirectory.Path);
      StyleSheet.Path.Append(T_HotLineConstant.StyleSheetFileName(HostNames[j]));
      this.Display('','Copying ' + StyleSheet.Path.ToDisplay() + ' to ' +
                      SkinDirectory.Name());
      if (!StyleSheet.CopyTo(SkinDirectory,Reason))
        this.Display('',Reason.Text);
      //  Copy the Log
      var ProfileDirectory=new T_Directory(new T_ProfileDirectory(T_HotLineConstant.BackupDirectory));
      var Log=new T_File(ProfileDirectory.Path);
      Log.Path.Append(T_HotLineConstant.LogFileName(HostNames[j]));
      this.Display('','Copying ' + Log.Path.ToDisplay() + ' to ' +
                       ContentDirectory.Name());
      if (!Log.CopyTo(ContentDirectory,Reason))
        this.Display('',Reason.Text);
    };  
  };

  var ProfileDirectory=new T_Directory(new T_ProfileDirectory(T_HotLineConstant.BackupDirectory));
  this.Display('','Restored Configuration Files from ' + ProfileDirectory.Name() + ' to ' +
                  ContentDirectory.Name());
  return true;
}// of  T_Configurator.prototype.RestoreConfiguration

T_Configurator.prototype.HasAttributeReference = function (HostName,AttributeURI,Reason){
  //  This function returns TRUE if the given attribute reference
  //  was already present, or has been inserted, in the Seq container
  //  for the specified HostName.
  var HostAttributes=new T_RDFContainer(this.RDF,this.RDF.SubjectURI(HostName));
  if (!(HostAttributes.IsContainer(Reason) ? HostAttributes.Open(Reason) : HostAttributes.Create(Reason)))
    return false;
  var Description = new T_RDFDescription(this.RDF);
  if (!Description.Initialize(AttributeURI,Reason))
    return false;
  HostAttributes.Insert(Description);
  return true;
}// of T_Configurator.prototype.HasAttributeReference

T_Configurator.prototype.DeleteAttributeReference = function (HostName,AttributeURI,Reason){
  //  This function returns TRUE if the given attribute reference
  //  was not present, or has been deleted, from the Seq container
  //  for the specified HostName.
  var HostAttributes=new T_RDFContainer(this.RDF,this.RDF.SubjectURI(HostName));
  if (!(HostAttributes.IsContainer(Reason) ? HostAttributes.Open(Reason) : HostAttributes.Create(Reason)))
    return false;
  var Description = new T_RDFDescription(this.RDF);
  if (!Description.Initialize(AttributeURI,Reason))
    return false;
  HostAttributes.Remove(Description);
  return true;
}// of T_Configurator.prototype.DeleteAttributeReference

//**********************************************************
//                Attribute                                *
//  AttResourceName - This is the name used as the last    *
//                    level of the subject URI for the     *
//                    attribute.                           *
//  AttName         - This is the Display name for the     *
//                    attribute given by the               *
//                    AttName_Predicate_URI.               *
//  AttValue        - This is the Display value for the    *
//                    attribute given by the               *
//                    AttValue_Predicate_URI.              *
//**********************************************************

T_Configurator.prototype.HasAttribute = function (HostName,AttResourceName,Reason){
  //  This function returns true if the AttResourceName is 
  //  configured for the HostName.
  return this.HasAttributeReference(HostName,this.RDF.SubjectURI(HostName + '/' + AttResourceName),Reason);
}// of T_Configurator.prototype.HasAttribute

T_Configurator.prototype.GetAttributeValue = function (HostName,AttResourceName,AttValue,Reason){
  //  If an AttResourceName is configured for the HostName, the
  //  function returns TRUE and the AttValue(T_ByReference),
  //  otherwise it returns FALSE and a Reason.
  return this.RDF.GetLiteral(this.RDF.SubjectURI(HostName + '/' + AttResourceName),
                              this.AttValue_Predicate_URI,
                              AttValue,
                              Reason);
}// of  T_Configurator.prototype.GetAttributeValue

T_Configurator.prototype.GetAttributeName = function (HostName,AttResourceName,AttName,Reason){
  //  If an AttResourceName is configured for the HostName, the
  //  function returns TRUE and the AttName(T_ByReference),
  //  otherwise it returns FALSE and a Reason.
  return this.RDF.GetLiteral(this.RDF.SubjectURI(HostName + '/' + AttResourceName),
                              this.AttName_Predicate_URI,
                              AttName,
                              Reason);
}// of  T_Configurator.prototype.GetAttributeName

T_Configurator.prototype.SetAttribute = function (HostName,AttResourceName,AttName,AttValue,Reason){
  //  This function modifies the AttResourceName for the HostName.
  //  If the attribute triple does not exist, it is created.,
  //  If there is an existing attribute, it is modified.
  if (!this.RDF.SetLiteral(this.RDF.SubjectURI(HostName + '/' + AttResourceName),
                            this.AttName_Predicate_URI,
                            AttName,
                            Reason))
    return false;
  if (!this.RDF.SetLiteral(this.RDF.SubjectURI(HostName + '/' + AttResourceName),
                            this.AttValue_Predicate_URI,
                            AttValue,
                            Reason))
    return false;
  if (!this.HasAttributeReference(HostName,this.RDF.SubjectURI(HostName + '/' + AttResourceName),Reason))
    return false;
  //  Save the Configuration File
  this.Flush();
  return true;
}// of  T_Configurator.prototype.SetAttribute

T_Configurator.prototype.DeleteAttribute = function (HostName,AttResourceName,Reason){
  //  This function deletes the AttResourceName for the HostName.
  if (!this.RDF.DeleteLiteral(this.RDF.SubjectURI(HostName + '/' + AttResourceName),
                              this.AttName_Predicate_URI,
                              Reason))
    return false;
  if (!this.RDF.DeleteLiteral(this.RDF.SubjectURI(HostName + '/' + AttResourceName),
                              this.AttValue_Predicate_URI,
                              Reason))
    return false;
  if (!this.DeleteAttributeReference(HostName,this.RDF.SubjectURI(HostName + '/' + AttResourceName),Reason))
    return false;
  //  Save the Configuration File
  this.Flush();
  return true;
}// of  T_Configurator.prototype.DeleteAttribute

//**********************************************************
//                UserName                                 *
//**********************************************************

T_Configurator.prototype.HasUserName = function (HostName,Reason){
  return this.HasAttribute(HostName,'username',Reason);
}// of  T_Configurator.prototype.HasUserName

T_Configurator.prototype.GetUserName = function (HostName,UserName,Reason){
  return this.GetAttributeValue(HostName,'username',UserName,Reason);
}// of  T_Configurator.prototype.GetUserName

T_Configurator.prototype.SetUserName = function (HostName,UserName,Reason){
  return this.SetAttribute(HostName,'username','UserName',UserName,Reason);
}// of  T_Configurator.prototype.SetUserName

T_Configurator.prototype.DeleteUserName = function (HostName,Reason){
  return this.DeleteAttribute(HostName,'username',Reason);
}// of  T_Configurator.prototype.DeleteUserName

//**********************************************************
//                IPAddress                                *
//**********************************************************

T_Configurator.prototype.HasIPAddress = function (HostName,Reason){
  return this.HasAttribute(HostName,'ipaddress',Reason);
}// of  T_Configurator.prototype.HasIPAddress

T_Configurator.prototype.GetIPAddress = function (HostName,IPAddress,Reason){
  return this.GetAttributeValue(HostName,'ipaddress',IPAddress,Reason);
}// of  T_Configurator.prototype.GetIPAddress

T_Configurator.prototype.SetIPAddress = function (HostName,IPAddress,Reason){
  return this.SetAttribute(HostName,'ipaddress','IPAddress',IPAddress,Reason);
}// of  T_Configurator.prototype.SetIPAddress

T_Configurator.prototype.DeleteIPAddress = function (HostName,Reason){
  return this.DeleteAttribute(HostName,'ipaddress',Reason);
}// of  T_Configurator.prototype.DeleteIPAddress

//**********************************************************
//                Comment                                  *
//**********************************************************

T_Configurator.prototype.HasComment = function (HostName,Reason){
  return this.HasAttribute(HostName,'comment',Reason);
}// of  T_Configurator.prototype.HasComment

T_Configurator.prototype.GetComment = function (HostName,Comment,Reason){
  return this.GetAttributeValue(HostName,'comment',Comment,Reason);
}// of  T_Configurator.prototype.GetComment

T_Configurator.prototype.SetComment = function (HostName,Comment,Reason){
  return this.SetAttribute(HostName,'comment','Comment',Comment,Reason);
}// of  T_Configurator.prototype.SetComment

T_Configurator.prototype.DeleteComment = function (HostName,Reason){
  return this.DeleteAttribute(HostName,'comment',Reason);
}// of  T_Configurator.prototype.DeleteComment

//**********************************************************
//                  Colour                                 *
//**********************************************************

T_Configurator.prototype.HasColour = function (HostName,Reason){
  return this.HasAttribute(HostName,'colour',Reason);
}// of  T_Configurator.prototype.HasColour

T_Configurator.prototype.GetColour = function (HostName,Colour,Reason){
  return this.GetAttributeValue(HostName,'colour',Colour,Reason);
}// of  T_Configurator.prototype.GetColour

T_Configurator.prototype.SetColour = function (HostName,Colour,Reason){
  return this.SetAttribute(HostName,'colour','Colour',Colour,Reason);
}// of  T_Configurator.prototype.SetColour

T_Configurator.prototype.DeleteColour = function (HostName,Reason){
  return this.DeleteAttribute(HostName,'colour',Reason);
}// of  T_Configurator.prototype.DeleteColour

//**********************************************************
//                  FontFamily                             *
//**********************************************************

T_Configurator.prototype.HasFontFamily = function (HostName,Reason){
  return this.HasAttribute(HostName,'fontfamily',Reason);
}// of  T_Configurator.prototype.HasFontFamily

T_Configurator.prototype.GetFontFamily = function (HostName,FontFamily,Reason){
  return this.GetAttributeValue(HostName,'fontfamily',FontFamily,Reason);
}// of  T_Configurator.prototype.GetFontFamily

T_Configurator.prototype.SetFontFamily = function (HostName,FontFamily,Reason){
  return this.SetAttribute(HostName,'fontfamily','FontFamily',FontFamily,Reason);
}// of  T_Configurator.prototype.SetFontFamily

T_Configurator.prototype.DeleteFontFamily = function (HostName,Reason){
  return this.DeleteAttribute(HostName,'fontfamily',Reason);
}// of  T_Configurator.prototype.DeleteFontFamily

//**********************************************************
//                  TextSize                                 *
//**********************************************************

T_Configurator.prototype.HasTextSize = function (HostName,Reason){
  return this.HasAttribute(HostName,'textsize',Reason);
}// of  T_Configurator.prototype.HasTextSize

T_Configurator.prototype.GetTextSize = function (HostName,TextSize,Reason){
  return this.GetAttributeValue(HostName,'textsize',TextSize,Reason);
}// of  T_Configurator.prototype.GetTextSize

T_Configurator.prototype.SetTextSize = function (HostName,TextSize,Reason){
  return this.SetAttribute(HostName,'textsize','TextSize',TextSize,Reason);
}// of  T_Configurator.prototype.SetTextSize

T_Configurator.prototype.DeleteTextSize = function (HostName,Reason){
  return this.DeleteAttribute(HostName,'textsize',Reason);
}// of  T_Configurator.prototype.DeleteTextSize

//**********************************************************
//                  Connect                                 *
//**********************************************************

T_Configurator.prototype.HasConnect = function (HostName,Reason){
  return this.HasAttribute(HostName,'connect',Reason);
}// of  T_Configurator.prototype.HasConnect

T_Configurator.prototype.GetConnect = function (HostName,Connect,Reason){
  return this.GetAttributeValue(HostName,'connect',Connect,Reason);
}// of  T_Configurator.prototype.GetConnect

T_Configurator.prototype.SetConnect = function (HostName,Connect,Reason){
  return this.SetAttribute(HostName,'connect','Connect',Connect,Reason);
}// of  T_Configurator.prototype.SetConnect

T_Configurator.prototype.DeleteConnect = function (HostName,Reason){
  return this.DeleteAttribute(HostName,'connect',Reason);
}// of  T_Configurator.prototype.DeleteConnect

//**********************************************************
//                  Disconnect                                 *
//**********************************************************

T_Configurator.prototype.HasDisconnect = function (HostName,Reason){
  return this.HasAttribute(HostName,'disconnect',Reason);
}// of  T_Configurator.prototype.HasDisconnect

T_Configurator.prototype.GetDisconnect = function (HostName,Disconnect,Reason){
  return this.GetAttributeValue(HostName,'disconnect',Disconnect,Reason);
}// of  T_Configurator.prototype.GetDisconnect

T_Configurator.prototype.SetDisconnect = function (HostName,Disconnect,Reason){
  return this.SetAttribute(HostName,'disconnect','Disconnect',Disconnect,Reason);
}// of  T_Configurator.prototype.SetDisconnect

T_Configurator.prototype.DeleteDisconnect = function (HostName,Reason){
  return this.DeleteAttribute(HostName,'disconnect',Reason);
}// of  T_Configurator.prototype.DeleteDisconnect

//**********************************************************
//                  Connected                                 *
//**********************************************************

T_Configurator.prototype.HasConnected = function (HostName,Reason){
  return this.HasAttribute(HostName,'connected',Reason);
}// of  T_Configurator.prototype.HasConnected

T_Configurator.prototype.GetConnected = function (HostName,Connected,Reason){
  return this.GetAttributeValue(HostName,'connected',Connected,Reason);
}// of  T_Configurator.prototype.GetConnected

T_Configurator.prototype.SetConnected = function (HostName,Connected,Reason){
  return this.SetAttribute(HostName,'connected','Connected',Connected,Reason);
}// of  T_Configurator.prototype.SetConnected

T_Configurator.prototype.DeleteConnected = function (HostName,Reason){
  return this.DeleteAttribute(HostName,'connected',Reason);
}// of  T_Configurator.prototype.DeleteConnected

//**********************************************************
//                  AutoConnect                            *
//**********************************************************

T_Configurator.prototype.HasAutoConnect = function (HostName,Reason){
  return this.HasAttribute(HostName,'autoconnect',Reason);
}// of  T_Configurator.prototype.HasAutoConnect

T_Configurator.prototype.GetAutoConnect = function (HostName,AutoConnect,Reason){
  var AttValue=new T_ByReference();
  if (!this.GetAttributeValue(HostName,'autoconnect',AttValue,Reason))
    return false;
  AutoConnect.Value=(AttValue.Value == T_Configurator.ItsSet);
  return true;
}// of  T_Configurator.prototype.GetAutoConnect

T_Configurator.prototype.SetAutoConnect = function (HostName,AutoConnect,Reason){
  var AttValue = (AutoConnect ? T_Configurator.ItsSet : T_Configurator.ItsReset);
  return this.SetAttribute(HostName,'autoconnect','AutoConnect',AttValue,Reason);
}// of  T_Configurator.prototype.SetAutoConnect

T_Configurator.prototype.DeleteAutoConnect = function (HostName,Reason){
  return this.DeleteAttribute(HostName,'autoconnect',Reason);
}// of  T_Configurator.prototype.DeleteAutoConnect

//**********************************************************
//                  SelectAll                              *
//**********************************************************

T_Configurator.prototype.HasSelectAll = function (HostName,Reason){
  return this.HasAttribute(HostName,'selectall',Reason);
}// of  T_Configurator.prototype.HasSelectAll

T_Configurator.prototype.GetSelectAll = function (HostName,SelectAll,Reason){
  var AttValue=new T_ByReference();
  if (!this.GetAttributeValue(HostName,'selectall',AttValue,Reason))
    return false;
  SelectAll.Value=(AttValue.Value == T_Configurator.ItsSet);
  return true;
}// of  T_Configurator.prototype.GetSelectAll

T_Configurator.prototype.SetSelectAll = function (HostName,SelectAll,Reason){
  var AttValue = (SelectAll ? T_Configurator.ItsSet : T_Configurator.ItsReset);
  return this.SetAttribute(HostName,'selectall','SelectAll',AttValue,Reason);
}// of  T_Configurator.prototype.SetSelectAll

T_Configurator.prototype.DeleteSelectAll = function (HostName,Reason){
  return this.DeleteAttribute(HostName,'selectall',Reason);
}// of  T_Configurator.prototype.DeleteSelectAll

//**********************************************************
//                  TypeMask                                 *
//**********************************************************

T_Configurator.prototype.HasTypeMask = function (HostName,Reason){
  return this.HasAttribute(HostName,'typemask',Reason);
}// of  T_Configurator.prototype.HasTypeMask

T_Configurator.prototype.GetTypeMask = function (HostName,TypeMask,Reason){
  return this.GetAttributeValue(HostName,'typemask',TypeMask,Reason);
}// of  T_Configurator.prototype.GetTypeMask

T_Configurator.prototype.SetTypeMask = function (HostName,TypeMask,Reason){
  return this.SetAttribute(HostName,'typemask','TypeMask',TypeMask,Reason);
}// of  T_Configurator.prototype.SetTypeMask

T_Configurator.prototype.DeleteTypeMask = function (HostName,Reason){
  return this.DeleteAttribute(HostName,'typemask',Reason);
}// of  T_Configurator.prototype.DeleteTypeMask

//**********************************************************
//                  HoursToKeep                            *
//**********************************************************

T_Configurator.prototype.HasHoursToKeep = function (HostName,Reason){
  return this.HasAttribute(HostName,'hourstokeep',Reason);
}// of  T_Configurator.prototype.HasHoursToKeep

T_Configurator.prototype.GetHoursToKeep = function (HostName,HoursToKeep,Reason){
  return this.GetAttributeValue(HostName,'hourstokeep',HoursToKeep,Reason);
}// of  T_Configurator.prototype.GetHoursToKeep

T_Configurator.prototype.SetHoursToKeep = function (HostName,HoursToKeep,Reason){
  return this.SetAttribute(HostName,'hourstokeep','HoursToKeep',HoursToKeep,Reason);
}// of  T_Configurator.prototype.SetHoursToKeep

T_Configurator.prototype.DeleteHoursToKeep = function (HostName,Reason){
  return this.DeleteAttribute(HostName,'hourstokeep',Reason);
}// of  T_Configurator.prototype.DeleteHoursToKeep

//**********************************************************
//                  StyleSheet                               *
//**********************************************************

T_Configurator.prototype.StyleSheetPath = function (HostName){
  //  Return the T_Path to the StyleSheet for this Host
  return this.BuildDataPath(T_HotLineConstant.StyleSheetFileName(HostName));
}// of  T_Configurator.prototype.StyleSheetPath

T_Configurator.prototype.StyleSheetURI = function (HostName){
  //  Return the T_Path to the StyleSheet for this Host
  return this.BuildDataPath(T_HotLineConstant.StyleSheetFileName(HostName)).ToURI();
}// of  T_Configurator.prototype.StyleSheetURI

T_Configurator.prototype.CreateStyleSheet = function (HostName,Colour,FontFamily,TextSize,Reason){
  //  This function creates a StyleSheet for the given HostName,
  //  using the attribute values provided.
  //  If the StyleSheet is created, the function returns true,
  //  otherwise false and areason.
  //  If there is already a StyleSheet resident, it is overwritten.
  var Style;
  var StyleSheet = new T_File(this.StyleSheetPath(HostName));
  if (!StyleSheet.Open(T_File.ReadAndWrite,true,Reason))
    return false;
  //  Construct the Style
  var RRGGBB=Colour.match(/^#([0-9A-F]*)/);
  if (RRGGBB != null)
    var Inverse='#' + InvertColour (RRGGBB[1])
  else
    var Inverse='#FFFFFF'; 
  Style='treechildren::-moz-tree-cell-text(' + HostName + '){\n' +
        'color : ' + Colour + ';\n'+
        'font-size : ' + TextSize + 'pt;\n'+
        'font-family : ' + FontFamily + ';\n'+
        '}\n' +

        //  HotLine Messages ListBox

       'listitem[value=' + HostName + ']{\n' +
        'color : ' + Colour + ';\n'+
        'font-size : ' + TextSize + 'pt;\n'+
        'font-family : serif;\n'+
        '}\n' +

        //  Host Class

        '.' + T_HotLineConstant.HostClass(HostName) + '{\n' +
        'color : ' + Colour + ';\n'+
        'font-size : ' + TextSize + 'pt;\n'+
        'font-family : ' + FontFamily + ';\n'+
        '}\n' +

        //  Host Details Button Class

        '.' + T_HotLineConstant.DetailsButtonClass(HostName) + '{\n' +
        'color : ' + Colour + ';\n'+
        'font-size : ' + (TextSize-2) + 'pt;\n'+
        'font-family : ' + FontFamily + ';\n'+
        '}\n' +

        //  Host Attribute Names Class (Details, Acknowledgement...)

        '.' + T_HotLineConstant.AttributeNameClass(HostName) + '{\n' +
        'font-size : ' + TextSize + 'pt;\n'+
        'font-family : ' + FontFamily + ';\n'+
        '}\n' +

        '.' + T_HotLineConstant.StatusRichListItemClass(HostName) + '{\n' +
        'border : 1px solid ' + Colour + ';\n' + 
        '}\n' +

        '.' + T_HotLineConstant.StaleDescriptionClass(HostName) + '{\n' +
        'background : ' + Colour + ';\n' + 
        'color : ' + Inverse + ';\n' + 
        'font-size : ' + TextSize + 'pt;\n'+
        'font-family : ' + FontFamily + ';\n'+
        '}\n' +

        '.' + T_HotLineConstant.StaleRichListItemClass(HostName) + '{\n' +
        'border : 2px solid black;\n' + 
        '}\n';
  if (!StyleSheet.Write(Style,Reason))
    return false;
  if (!StyleSheet.Close(Reason))
    return false;
  return true;
} // of T_Configurator.prototype.CreateStyleSheet

//**********************************************************
//                  LogFile                                *
//**********************************************************

T_Configurator.prototype.LogFilePath = function (HostName){
  //  Return the T_Path to the Log for this Host
  return this.BuildDataPath(T_HotLineConstant.LogFileName(HostName));
}// of  T_Configurator.prototype.LogFilePath

//*************************************************************
//              TypeNames and Filters                         *
// The RDF configuration file has a Seq container for the     *
// TypeName resources being used by each host. The TypeName   *
// resources are specific to a host since different hosts     *
// may use the same name with a different record number.      *
// Whenever the TypeNames are received from a host,           *
// they are merged with the information in the                *
// configuration file.                                        *
//*************************************************************

T_Configurator.prototype.TypeNamesURI = function (HostName){
  return this.RDF.SubjectURI(HostName + '/typenames');
} // of T_Configurator.prototype.TypeNamesURI

T_Configurator.prototype.TypeNameURI = function (HostName,TypeName){
  return this.RDF.SubjectURI(HostName + '/typename/' + TypeName.toLowerCase());
} // of T_Configurator.prototype.TypeNameURI

T_Configurator.prototype.ChangeTypeNames = function (HostName,ElementName,Rebuild){
  //  Change the ref attribute to the TypeNames for this Host
  var ElementRef=document.getElementById(ElementName);
  ElementRef.ref=this.TypeNamesURI(HostName);
  if (Rebuild)
    ElementRef.builder.rebuild();
}// of T_Configurator.prototype.ChangeTypeNames

T_Configurator.prototype.MergeTypeNames = function (HostName,TypeNames,Reason){
  //  The TypeNames are received as a <name>=<value> comma
  //  separated list.
  //  For Example:  META_MIN=1,META_MAX=2
  //  Extract each name with its value and check if it 
  //  already exists in the configuration file.
  function HandleTypeNameAndValue(TypeName,TypeValue){
    if (this.Verbose)
      this.Display(HostName,TypeName + ' = ' + TypeValue);
    if (!this.StoreTypeNameAndValue(HostName,TypeName,TypeValue,Reason))
      this.Display(HostName,Reason.Text);
  }// of HandleTypeNameAndValue
  if (this.Verbose)
    this.Display(HostName,'Merging ' + HostName + ' TypeNames: ' + TypeNames);
  if (!this.ParseTypeNameAndValueList(this,TypeNames,HandleTypeNameAndValue,Reason))
    return false;
  if (this.Verbose)
    this.Display(HostName,HostName + ' Merge Completed');
  return true;
}// of T_Configurator.prototype.MergeTypeNames

T_Configurator.prototype.SetTypeNames = function (HostName,Reason){
  return this.SetAttribute(HostName,'typenames','TypeNames','List',Reason);
}// of  T_Configurator.prototype.SetTypeNames

T_Configurator.prototype.StoreTypeNameAndValue = function (HostName,TypeName,Value,Reason){
  //  Store the TypeName and Value in the TypeNames Seq
  //  Container for the Host. If the Seq Container does
  //  not exist, it is created and added as an attribute
  //  to the Host.

  //  Create the TypeNames Container for the Host if Not Present
  var TypeNamesList=new T_RDFContainer(this.RDF,this.TypeNamesURI(HostName));
  if (!(TypeNamesList.IsContainer(Reason) ? TypeNamesList.Open(Reason) : TypeNamesList.Create(Reason)))
    return false;
  //  Check that there is a TypeNames Resource for the Host
  if (!this.SetTypeNames(HostName,Reason))
    return false;

  //  Create a TypeName Resource
  var TypeNameURI=this.TypeNameURI(HostName,TypeName);
  var TypeNameResource=new T_RDFDescription(this.RDF);
  if (!TypeNameResource.Initialize(TypeNameURI,Reason))
    return false;
  //  Check whether this TypeName already Exists
  if (TypeNamesList.Contains(TypeNameResource)){
    Reason.Text=TypeName + ' Already Exists';
    return false;
  };

  //  Create an AttName Attribute for the TypeName
  var AttName=new T_RDFAttribute(this.RDF,this.AttName_Predicate_URI);
  AttName.SetToLiteral('TypeName');
  TypeNameResource.Insert(AttName);
  //  Create an AttValue Attribute for the Mnemonic
  var AttValue=new T_RDFAttribute(this.RDF,this.AttValue_Predicate_URI);
  AttValue.SetToLiteral(TypeName);
  TypeNameResource.Insert(AttValue);
  //  Create a MnemValue Attribute for the Mnemonic Value
  var MnemValue=new T_RDFAttribute(this.RDF,this.MnemValue_Predicate_URI);
  MnemValue.SetToLiteral(Value);
  TypeNameResource.Insert(MnemValue);
  //  Insert the TypeName Resource into the TypeNamesList Container
  TypeNamesList.Insert(TypeNameResource);
  //  Save the Configuration File
  this.Flush();

  return true;
} // of T_Configurator.prototype.StoreTypeNameAndValue

T_Configurator.prototype.DeleteTypeName = function (HostName,TypeName,Reason){
  //  Delete the TypeName from the TypeNames Seq
  //  Container for the Host. If the Seq Container does
  //  not exist, then it has already been deleted.
  var TypeNamesList=new T_RDFContainer(this.RDF,this.TypeNamesURI(HostName));
  if (!TypeNamesList.IsContainer(Reason))
    return true;
  //  Open the TypeNames Container for the Host
  if (!TypeNamesList.Open(Reason))
    return false;
  //  Create a TypeName Resource
  var TypeNameURI=this.TypeNameURI(HostName,TypeName);
  TypeNameResource=new T_RDFDescription(this.RDF);
  if (!TypeNameResource.Initialize(TypeNameURI,Reason))
    return false;
  //  Remove the TypeName from the Container
  TypeNamesList.Remove(TypeNameResource);
  //  Delete the REsource
  if (!TypeNameResource.Delete(Reason))
    return false;
  //  Save the Configuration File
  this.Flush();
  return true;
} // of T_Configurator.prototype.DeleteTypeName

T_Configurator.prototype.ItsAFilter = function (TypeNameURI,FilterPredicateURI,TypeName,MnemValue,Reason){
  //  Returns true if the TypeNameURI refers to the Filter specified by the
  //  FilterPredicateURI, otherwise false and a Reason. If it is a Filter,
  //  the TypeName (T_ByReference) and the MnemValue (T_ByReference) 
  //  are returned.
  var Key = new T_ByReference();
  if (!this.RDF.GetLiteral(TypeNameURI,
                           FilterPredicateURI,
                           Key,
                           Reason))
    return false;
  if (!this.RDF.GetLiteral(TypeNameURI,
                           this.AttValue_Predicate_URI,
                           TypeName,
                           Reason))
    return false;
  if (!this.RDF.GetLiteral(TypeNameURI,
                           this.MnemValue_Predicate_URI,
                           MnemValue,
                           Reason))
    return false;
  return true;
} // of T_Configurator.prototype.ItsAFilter

T_Configurator.prototype.MarkAsRecordFilter = function (HostName,TypeName,Reason){
  //  Insert a recordfilter="yes" attribute 
  if (!this.RDF.SetLiteral(this.TypeNameURI(HostName,TypeName),
                           this.RecordFilterPredicateURI,
                           "RecordFilter",
                            Reason))
    return false;
  return true;
} // of T_Configurator.prototype.MarkAsRecordFilter

T_Configurator.prototype.UnMarkAsRecordFilter = function (HostName,TypeName,Reason){
  //  Delete the recordfilter attribute
  if (!this.RDF.DeleteLiteral(this.TypeNameURI(HostName,TypeName),
                              this.RecordFilterPredicateURI,
                              Reason))
    return false;
  return true;
} // of T_Configurator.prototype.UnMarkAsRecordFilter

T_Configurator.prototype.MarkAsAlertFilter = function (HostName,TypeName,Reason){
  //  Insert an alertfilter="yes" attribute 
  if (!this.RDF.SetLiteral(this.TypeNameURI(HostName,TypeName),
                           this.AlertFilterPredicateURI,
                           "AlertFilter",
                           Reason))
    return false;
  return true;
} // of T_Configurator.prototype.MarkAsAlertFilter

T_Configurator.prototype.UnMarkAsAlertFilter = function (HostName,TypeName,Reason){
  //  Delete the alertfilter attribute
  if (!this.RDF.DeleteLiteral(this.TypeNameURI(HostName,TypeName),
                               this.AlertFilterPredicateURI,
                               Reason))
    return false;
  return true;
} // of T_Configurator.prototype.UnMarkAsAlertFilter

T_Configurator.prototype.MarkAsStatusFilter = function (HostName,TypeName,Reason){
  //  Insert a statusfilter="yes" attribute 
  if (!this.RDF.SetLiteral(this.TypeNameURI(HostName,TypeName),
                           this.StatusFilterPredicateURI,
                           "StatusFilter",
                           Reason))
    return false;
  return true;
} // of T_Configurator.prototype.MarkAsStatusFilter

T_Configurator.prototype.UnMarkAsStatusFilter = function (HostName,TypeName,Reason){
  //  Delete the statusfilter attribute
  if (!this.RDF.DeleteLiteral(this.TypeNameURI(HostName,TypeName),
                               this.StatusFilterPredicateURI,
                               Reason))
    return false;
  return true;
} // of T_Configurator.prototype.UnMarkAsStatusFilter

T_Configurator.prototype.BuildFilter = function (HostName,FilterName,Table,Reason){
  //  This function builds the Table of entries for the given
  //  FilterName and HostName. If any error occurs, the function
  //  returns false and a Reason, otherwise true.
  //  The Table is by reference and is indexed by the ClassId.
  //  Each entry contains either null, or the TypeName associated
  //  with the ClassId.
  var Index;
  var TypeName = new T_ByReference();
  var MnemValue = new T_ByReference();
  var FilterPredicateURI = this.RDF.PredicateURI(FilterName);

  //  Create the TypeNames Container for the Host if Not Present
  var TypeNamesList=new T_RDFContainer(this.RDF,this.TypeNamesURI(HostName));
  if (!(TypeNamesList.IsContainer(Reason) ? TypeNamesList.Open(Reason) : TypeNamesList.Create(Reason)))
    return false;
  //  Build the URIList of TypeName Entries
  var URIList=TypeNamesList.GetURIList(Reason);
  if (URIList==null)
    return false;
  for(var j=0;j<URIList.length;j++)
    //  Check for the HotLine NameSpace
    if ((Index=URIList[j].search(T_HotLineConstant.RDFNameSpace + '/')) >= 0)
      if (this.ItsAFilter(URIList[j],FilterPredicateURI,TypeName,MnemValue,Reason)){
        var k=parseInt(MnemValue.Value);
        if (k == 'NaN')
          this.Display(HostName,'Invalid Mnemonic Value. ' +
                                this.RDF.Triple(URIList[j],FilterPredicateURI,MnemValue.Value))
        else
          Table[k]=TypeName.Value;
      };
  return true;
} // of T_Configurator.prototype.BuildFilter

T_Configurator.prototype.BuildTypeNamesTable = function (HostName,Table,Reason){
  //  This function builds a Table of all TypeNames for
  //  the HostName.If any error occurs, the function
  //  returns false and a Reason, otherwise true.
  //  The Table is by reference and is indexed by the ClassId.
  //  Each entry contains either null, or the TypeName associated
  //  with the ClassId.
  var Index;
  var TypeName = new T_ByReference();
  var MnemValue = new T_ByReference();

  //  Create the TypeNames Container for the Host if Not Present
  var TypeNamesList=new T_RDFContainer(this.RDF,this.TypeNamesURI(HostName));
  if (!(TypeNamesList.IsContainer(Reason) ? TypeNamesList.Open(Reason) : TypeNamesList.Create(Reason)))
    return false;
  //  Build the URIList of TypeName Entries
  var URIList=TypeNamesList.GetURIList(Reason);
  if (URIList==null)
    return false;
  for(var j=0;j<URIList.length;j++)
    //  Check for the HotLine NameSpace
    if ((Index=URIList[j].search(T_HotLineConstant.RDFNameSpace + '/')) >= 0){
      if (this.RDF.GetLiteral(URIList[j],
                              this.AttValue_Predicate_URI,
                              TypeName,
                              Reason) &&
           this.RDF.GetLiteral(URIList[j],
                               this.MnemValue_Predicate_URI,
                               MnemValue,
                               Reason)){
        var k=parseInt(MnemValue.Value);
        if (k == 'NaN')
          this.Display(HostName,'Invalid Mnemonic Value. ' +
                                this.RDF.Triple(URIList[j],this.MnemValue_Predicate_URI,MnemValue.Value))
        else
          Table[k]=TypeName.Value;
      };
    };
  return true;
} // of T_Configurator.prototype.BuildTypeNamesTable

T_Configurator.prototype.DeleteTypeNames = function (HostName,Reason){
  //  This function deletes all the TypeNames in the
  //  RDF Container for the HostName.
  var Index;
  var TypeNamesList=new T_RDFContainer(this.RDF,this.TypeNamesURI(HostName));
  if (!TypeNamesList.IsContainer(Reason))
     return true;
  if (!TypeNamesList.Open(Reason))
     return false;
  //  Build the URIList of TypeName Entries
  var URIList=TypeNamesList.GetURIList(Reason);
  if (URIList==null)
    return false;
  for(var j=0;j<URIList.length;j++)
    //  Check for the HotLine NameSpace
    if ((Index=URIList[j].search(T_HotLineConstant.RDFNameSpace + '/')) >= 0){
      var Resource=new T_RDFDescription(this.RDF);
      if (!Resource.Initialize(URIList[j],Reason))
         return false;
      if (!Resource.Delete(Reason))
         return false;
    };
  if (!TypeNamesList.Destroy(Reason))
    return false;
  return true;
} // of T_Configurator.prototype.DeleteTypeNames

T_Configurator.prototype.DeleteAttributes = function (HostName,Reason){
  //  This function deletes all the Attributes in the
  //  RDF Container for the HostName, and the Container.
  var Index;
  var AttributeList=new T_RDFContainer(this.RDF,this.RDF.SubjectURI(HostName));
  if (!AttributeList.IsContainer(Reason))
     return true;
  if (!AttributeList.Open(Reason))
     return false;
  //  Build the URIList of Attribute Entries
  var URIList=AttributeList.GetURIList(Reason);
  if (URIList==null)
    return false;
  for(var j=0;j<URIList.length;j++)
    //  Check for the HotLine NameSpace
    if ((Index=URIList[j].search(T_HotLineConstant.RDFNameSpace + '/')) >= 0){
      var Resource=new T_RDFDescription(this.RDF);
      if (!Resource.Initialize(URIList[j],Reason))
         return false;
      if (!Resource.Delete(Reason))
         return false;
    };
  if (!AttributeList.Destroy(Reason))
    return false;
  return true;
} // of T_Configurator.prototype.DeleteAttributes


//  T_Configurator Class Items

var Configurator = new T_Configurator();

