/***********************************************************
 *                   CSSSupport                            *
 * This file contains JavaScript types and functions for   *
 * use with CSS files.                                     *
 *                                                         *
 *                                                         *
 * Uses                                                    *
 *   filemanager.js                                        *
 *                                                         *
 *                      History                            *
 * 11/04/2006  Initial Version                             *
 * 23/05/2006  HotLine Version 4.0                         *
 ***********************************************************/

//**********************************************************
//                T_StyleSheet                             *
//  This type represents a StyleSheet.                     *
//**********************************************************

//  T_StyleSheet Class and Properties

function T_StyleSheet(Agent){
  this.StyleSheetService = Components.classes["@mozilla.org/content/style-sheet-service;1"].
                                                 getService(Components.interfaces.nsIStyleSheetService);
  if (Agent)
    this.Type=this.StyleSheetService.AGENT_SHEET;
  else
    this.Type=this.StyleSheetService.USER_SHEET;
}// of T_StyleSheet

//  T_StyleSheet Methods

T_StyleSheet.prototype.Initialize = function(Path,Reason){
  //  The Path is a T_Path object which references
  //  the StyleSheet File.
  //  Example:
  //    StyleSheet.Initialize(new T_ExtensionDirectory(
  //                                 "hotline@www.metalogic.eu.com",
  //                                 "content",
  //                                 HostName + '.css'));
  //  If any error occurs, the function returns FALSE and
  //  a Reason, otherwise TRUE.
  this.Path=Path;
  this.StyleSheet=new T_File(Path);
  return true;
} // of T_StyleSheet.prototype.Initialize
 
T_StyleSheet.prototype.Load = function(Reason){
  //  This function returns TRUE if the StyleSheet has been
  //  loaded, or was already loaded, otherwise it 
  //  returns FALSE and a Reason.
  if (this.StyleSheetService.sheetRegistered(this.StyleSheet.Path.To_nsIURI(),
                                             this.Type))
    return true;
  this.StyleSheetService.loadAndRegisterSheet(this.StyleSheet.Path.To_nsIURI(),
                                              this.Type);
  return true;
} // of T_StyleSheet.prototype.Load
 
T_StyleSheet.prototype.LoadChrome = function(Chrome,Reason){
  //  This function returns TRUE if the StyleSheet has been
  //  loaded, or was already loaded, otherwise it 
  //  returns FALSE and a Reason.
  var ioservice = Components.classes["@mozilla.org/network/io-service;1"]
                                          .getService(Components.interfaces.nsIIOService);
  var uri=ioservice.newURI(Chrome,null,null);

  /*if (this.StyleSheetService.sheetRegistered(uri,
                                             this.Type))
{alert('regsietered');
    return true; }; */
  this.StyleSheetService.loadAndRegisterSheet(uri,
                                              this.Type);
  return true;
} // of T_StyleSheet.prototype.Load
 
T_StyleSheet.prototype.UnLoad = function(Reason){
  //  This function returns TRUE if the StyleSheet has been
  //  unloaded, or was already unloaded, otherwise it 
  //  returns FALSE and a Reason.
  if (!this.StyleSheetService.sheetRegistered(this.StyleSheet.Path.To_nsIURI(),
                                              this.Type))
    return true;
  this.StyleSheetService.unRegisterSheet(this.StyleSheet.Path.To_nsIURI(),
                                         this.Type);
  return true;
} // of T_StyleSheet.prototype.UnLoad

T_StyleSheet.prototype.Loaded = function(Reason){
  //  This function returns TRUE if the StyleSheet has been
  //  loaded, otherwise it returns FALSE and a Reason.
  if (this.StyleSheetService.sheetRegistered(this.StyleSheet.Path.To_nsIURI(),
                                             this.Type))
    return true;
  Reason.Text='StyleSheet is Not Loaded';
  return false;
} // of T_StyleSheet.prototype.Loaded
 
//**********************************************************
//        T_StyleSheet Class Functions                     *
//  These class functions manage the StyleSheet processing *
//  instructions in the DOM for the document. This should  *
//  not be confused with the StyleSheet array which        *
//  only contains those style sheets which have actually   *
//  been loaded. These functions use a Chrome reference    *
//  for the style sheet file, such as                      *
//   var ChromeReference = 'chrome://hotline/skin/fred.css'*
//  These functions are used instead of the StyleSheet     *
//  service above which is not working for hotline.        *
//**********************************************************

T_StyleSheet.FindDOMStyleSheetNode = function (ChromeReference,CaseSensitive){
  //  Searches the Child Nodes of the Document for 
  //  a StyleSheet node which has a matching target 
  //  attribute. If found, the function returns the
  //  Node, otherwise it returns null.
  var Children = document.childNodes;
  var Pattern = new RegExp(ChromeReference,(CaseSensitive ? '' : 'i'));
  for (var j=0;j<Children.length;j++){
    var Node=Children[j];
    if ((Node.nodeType == Node.PROCESSING_INSTRUCTION_NODE) && (Node.target == "xml-stylesheet"))
      if (Node.data.search(Pattern) >= 0)
         return Node;
  };
  return null;
}// of T_StyleSheet.FindDOMStyleSheetNode

T_StyleSheet.FindDOMStyleSheet = function (ChromeReference,CaseSensitive){
  //  Searches the Child Nodes of the Document for 
  //  a StyleSheet node which has a matching target 
  //  attribute. If found, the function returns true
  //  otherwise false.
  return (T_StyleSheet.FindDOMStyleSheetNode(ChromeReference,CaseSensitive) != null);
}// of T_StyleSheet.FindDOMStyleSheet


T_StyleSheet.AddDOMStyleSheet = function (ChromeReference,CaseSensitive,Reason){
  //  This function adds a StyleSheet processing instruction
  //  with the given ChromeReference to the DOM if it is not already
  //  defined. If it is defined or it is added, the function returns
  //  true otherwise false and a reason.
  if (T_StyleSheet.FindDOMStyleSheet(ChromeReference,CaseSensitive,Reason)) 
    return true;
  var StyleSheet=document.createProcessingInstruction("xml-stylesheet",
                                                       "href='" + ChromeReference + "' type='text/css'");
  document.insertBefore(StyleSheet, document.documentElement); 
  return true;
}// of T_StyleSheet.AddDOMStyleSheet

T_StyleSheet.DeleteDOMStyleSheet = function (ChromeReference,CaseSensitive,Reason){
  //  This function searches for a StyleSheet processing instruction
  //  with the given ChromeReference in the DOM and if found, it is
  //  deleted. If it was not defined or was deleted, the function 
  //  returns true otherwise false and a reason.
  var Node=T_StyleSheet.FindDOMStyleSheetNode(ChromeReference,CaseSensitive);
  if (Node == null)
    return true; 
  Node.parentNode.removeChild(Node);  // i.e. document.removeChild(Node)
  return true;
}// of T_StyleSheet.DeleteDOMStyleSheet

T_StyleSheet.RebuildDOMStyleSheets = function (Reason){
  //  Forces the Skins to be reloaded, including the FireFox skins.
  //  If any error occurs, the function returns false and a Reason,
  //  otherwise true.
  
  //  Note:  Disabled because it has unpredictable side effects
  return true;
  
  var ChromeRegistry = Components.classes["@mozilla.org/chrome/chrome-registry;1"].
                                               getService(Components.interfaces.nsIXULChromeRegistry);
  ChromeRegistry.refreshSkins();
  return true;
}// of T_StyleSheet.RebuildDOMStyleSheets

T_StyleSheet.GetDOMStyleSheetNodes = function (){
  //  This function returns an array of Nodes. Each Node represents
  //  a StyleSheet processing instruction.
  var List = new Array();
  var Children = document.childNodes;
  for (var j=0;j<Children.length;j++){
    var Node=Children[j];
    if ((Node.nodeType == Node.PROCESSING_INSTRUCTION_NODE) && (Node.target == "xml-stylesheet"))
      List.push(Node);
  };
  return List;
}// of T_StyleSheet.GetDOMStyleSheetNodes

T_StyleSheet.GetDOMStyleSheetData = function (){
  //  This function returns an array of Strings. Each string contains
  //  the data value for a StyleSheet processing instruction.
  var Nodes = T_StyleSheet.GetDOMStyleSheetNodes();
  var Data = new Array(Nodes.length);
  for (var j=0;j<Nodes.length;j++)
    Data[j]=Nodes[j].data;
  return Data;
}// of T_StyleSheet.GetDOMStyleSheetData


