/***********************************************************
 *                   DetailSupport                         *
 * This file contains JavaScript types and functions which *
 * support the Details button created when Recorder sends  *
 * <?xul>.                                                 *
 *                                                         *
 * The details dialog contains a box called 'Details-Box'  *
 * which is the container for the OPAL generated content.  *
 *                                                         *
 * OPAL can generate <box> elements which contain a        *
 * script='<script name> attribute and a data='<data name> *
 * attribute.                                              *
 *                                                         *
 * The variable called <data name> is defined in a         *
 * <script> tag and must be in the format expected by the  *
 * associated script.                                      *
 *                                                         *
 * These scripts have been defined:                        *
 *   System - Data is an Object.                           *
 *   Mix    - Data is an array of Objects.                 *
 *                                                         *
 * <script>var Entries=[{Name:'A/B on C',RSVP:'OK,DS'},    *
 *                      {Name:'*X',RSVP:'OK'}];</script>   *
 *                                                         *
 * Formatting Rules:                                       *
 *  1. If the attribute name begins with an underscore     *
 *     the attribute name and colon are suppressed.        *
 *     The attribute value is styled with the              *
 *     host class.                                         *
 *                                                         *
 *  2. If the attribute value is null, the colon and       *
 *     the attribute value are suppressed. The attribute   *
 *     value is styled with the same font as the Host,     *
 *     but not the colour.                                 *
 *                                                         *
 *  3. An underscore or hyphen in an attribute name        *
 *     is always replaced by a space.                      *
 *                                                         *
 *                                                         *
 *                                                         *
 * Note:  In the first version, the OPAL called an Execute *
 *        function which specified an overlay xul to       *
 *        provide the content. Do not preclude the use     *
 *        of an overlay when enhancing this module.        *
 *                                                         *
 *                      History                            *
 * 04/06/2006  Initial Version                             *
 * 09/06/2006  Add ShowSource                              *
 * 13/06/2006  Remove suppressOnSelect                     *
 * 14/06/2006  Add null rule for details                   *
 * 15/06/2006  Use ReplaceCDATA to avoid xml parser errors *
 * 04/06/2006  Implement Pie Charts using PlotKit          *
 * 05/06/2006  Transform OPAL Data to PlotKit form         *
 * 06/06/2006  Implement Plot for direct PlotKit access    *
 * 20/04/2011  Remove Remote XUL for FireFox 4             *
 * 06/09/2011  Check there are ScriptTags before use       *
 *             Use openDialog to get window.arguments      *
 ***********************************************************/

function OnLoad(){
  //  This function is called when the onload event for the
  //  details window happens. It is referenced by the onload
  //  attribute of the <dialog> in SelectDetails of HotLine.js.
  this.HostName=window.arguments[0];
  this.RecorderData=window.arguments[1];
  //  Counter for creating unique box tag
  this.Counter=1;
  
  //  Load the StyleSheet for the Host
//var StyleSheet = Components.classes["@mozilla.org/content/style-sheet-service;1"]
//                            .getService(Components.interfaces.nsIStyleSheetService);
//var IOS = Components.classes["@mozilla.org/network/io-service;1"] .getService(Components.interfaces.nsIIOService);
//var URI = IOS.newURI("chrome://hotline/skin/"+HostName+'.style', null, null);

//if(!StyleSheet.sheetRegistered(URI, StyleSheet.USER_SHEET)) {
   // StyleSheet.loadAndRegisterSheet(URI, StyleSheet.USER_SHEET);
//}
  
  var XUL='<?xml version="1.0"?>'+
           '<window xmlns="http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul" ' +
                   'xmlns:html="http://www.w3.org/1999/xhtml"><vbox>'+  
          this.RecorderData.XUL()+
          "</vbox></window>";
  //  Check the markup with the DOM Parser
  var Parser=new DOMParser();
  var Document=Parser.parseFromString(XUL,"text/xml");
  var RootTag = Document.documentElement;
  var DetailsBox=document.getElementById('Details-Box');
  
  var Fragment = document.importNode(RootTag.firstChild,true);
  DetailsBox.appendChild(Fragment);

  //  Check each box in the 'Details-Box' and evaluate any
  //  'script' attribute.
  var Box=document.getElementById('Details-Box');
  if (Box != null){
    BuildBoxes(Box);
  };
  //*-----------------------< 
  window.sizeToContent();
//  var w=document.getElementById('Details-Window');
//  var c=w.className;
//  w.className="force-reflow";
//  w.className=c;
}// of OnLoad    

function BuildBox(Box,Tag){
  var Script=Box.getAttribute('script');
  var Data=Box.getAttribute('data');
  //  Must eval the script since Remote XUL has been disabled.
  //  Assume there is always one and only one script tag per box
  var ScriptTags=Box.getElementsByTagName('script');
  if (ScriptTags.length==0){
    return;
  };
  var JavaScript=ScriptTags[0];
  var Code=JavaScript.textContent;
  JavaScript.parentNode.removeChild(JavaScript);
  eval(Code);
  //  Data contains the name of the Array created by the eval of Code
  Box.setAttribute('tag',Tag);
  eval(Script + '("' + Tag + '",' + Data + ')');
}// of BuildBox

function BuildBoxes(Details){
  var ChildNodes=Details.childNodes;
  for (var j=0;j<ChildNodes.length;j++){
    Element=ChildNodes[j];
    if (Element.nodeType == Element.ELEMENT_NODE)
      if (Element.hasAttribute('script')){
        BuildBox(Element,'box'+this.Counter++);
      }
      else
        BuildBoxes(Element)
  };
}// of BuildBoxes

function AttributeBox(AttName,AttValue){
  var Box=document.createElement('hbox');

  var Element=document.createElement('description');
  Element.setAttribute('class',T_HotLineConstant.AttributeNameClass(this.HostName));
  Element.setAttribute('value',AttName.replace(/[-_]/g,' '));
  Box.appendChild(Element);

  var Element=document.createElement('description');
  Element.setAttribute('class',T_HotLineConstant.AttributeNameClass(this.HostName));
  Element.setAttribute('value',':');
  Box.appendChild(Element);

  var Element=document.createElement('description');
  Element.setAttribute('class',T_HotLineConstant.HostClass(this.HostName));
  Element.setAttribute('value',AttValue);
  Box.appendChild(Element);

  return Box;
}// of AttributeBox

function TextBox(Text){
  var Box=document.createElement('hbox');
  var Element=document.createElement('description');
  Element.setAttribute('class',T_HotLineConstant.HostClass(this.HostName));
  Element.setAttribute('value',Text);
  Box.appendChild(Element);

  return Box;
}// of TextBox

function Heading(Text){
  var Box=document.createElement('hbox');

  var Element=document.createElement('description');
  Element.setAttribute('class',T_HotLineConstant.AttributeNameClass(this.HostName));
  Element.setAttribute('value',Text);
  Box.appendChild(Element);

  return Box;
}// of Heading


function System(Tag,Entry){
  //  Tag is the uniqueid box<n> attached to 
  //  the box which requires this function.
  //  Entry is an Object

  //  Get a reference to the Box
  var Boxes=document.getElementsByAttribute('tag',Tag);
  if (Boxes.length <= 0)
    return;
  var Box=Boxes[0];

  Box.orient='vertical'
  Box.flex='1';
  var RichListBox=document.createElement('richlistbox');
  RichListBox.flex=1;
  Box.appendChild(RichListBox);
  //  Create Label Elements
  var RichListItem=document.createElement('richlistitem');
  RichListItem.setAttribute('class',
                              T_HotLineConstant.StatusRichListItemClass(this.HostName));
  RichListBox.appendChild(RichListItem);

  var ItemBox=document.createElement('vbox');
  ItemBox.flex=1;
  RichListItem.appendChild(ItemBox);
  //  Build the Items
  for (var k in Entry)
    if (k[0]=='_')
      ItemBox.appendChild(TextBox(Entry[k]))
    else
      if (Entry[k] == null)
        ItemBox.appendChild(Heading(k))
      else
        ItemBox.appendChild(AttributeBox(k,Entry[k]));
  RichListBox.selectedIndex=-1;
  //  Setting suppressOnSelect caused an error 
  //  on Windows 2000 FireFox 1.5.0.4
  //  RichListBox.suppressOnSelect=true;
}// of System

function Mix(Tag,Entries){
  //  Tag is the uniqueid box<n> attached to 
  //  the box which requires this function.
  //  Entries is an array of Mix Objects

  //  Get a reference to the Box which
  var Boxes=document.getElementsByAttribute('tag',Tag);
  if (Boxes.length <= 0)
    return;
  var Box=Boxes[0];
  Box.orient='vertical'
  Box.flex='1';
  var RichListBox=document.createElement('richlistbox');
  RichListBox.flex=1;
  Box.appendChild(RichListBox);
  //  Create Label Elements
  for (var j=0;j<Entries.length;j++){
    //  Mix Entry Entries[j]
    var RichListItem=document.createElement('richlistitem');
    RichListItem.setAttribute('class',
                               T_HotLineConstant.StatusRichListItemClass(this.HostName));
    RichListBox.appendChild(RichListItem);

    var ItemBox=document.createElement('vbox');
    ItemBox.flex=1;
    RichListItem.appendChild(ItemBox);
    //  Build the Items
    for (var k in Entries[j]){
      //  Attribute Entries[j][k] 
     if (k[0]=='_')
       ItemBox.appendChild(TextBox(Entries[j][k]))
     else
       if (Entries[j][k] == null)
         ItemBox.appendChild(Heading(k))
       else
         ItemBox.appendChild(AttributeBox(k,Entries[j][k]));
    };
  }
  RichListBox.selectedIndex=-1;
  //  Setting suppressOnSelect caused an error 
  //  on Windows 2000 FireFox 1.5.0.4
  //  RichListBox.suppressOnSelect=true;
}// of Mix

/***************************************************************
 *               Simple Plot Kit Scripts (Pie,Bar,Line)        *
 ***************************************************************/

function BuildPlotKitData(Data,Options,DataSets){
  //  Data is a 2 dimensional array built in OPAL such as
  //  var Data=[["*",23,32,55],["OPS",45,30,25]];
  //  Each row contains the Label for the row followed by
  //  the data for each dataset. The x coordinate is the ordinal
  //  position of the row.
  //  This function transforms the Data into 2 structures, which
  //  can be passed to PlotKit. 
  //  The first element of each row is used to create the 
  //  xTicks array of Labels which are used for the x coordinates.
  //  The xTicks option and any other options associated with the
  //  Data are used to create the Options object.
  //  DataSets is an array of DataSet objects. A DataSet object 
  //  is an array of (x,y) coordinates and a name property, which can
  //  be used to set the parameters of the Layout addDataSet method.

  //  Get any Options specified
  if (Data.Options != undefined){
    for (var Option in Data.Options){
       Options[Option]=Data.Options[Option];
    };
  };

  //  Build the Labels for the xTicks property
  var Labels = new Array();
  var nRows=Data.length;
  for (var j=0;j<nRows;j++){
    var Label=new Object();
    Label['v']=j;
    Label['label']=Data[j][0];
    Labels.push(Label);
  };
  Options.xTicks=Labels;
  //  Build the DataSets. The first element determines the number
  //  of DataSets
  var nDataSets=Data[0].length-1;
  for (var j=1;j<=nDataSets;j++){
    var DataSet=new Array();
    DataSet['name']='data'+j;
    DataSets.push(DataSet);
    for (var k=0;k<nRows;k++){
      var Point=new Array(2);
      Point[0]=k;
      Point[1]=Data[k][j];
      DataSet.push(Point);
    };
  };
}// of BuildPlotKitData

function FormatPlotKitData(Options,DataSets){
  var Labels = Options.xTicks;
  var Text='';
  for (var j in DataSets){
    Text+='"' + DataSets[j].name + '"';
    for (var k=0;k<DataSets[j].length;k++){
       var Point=DataSets[j][k];
       if ((Labels==undefined)||(Point[0]>=Labels.length))
         Text+='[' + Point[0] + ',' + Point[1] + ']'
       else
         Text+='[' + Labels[Point[0]].label + ',' + Point[1] + ']';
    };
    Text+='\n';
  };
  for (var Option in Options)
    if (Option != 'xTicks')
      Text+=Option + ':' + Options[Option] + '\n';
  return Text;
}// of FormatPlotKitData

function Pie(Tag,Data){
  DoPlotKit('pie',Tag,Data);
}// of Pie

function Bar(Tag,Data){
  DoPlotKit('bar',Tag,Data);
}// of Bar

function Line(Tag,Data){
  DoPlotKit('line',Tag,Data);
}// of Line

function DoPlotKit(Which,Tag,Data){
  //  Additional Options:
  //    SVG : <boolean>  if set, then the rendering is done
  //                     using SVG rather than Canvas.
  //    Debug : <boolean> if set, an alert showing the
  //                      Options and DataSets from Data
  //                      is displayed.
  var Options = new Object();
  var DataSets = new Array();
  BuildPlotKitData(Data,Options,DataSets);
  if ((Options.Debug != undefined) && Options.Debug)
    alert(Which + ':\n' +FormatPlotKitData(Options,DataSets));
  if ((Options.SVG != undefined) && Options.SVG)
    DoPlotKitSVG(Which,Tag,Options,DataSets)
  else
    DoPlotKitCanvas(Which,Tag,Options,DataSets);
}// of DoPlotKit

/***************************************************************
 *            Native Plot Kit (Plot)                           *
 ***************************************************************/

function Plot(Tag,Data){
  //  This script provides complete access to the PlotKit objects
  //  var Data={DataSets:[{Name:'sqrt',
  //                       Points:[[0,0],[4,2]]},
  //                      {Name:'sqre',
  //                       Points:[[0,0],[4,16]]}],
  //            Options:{drawXAxis:true},   // as defined by PlotKit
  //            Style:'pie',
  //            Debug:true,
  //            SVG : false};

  //  Create a Reference to the Options
  if (Data.Options==undefined)
    var Options=new Object()
  else
    var Options = Data.Options;
  //  Get the Style
  if (Data.Style==undefined)
    var Style='pie'
  else
    var Style=Trim(Data.Style.toLowerCase());
  //  Build the DataSets
  var DataSets = new Array();
  for (var j=0;j < Data.DataSets.length;j++){
    var DataSet = Data.DataSets[j].Points.slice();
    DataSet['name']=Data.DataSets[j].Name;
    DataSets.push(DataSet);
  };
  if ((Data.Debug != undefined) && Data.Debug)
    alert(Style + '(SVG ' + Data.SVG + '):\n' + FormatPlotKitData(Options,DataSets));
  if ((Data.SVG != undefined) && Data.SVG)
    DoPlotKitSVG(Style,Tag,Options,DataSets)
  else
    DoPlotKitCanvas(Style,Tag,Options,DataSets);
} // of Plot

function DoPlotKitCanvas(Which,Tag,Options,DataSets){
  //  Whih is the style of chart
  //  Tag is the uniqueid box<n> attached to 
  //  the box which requires this function.
  //  Options is an Object defined by PlotKit.
  //  DataSets are the plotting points.

  //  Get a reference to the Box
  var Boxes=document.getElementsByAttribute('tag',Tag);
  if (Boxes.length <= 0)
    return;
  var Box=Boxes[0];

  Box.orient='vertical'
  Box.flex='1';
  //  Create html:div Element and append it to the Box
  var Div = document.createElementNS('http://www.w3.org/1999/xhtml', 'div');
  Box.appendChild(Div);
  //  Create html:canvas Element and append it to html:div
  var Canvas = document.createElementNS('http://www.w3.org/1999/xhtml', 'canvas');
  Div.appendChild(Canvas);
  Canvas.setAttribute('id','graph');
  Canvas.setAttribute('height','300');
  Canvas.setAttribute('width','300');
  Canvas.setAttribute('flex','300');
  //  Draw the Graph
  var Layout= new PlotKit.Layout(Which,Options);
  for (var j in DataSets)
    Layout.addDataset(DataSets[j].name,DataSets[j]);
  Layout.evaluate();
  var Plotter = new PlotKit.SweetCanvasRenderer(Canvas,Layout,Options);
  Plotter.render();
}// of DoPlotKitCanvas

function DoPlotKitSVG(Which,Tag,Options,DataSets){
  //  Tag is the uniqueid box<n> attached to 
  //  the box which requires this function.
  //  Options is an Object defined by PlotKit.
  //  DataSets are the plotting points.

  //  Get a reference to the Box
  var Boxes=document.getElementsByAttribute('tag',Tag);
  if (Boxes.length <= 0)
    return;
  var Box=Boxes[0];

  Box.orient='vertical'
  Box.flex='1';
  //  Create html:div Element and append it to the Box
  var Div = document.createElementNS('http://www.w3.org/1999/xhtml', 'div');
  Box.appendChild(Div);
  //  Create svg:svg Element and append it to html:div
  var SVG = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
  Div.appendChild(SVG);
  SVG.setAttribute('id','graph');
  SVG.setAttribute('height','300');
  SVG.setAttribute('width','300');
  SVG.setAttribute('flex','300');
  //  Draw the Graph
  var Layout= new PlotKit.Layout(Which,Options);
  for (var j in DataSets)
    Layout.addDataset(DataSets[j].name,DataSets[j]);
  Layout.evaluate();
  var Plotter = new PlotKit.SweetSVGRenderer(SVG,Layout,Options);
  Plotter.render();
}// of DoPlotKitSVG

function OnLoadSourceDialog(){
  //  This function is called when the onload event for the
  //  source dialog happens. It is referenced by the onload
  //  attribute of the <dialog> in ShowSource of DetailSupport.js
  window.sizeToContent();
  this.HostName=window.arguments[0];
  this.RecorderData=window.arguments[1];
}// of OnLoadSourceDialog    

function ShowSource(){
  //  Ensure that the XUL does not have any CDATA sections otherwise
  //  the error message xml will be invalid
  var Text=ReplaceCDATA(RecorderData.XUL());
  var XUL='<?xml version="1.0"?>' + NL +
          '<?xml-stylesheet href="chrome://global/skin/" type="text/css"?>' + NL +
          '<?xml-stylesheet href="' + 
                T_HotLineConstant.ChromeSkin('hotline.css') + '" type="text/css"?>' + NL +
          '<?xml-stylesheet href="' + 
                T_HotLineConstant.ChromeSkin(T_HotLineConstant.StyleSheetFileName(HostName)) + 
                                                         '" type="text/css"?>' +
          '<!DOCTYPE window SYSTEM "' + T_HotLineConstant.ChromeLocale('hotline.dtd') + '">' +
          '<dialog id="Source-Dialog"' +
                 ' xmlns="http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul"' +
                 ' buttons="accept"' +
                 ' onload="OnLoadSourceDialog();"' +
                 ' title="&Source-Dialog-Title;"' +
                 ' context="Source-PopUp"' +
                 ' flex="1">'+
            '<script src="' + T_HotLineConstant.ChromeContent('detailsupport.js') + '"/>' +
            '<script src="' + T_HotLineConstant.ChromeContent('generalsupport.js') + '"/>' +
            '<script src="' + T_HotLineConstant.ChromeContent('hotlineconstants.js') + '"/>' +
            '<popupset>' +
               '<popup id="Source-PopUp" flex="1">' +
                  '<menuitem id="Source-PopUp-CopyToClipBoard"' +
                             ' oncommand="CopySource();"' +            
                             ' label="&CopySource-PopUp-Label;"/>' +
               '</popup>' +
             '</popupset>' +
            '<groupbox id="Source-Box" flex="1">' +
               '<caption id="Source-Caption" label="&Source-Caption-Label;"/>' +
               '<description class="' + T_HotLineConstant.HostClass(HostName) + '">' + 
                   '<![CDATA[' + Text + ']]>' +
               '</description>' +
            '</groupbox>' +
          '</dialog>'; 
  var Dialog=window.openDialog('data:application/vnd.mozilla.xul+xml,' + encodeURIComponent(XUL),
                         '',
                         'chrome,modal,dialog,resizable',
                         this.HostName,
                         this.RecorderData);
}// of ShowSource

function CopySource(){
  CopyToClipBoard(RecorderData.XUL());
}// of CopySource



