/***********************************************************
 *                GeneralSupport                           *
 * This file contains JavaScript types and functions which *
 * which are common to all scripts.                        *
 *                                                         *
 * This Script does not require any other script.          *
 *                                                         *
 *                      History                            *
 * 16/02/2006  Initial Version                             *
 * 15/05/2006  Add FormattedYYYYMMDDHHMMSSNNNToDate        *
 * 23/05/2006  HotLine Version 4.0                         *
 * 07/06/2006  Add RoundUp                                 *
 * 09/06/2006  Add HexToNumber,ConvertRRGGBBToNumber       *
 *             Add InvertColour                            *
 * 15/06/2006  Add ReplaceCDATA                            *
 ***********************************************************/

const NL = '\n';

function DIV(X,Y) {
    return Math.floor(X / Y);
}// of DIV (Ref J.R. Stockton)

function RoundUp(Val,Divisor) {
  return DIV(((Val)+(Divisor)-1),Divisor);
}// of RoundUp

function MustBeString(who,v){
  if (typeof v != "string")
    throw who + ':' + v + ' is a ' + typeof v + ',not a string';
  return 1;
}// of MustBeString

function MustBeNumber(who,v){
  if (typeof v != "number")
    throw who + ':' + v + ' is a ' + typeof v + ',not a number';
  return 1;
}// of MustBeNumber

function HexDigit(i){
  var HEXDIGITS='0123456789ABCDEF';
  //  Return the Hex representation of the Number in the range 0..15
  return HEXDIGITS[i];
}// of HexDigit

function HexString(AsciiString){
  var b;
  var s='';
  //  Return the Hex representation of the ASCII string
  MustBeString('HexString',AsciiString);
  for (var i=0;i<AsciiString.length;i++){
    b=AsciiString.charCodeAt(i) & 0xFF;
    s+=HexDigit(b >> 4) + HexDigit(b & 0xF);
  }
  return s;
}// of HexString

function Int48ToString(Number){
  //  Convert the number x to a 6 byte Integer48 in Network Order
  return String.fromCharCode(0x00) + String.fromCharCode(0x00) + Int32ToString(Number);
}// of Int48ToString

function Int32ToString(Number){
  var j,b = 0;
  var s = '';
  //  Convert the number x to a 4 byte Integer32 in Network Order
  MustBeNumber('Int32ToString',Number);
  for (var j=0;j<4;j++){
    b=(Number >> ((3-j)*8))&0xFF;
    s+=String.fromCharCode(b);
  }
  return s;
}// of Int32ToString

function Int16ToString(Number){
  //  Convert the number x to a 2 byte Integer16 in Network Order
  MustBeNumber('Int16ToString',Number);
  return String.fromCharCode((Number >> 8) & 0xFF,Number & 0xFF);
}// of Int16ToString

function ByteToString(b){
  //  Convert the Byte b to a 1 byte Integer8
  return String.fromCharCode(b);
}// of ByteToString

function BooleanToString(b){
  //  Convert the Boolean b to a 1 byte Integer8
  if (b)
    return String.fromCharCode(0x01)
  else
    return String.fromCharCode(0x00);
}// of BooleanToString

function SwapBytes4(Number){
  var j,b,v=0;
  //  Treat the Number as an Integer32 and swap the bytes
  MustBeNumber('SwapBytes4',Number);
  for (var j=0;j<4;j++){
    b=(Number >> ((3-j)*8))&0xFF;
    v+=(b << (j*8));
  }
  return v;
}// of SwapBytes4

function Empty(Source){
  //  Return TRUE if the String is Empty or Spaces
  return (Trim(Source).length==0);
}// of Empty

function Trim(Text){
  //  Remove leading and trailing spaces
  //  Reference: Regular Expressions, J Friedl p199
  if (typeof Text == 'string')
    return (Text.replace(/^\s+/,'')).replace(/\s+$/,'')
  else
    return (Text + '(Trim ' + typeof Text + ' ?)');
}// of Trim

function Now(){
  var TimeOfDay=new Date();
  return TimeOfDay.getHours()+':'+TimeOfDay.getMinutes()+':'+TimeOfDay.getSeconds();
} //  of Now

function YYYYMMDDHHMMSSNNN(DateObject){
  // Reference: J.R.Stockton(Merlyn)
  // Date used for sorting returned as String 
 with (DateObject)
   return String((((((getFullYear() * 100 + getMonth() + 1) * 100 + getDate()) * 100 +
                                getHours()) * 100 + getMinutes()) * 100 + getSeconds()) * 1000 + getMilliseconds());
}// of YYYYMMDDHHMMSSNNN

function FormatYYYYMMDDHHMMSSNNN(YYYYMMDDHHMMSSNNN){
 // Retain sort order of String YYYYMMDDHHMMSS
 //   YYYY/MM/DD HH:MM:SS:NNN
   return  YYYYMMDDHHMMSSNNN.slice(0,4)+'/'+YYYYMMDDHHMMSSNNN.slice(4,6)+'/'+YYYYMMDDHHMMSSNNN.slice(6,8)+' '+
            YYYYMMDDHHMMSSNNN.slice(8,10)+':'+YYYYMMDDHHMMSSNNN.slice(10,12)+':'+YYYYMMDDHHMMSSNNN.slice(12,14) + ':' + 
            YYYYMMDDHHMMSSNNN.slice(14,17)
}// of FormatYYYYMMDDHHMMSS

function FormattedYYYYMMDDHHMMSSNNNToDate(FormattedYYYYMMDDHHMMSSNNN){
 //   YYYY/MM/DD HH:MM:SS:NNN
  return new Date(FormattedYYYYMMDDHHMMSSNNN.slice(0,4),     //  YYYY
                    FormattedYYYYMMDDHHMMSSNNN.slice(5,7)-1,   //  MM
                    FormattedYYYYMMDDHHMMSSNNN.slice(8,10),    //  DD
                    FormattedYYYYMMDDHHMMSSNNN.slice(11,13),   //  HH
                    FormattedYYYYMMDDHHMMSSNNN.slice(14,16),   //  MM
                    FormattedYYYYMMDDHHMMSSNNN.slice(17,19),   //  SS
                    FormattedYYYYMMDDHHMMSSNNN.slice(20,23));  //  NNN
}// of FormatYYYYMMDDHHMMSS

function HexToNumber(H){
  //  Character H
  switch (H[0]) {
 case '0':
 case '1':
 case '2':
 case '3':
 case '4':
 case '5':
 case '6':
 case '7':
 case '8': 
 case '9':
   return H.charCodeAt(0)-0x30;
 case 'A':
 case 'B':
 case 'C':
 case 'D':
 case 'E':
 case 'F':
   return H.charCodeAt(0)-0x37;
 };
}// of HexToNumber

function ConvertRRGGBBToNumber(RRGGBB){
  //  String RRGGBB Hex Digits
  var T=0;
  for (var j=0;j<6;j++){
    T+=(HexToNumber(RRGGBB[j]) << ((5-j)*4));
  };
  return T;
}// of ConvertRRGGBBToNumber

function InvertColour(RRGGBB){
  //  String RRGGBB Hex Digits (0000FF=>FFFF00)
  return HexString(Int32ToString(0xFFFFFF-ConvertRRGGBBToNumber(RRGGBB))).slice(2,8);
}// of ConvertRRGGBBToNumber

function CopyToClipBoard(Text){
  // String Text
  var ClipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
                                         .getService(Components.interfaces.nsIClipboardHelper);
  ClipboardHelper.copyString(Text);
}// of CopyToClipBoard

function ReplaceCDATA(xml){
  var Text=xml.replace(/<!\[CDATA\[/,"[CDATA]");
  return Text.replace(/\]\]>/,"[/CDATA]"); 
}// of ReplaceCDATA

//**********************************************************
//                T_Result                                 *
//**********************************************************

//  T_Result Properties

function T_Result(Result,Reason){
  this.Result=Result;
  this.Reason=Reason;
}// of T_Result

//  T_Result Class Items

function Error(Reason){
  return new T_Result(false,Reason);
}// of Error

function OK(){
  return new T_Result(true,null);
}// of OK

//**********************************************************
//                T_Reason                                 *
//**********************************************************

//  T_Reason Properties

function T_Reason(){
  this.Text='';
}// of T_Reason

//**********************************************************
//                T_ByReference                            *
//**********************************************************

function T_ByReference(){
  this.Value=0;
}// of T_ByReference;



