/***********************************************************
 *                   HotLine                               *
 * This file contains the main HotLine application.        *
 *                                                         *
 * Uses                                                    *
 *   generalsupport.js                                     *
 *   mcpsupport.js                                         *
 *                                                         *
 *                      History                            *
 * 20/02/2006  Initial Version                             *
 * 16/05/2006  Style Alerts and Status with Host Style     *
 *             Add a hint to Alerts and Status entries     *
 *             Make the selected <treecols> persistent     *
 *             Add Open and Close Configurator function    *
 *             Delete Status using Tag and HostName        *
 * 23/05/2006  HotLine Version 4.0                         *
 *             Implement HoursToKeep attribute             *
 * 24/05/2006  Age Alerts                                  *
 *             Discard expired Recorder Messages           *
 * 30/05/2006  Implement Save and Restore Configuration    *
 *             Add textNode to RichListItem description    *
 * 31/05/2006  TextNode prevents fixed formatting of Status*
 *             Implement RefreshTypeNames                  *
 * 02/06/2006  Implement Details button for <?xul>         *
 * 06/06/2006  Remove XUL from Acknowledgement             *
 *             Implemented detailsupport.js Version 4.4    *
 * 07/06/2006  Implemented Heartbeat                       *
 * 09/06/2006  Add parameter Stale to DoAlert and DoStatus *
 * 11/06/2006  Display Parse Error and Location            *
 * 12/06/2006  Flush RecorderFile during ShutDown          *
 *             Set dom.max_script_run_time to 20 seconds   *
 *               in FireFox about:config to avoid          *
 *               unresponsive script warnings.             *
 * 13/06/2006  Check if HotLineOpen when onload event      *
 * 15/06/2006  Use ReplaceCDATA to avoid xml parser errors *
 * 16/06/2006  Log Recorder message at Connect and ShutDown*
 * 19/06/2006  Add Details to Recorder Tree                *
 *             Auto Configure Filters for new host         *
 * 20/06/2006  Replace Details Button with Information Icon*
 * 04/07/2006  Graph Details with PlotKit                  *
 * 09/10/2009  Convert to new Drag and Drop Events         *
 ***********************************************************/

var Reason = new T_Reason();

//**********************************************************
//                T_Status                                 *
//  This type is used to manage the Status Panel.          *
//  The Display function adds text to the Status ListBox.  *
//**********************************************************

//  T_Status Properties
function T_Status(){
  this.status_listbox=null;
  this.Today=new Date(0);
}// of T_Status

//  T_Status Methods
T_Status.prototype.Initialize = function (Reason){
  //  The document object is not defined until the
  //  window has been loaded. The Status variable
  //  is created before the window has been loaded,
  //  so the status_listbox reference cannot be 
  //  created until the onload event has happened.
  this.status_listbox=document.getElementById('status_listbox');
  return true;
}// of T_Status.prototype.Initialize

T_Status.prototype.Display = function (HostName,Text){
  var Now=new Date();
  var StatusListBox=this.status_listbox;
  var General=Empty(HostName);
  if (General)
    HostName='Unknown';
  if (Empty(Text))
    Text='<empty text>';
  if (Now.getDate() != this.Today.getDate()){
    StatusListBox.appendItem('--------- ' + Now.toLocaleString() + ' ---------');
    this.Today=Now;
  };
  var Item=StatusListBox.appendItem(Now.getHours()+':'+Now.getMinutes()+':'+Now.getSeconds() +
                                    ' '+Text + (General?'':'(' + HostName + ')'),HostName);
  Item.tooltipText=Text;
  //  This does not display properly until
  //  there are scroll bars, even when the
  //  function is only called with RowCount >
  //  VisibleRowCount.
  //  
  //  if (StatusListBox.getRowCount() > StatusListBox.getNumberOfVisibleRows())
  //   this.status_listbox.ensureElementIsVisible(Item);
}// of T_Status.prototype.Display

T_Status.prototype.Trace = function (Text){
  this.status_listbox.appendItem(Text,0);
}// of T_Status.prototype.Trace

T_Status.prototype.Clear = function (Reason){
  var StatusListBox=this.status_listbox;
  var Count=StatusListBox.getRowCount();
  for (var j=Count-1;j>=0;j--)
    StatusListBox.removeItemAt(j);
  StatusListBox.appendItem('--------- ' + (new Date()).toLocaleString() + ' ---------');
  return true;
}// of T_Status.prototype.Clear


//  T_Status Class Items

var Status = new T_Status();

//**********************************************************
//                T_HotLine                                *
//  This type represents an invocation of HotLine.         *
//  It is created when the onload event of the HotLine     *
//  window happens.                                        *
//  onload="var HotLine=new T_HotLine();                   *
//          HotLine.Initialize();"                         *
//**********************************************************

//  T_HotLine Properties
function T_HotLine(){
  this.DragService = Components.classes["@mozilla.org/widget/dragservice;1"]
                             .getService(Components.interfaces.nsIDragService);
}// of T_HotLine

//  T_Hotline Methods 
T_HotLine.prototype.Initialize = function (Status,Configurator,Reason){
  var HostNames;
  var UserName = new T_ByReference();
  var IPAddress = new T_ByReference();
  var Result;
  //  Status is used for reporting things about HotLine.
  this.Status=Status;
  //  The Configurator holds information about the configured
  //  Hosts. It is an RDF DataSource. Although it is global,
  //  it is provided as a parameter to dependent objects,
  //  such as T_HotLineHost.
  this.Configurator=Configurator;
  //  Rebuild the HostNames MenuList until using RDF
  //  Templates with MenuLists doesn't cause FireFox
  //  to crash.
  if (!this.RebuildHostNamesMenuList('',Reason))
    this.Display('',Reason.Text);
  //  Setup the default Configurator State
  var Box=document.getElementById('HostConfiguration-VBox');
  this.UseCollapsed=true;
  if (this.UseCollapsed){
    Box.hidden=false;
    this.ChangeConfiguratorButton(Box.collapsed);
  }
  else{
    Box.collapsed=false;
    this.ChangeConfiguratorButton(Box.hidden);
  };

  //  The HostList is an associative array indexed by HostName.
  this.HostList=new Array();
  //  Get the Configured Hosts and Check whether they should
  //  be AutoConnected
  HostNames=this.Configurator.GetHosts(Reason);
  if (HostNames == null){
    this.Display('',Reason.Text);
    this.DoChangeConfiguratorBox(true);
    this.DoHelpButton(false);
    this.OnCommandEnterConfigurator();
  }
  else{
    for (var j=0;j<HostNames.length;j++){
      //  Check for AutoConnect
      var AutoConnect=new T_ByReference(); 
      if (this.Configurator.GetAutoConnect(HostNames[j],AutoConnect,Reason) && AutoConnect.Value)
        if (!this.DoAutoConnect(HostNames[j],Reason))
          this.Display(HostNames[j],Reason.Text);
    };
    this.InitializeHelpButton();
  };
  //  Start Timers
  window.setInterval('HotLine.OneMinuteBurden()',60*1000);
  window.setInterval('HotLine.OneHourBurden()',(60*60*1000));

  //  The suppressonselect attribute does not seem to
  //  be working, but changing the attribute here does.
  var RichListBox=document.getElementById('Status-RichListBox');
  RichListBox.setAttribute('suppressOnSelect','true');
  var RichListBox=document.getElementById('Alert-RichListBox');
  RichListBox.setAttribute('suppressOnSelect','true');

  return true;
}// of T_HotLine.prototype.Initialize

T_HotLine.prototype.ShutDown = function (Reason){
  var HostName;
  var Why=new T_Reason();
  Reason.Text='';
  //  ShutDown the Hosts
  for (HostName in this.HostList){
    if (!this.HostList[HostName].ShutDown(Why))
      Reason.Text+=HostName + ':' + Why.Text + '. ';
  };
  return (Reason.Text.length == 0);
}// of T_HotLine.prototype.ShutDown

T_HotLine.prototype.OneMinuteBurden = function (){
  var AutoConnect=new T_ByReference(); 
  var Reason=new T_Reason();
  var IPAddress=new T_ByReference();

  for (var HostName in this.HostList){
    if (this.HostList[HostName].ItsConnected){
      if (!this.AgeAlertRichListItems(Reason))
        this.Display(HostName,Reason.Text);
      if (!this.AgeStatusRichListItems(Reason))
        this.Display(HostName,Reason.Text);
      if (!this.HostList[HostName].OneMinuteBurden(Reason))
        this.Display(HostName,Reason.Text);
    }
    else
      if (this.Configurator.GetAutoConnect(HostName,AutoConnect,Reason) && AutoConnect.Value){
        this.Display(HostName,'AutoConnecting ' + HostName);
        if (!this.DoAutoConnect(HostName,Reason))
          this.Display(HostName,Reason.Text);
      }; 
  };
}// of T_HotLine.prototype.OneMinuteBurden

T_HotLine.prototype.OneHourBurden = function (){
  var Reason=new T_Reason();
  for (HostName in this.HostList){
    if (this.HostList[HostName].ItsConnected){
      //  Discard EXpired Recorder Messages
      if (!this.DoCleanUp(HostName,Reason))
        this.Display(HostName,Reason.Text);
    };// if
  };// for
}// of T_HotLine.prototype.OneHourBurden

T_HotLine.prototype.Display = function (HostName,Text){
  this.Status.Display(HostName,Text);
}// of T_HotLine.prototype.Display

T_HotLine.prototype.HideConfigurator = function (HostName,Hide){
  //  The Configurator is made visible when the Configure 
  //  Host button is pressed. After the changes, the 
  //  Configurator window is hidden, and the Configure
  //  Host button is made visible
  var ConfigureButton=document.getElementById('Configure-Button');
  var ConfigurationGroupBox=document.getElementById('Configure-GroupBox');
  ConfigurationGroupBox.collapsed=Hide;
  ConfigureButton.collapsed=!Hide;
  if (!Hide){
    //  Rebuild the HostNames MenuList 
    if (!this.RebuildHostNamesMenuList('',Reason))
      this.Display('',Reason.Text);
    if (Empty(HostName))
      HostName=this.GetSelectedHostName();
    if (Empty(HostName))
      this.HostNameChanged(HostName)
    else
      this.OnSelectHost(HostName);
  };
}// of T_HotLine.prototype.HideConfigurator

T_HotLine.prototype.OnCommandExitConfigurator = function (){
  this.HideConfigurator('',true);
}// of T_HotLine.prototype.OnCommandExitConfigurator

T_HotLine.prototype.OnCommandEnterConfigurator = function (){
  this.HideConfigurator(this.GetSelectedHostName(),false);
}// of T_HotLine.prototype.OnCommandEnterConfigurator

T_HotLine.prototype.ValidHostName = function (Name,HostName,Reason){
  //  Name to be validated returned in HostName T_ByREference
  if (Empty(Name)){
    Reason.Text='No Name';
    return false;
  };
  return this.Configurator.ValidHostName(Name,HostName,Reason);
}// of T_HotLine.prototype.ValidHostName

T_HotLine.prototype.GetHotLineHost = function (HostName,Always,Reason){
  var Host = this.HostList[HostName];
  //  Add it to the HostList if necessary
  if (Host==undefined){
   if (!Always){
     Reason.Text=HostName + ' has not been added to the HostList';
     return null;
   };
   Host=new T_HotLineHost(this);
   if (!Host.Initialize(HostName,'Recorder-Tree',Reason)){
     return null;
    
   };
   this.HostList[HostName]=Host;
  };
  return Host;
}// of T_HotLine.prototype.GetHotLineHost

T_HotLine.prototype.RebuildStyleSheets = function (Reason){
  //  This function should be called whenever 
  //  any StyleSheet changes have been made.
  var Element;
  if (!T_StyleSheet.RebuildDOMStyleSheets(Reason))
    return false;
  //  Rebuild the Trees after the StyleSheet Change
  this.RebuildRecorderTree();
  this.RebuildHostsTree();
  return true;
}// of T_HotLine.prototype.RebuildStyleSheets

//**********************************************************
//           Configuring HostName                          *
//**********************************************************

T_HotLine.prototype.HostNameChanged = function (HostName){
  // This function is called when the HostName has been 
  // or is to be changed.
  var Reason = new T_Reason();
  var NoHostName=Empty(HostName);
  //  Ensure that the HostNames_MenuList matches the
  //  given HostName
  var Element=document.getElementById('HostNames-MenuList');
  Element.label=HostName;
  //  Identification GroupBox
  Element=document.getElementById('Comment-Row');
  Element.collapsed=NoHostName;
  Element=document.getElementById('Advice-Row');
  Element.collapsed=!NoHostName;
  //  Connection GroupBox
  Element=document.getElementById('Connection-GroupBox');
  Element.collapsed=NoHostName;
  if (!NoHostName){
    //  Fill in the Comment
    Element=document.getElementById('Comment-TextBox');
    var Comment=new T_ByReference; 
    if (this.Configurator.GetComment(HostName,Comment,Reason))
      Element.value=Comment.Value;
    //  Fill in the UserName
    Element=document.getElementById('UserName-TextBox');
    var UserName=new T_ByReference; 
    if (!this.Configurator.GetUserName(HostName,UserName,Reason))
      Element.value=''
    else
      Element.value=UserName.Value;
    //  Fill in the IPAddress
    Element=document.getElementById('IPAddress-TextBox');
    var IPAddress=new T_ByReference; 
    if (!this.Configurator.GetIPAddress(HostName,IPAddress,Reason))
      Element.value='0.0.0.0'
    else
      Element.value=IPAddress.Value;
    //  Fill in the Colour
    Element=document.getElementById('Host-ColorPicker');
    var Colour=new T_ByReference; 
    if (!this.Configurator.GetColour(HostName,Colour,Reason))
      Element.color=T_HotLineConstant.DefaultColour
    else
      Element.color=Colour.Value;
    //  Fill in the FontFamily
    Element=document.getElementById('Host-FontFamily-MenuPopUp');
    var FontFamily=new T_ByReference; 
    if (!this.Configurator.GetFontFamily(HostName,FontFamily,Reason))
      FontFamily.Value=T_HotLineConstant.DefaultFontFamily;
    var MenuItems=Element.childNodes;
    for(var j=0;j<MenuItems.length;j++)
      if (MenuItems[j].label == FontFamily.Value){
        Element.parentNode.selectedIndex=j;
        break;
      };
    //  Fill in the TextSize
    Element=document.getElementById('Host-TextSize');
    var TextSize=new T_ByReference; 
    if (!this.Configurator.GetTextSize(HostName,TextSize,Reason))
      Element.value=T_HotLineConstant.DefaultTextSize
    else
      Element.value=TextSize.Value;
    //  Set the All CheckBox
    Element=document.getElementById('Recorder-Filter-All-CheckBox');
    var SelectAll=new T_ByReference(); 
    Element.checked=(this.Configurator.GetSelectAll(HostName,SelectAll,Reason) && SelectAll.Value);
    //  Change the TypeNames ref attribute
    this.ClearTypeNameAndValue();
    this.Configurator.ChangeTypeNames(HostName,'TT-REC-NAMES-ArrowScrollBox',true);
    this.Configurator.ChangeTypeNames(HostName,'Record-Filter-ArrowScrollBox',true);
    this.Configurator.ChangeTypeNames(HostName,'Alert-Filter-ArrowScrollBox',true);
    this.Configurator.ChangeTypeNames(HostName,'Status-Filter-ArrowScrollBox',true);
    //  Set the AutoConnect Attribute
    Element=document.getElementById('AutoConnect-CheckBox');
    var AutoConnect=new T_ByReference(); 
    if (this.Configurator.GetAutoConnect(HostName,AutoConnect,Reason))
      Element.checked=AutoConnect.Value
    else
      Element.checked=true;
    //  Fill in the HoursToKeep
    Element=document.getElementById('HoursToKeep-TextBox');
    var HoursToKeep=new T_ByReference; 
    if (!this.Configurator.GetHoursToKeep(HostName,HoursToKeep,Reason))
      Element.value=T_HotLineConstant.DefaultHoursToKeep
    else
      Element.value=HoursToKeep.Value;
  };
  //  Recorder Names GroupBox
  Element=document.getElementById('Recorder-Names-GroupBox');
  Element.collapsed=NoHostName;
  //  Presentation GroupBox
  Element=document.getElementById('Presentation-GroupBox');
  Element.collapsed=NoHostName;
  //  Options GroupBox
  Element=document.getElementById('Options-GroupBox');
  Element.collapsed=NoHostName;
  //  Delete Button
  Element=document.getElementById('Delete-Button');
  Element.collapsed=(NoHostName || (!this.Configurator.FindHost(HostName,Reason)));
  //  Exit Button
//  Element=document.getElementById('Exit-Button');
//  Element.label='Exit';
}// of T_HotLine.prototype.HostNameChanged

//**********************************************************
//             HostName Event Handlers                     *
//**********************************************************

T_HotLine.prototype.ShowFocused = function (who){
  var Focused=document.commandDispatcher.focusedElement;
  this.Display(who,Focused.tagName + ',' + Focused.getAttribute('id'));
}// of T_HotLine.prototype.ShowFocused

T_HotLine.prototype.OnSelectHost = function (Name){
  // This function is called by the oncommand event
  // in the Configurator form when a Name has been
  // selected. It is also called by OnChangeHost after
  // it has created a new Name. There may be missing
  // information, but fill in the attributes that are
  // available.

  var Host,Element;
  var HostName = new T_ByReference();
  var Reason = new T_Reason();
  //  Validate the HostName
  if (!this.ValidHostName(Name,HostName,Reason)){
    this.Display(Name,Reason.Text);
    return false;
  };

  Host = this.GetHotLineHost(HostName.Value,true,Reason);
  if (Host == null)
    return false;
  this.HostNameChanged(HostName.Value);
}// of T_HotLine.prototype.OnSelectHost

T_HotLine.prototype.OnChangeHost = function (Name){
  // This function is called by the onchange event
  // in the Configurator form when a HostName has been
  // entered. The event happens on leaving the TextBox. 
  var HostName = new T_ByReference();
  var Reason = new T_Reason();
  if (Empty(Name)){
    this.HostNameChanged('');
    return;
  };
  //  Validate the HostName
  if (!this.ValidHostName(Name,HostName,Reason)){
    //  Clear the TextBox
    var Element=document.getElementById('HostNames-MenuList');
    Element.label='';
    this.HostNameChanged(HostName.Value);
    //  Tell them why
    alert(Reason.Text);
    return;
  };
  //  Check if the HostName already exists.
  if (this.Configurator.FindSimilarHost(HostName.Value,HostName,Reason)){
    this.OnSelectHost(HostName.Value);
    return;
  };
  //  Create the Host in the Hosts Configuration
  if (!this.Configurator.CreateHost(HostName.Value,Reason)){
    this.Display('',Reason.Text);
    return;
  };
  //  A default StyleSheet is created now so that when the
  //  reference is added to the DOM, the file exists and
  //  is added to the StyleSheet array. If the file is not
  //  resident when the reference is added to the DOM it
  //  seems never to be used. These should be the same 
  //  defaults used in the Configure Host form.
  if (!this.ChangeStyle(HostName.Value,
                        T_HotLineConstant.DefaultColour,
                        T_HotLineConstant.DefaultFontFamily,
                        T_HotLineConstant.DefaultTextSize,
                        Reason)){
    this.Display(HostName.Value,Reason.Text);
    return;
  };
  //  Update the AutoConnect Attribute based on the CheckBox Value
  Element=document.getElementById('AutoConnect-CheckBox');
  if (!this.Configurator.SetAutoConnect(HostName.Value,Element.checked,Reason))
    this.Display(HostName.Value,Reason);

  this.Display(HostName.Value,
               'Created Host ' + HostName.Value + ' in '
                    + this.Configurator.ConfigurationFileName());
  //  Rebuild the HostNames MenuList 
  if (!this.RebuildHostNamesMenuList(HostName.Value,Reason))
    this.Display(HostName.Value,Reason.Text);
  //  Select the Host
  this.OnSelectHost(HostName.Value);
}// of T_HotLine.prototype.OnChangeHost

T_HotLine.prototype.OnInputHost = function (HostName){
  //  This function is called when the oninput event
  //  of the HostName TextBox happens. It adjusts
  //  the Configurator panel depending on the current
  //  HostName.
  this.HostNameChanged(HostName);
}// of T_HotLine.prototype.OnInputHost

//**********************************************************
//              HostName MenuList                          *
// The HostNames MenuList of the Configuration Screen was  *
// originally implemented using an Template which          *
// was rebuilt using the Host Configuration RDF.           *
// As it seemes to cause FireFox to crash on Windows       *
// 2000, it is now rebuilt in  code.                       *
//**********************************************************

/*  This code was used to find the Selected HostName when
    the Hostames-MenuList was built from an RDF Template 

T_HotLine.prototype.GetHostNameMenuList = function (){
  //  Gets a Reference to the the HostName MenuList Element.
  //  getElementById cannot be used because it returns the
  //  template menulist, not the RDF constructed menulist, which
  //  has an empty id attribute. The made up attribute called tag
  //  is set in the xul, and is applied to both menulists.
  var HostNamesBox=document.getElementById('HostNames-Box');
  var MenuLists=HostNamesBox.getElementsByAttribute('tag','HostName-MenuList');
  if (MenuLists.length == 0)
    return '';
  for (var j=0;j<MenuLists.length;j++){
     var MenuList=MenuLists[j];
     if (MenuList.id == '')
        break;
  };
  return MenuList;
}// of T_HotLine.prototype.GetHostNameMenuList
*/
T_HotLine.prototype.GetHostNameMenuList = function (){
  return document.getElementById('HostNames-MenuList');
}// of T_HotLine.prototype.GetHostNameMenuList

T_HotLine.prototype.GetSelectedHostName = function (){
  //  Retrieves the HostName selected in the HostNames MenuList.
  var MenuList=this.GetHostNameMenuList();
  return MenuList.label;
}// of T_HotLine.prototype.GetSelectedHostName

T_HotLine.prototype.SetSelectedHostName = function (HostName){
  //  Sets the HostName in the HostNames MenuList
  var MenuList=this.GetHostNameMenuList();
  MenuList.label=HostName;
  this.HostNameChanged(HostName);
}// of T_HotLine.prototype.SetSelectedHostName


T_HotLine.prototype.RebuildHostNamesMenuList = function (HostName,Reason){
  //  Rebuilds the HostNames-MenuList. If a HostName is
  //  provided the entry that contains the HostName is
  //  selected.
  var SelectHostName = (!Empty(HostName));
  //  Get a Reference to the HostNames-MenuList-PopUp
  var Element=document.getElementById('HostNames-MenuPopUp');
  //  Delete the MenuPopUp
  var MenuList=Element.parentNode;
  MenuList.removeChild(Element);
  //  Create a new MenuPopUp
  Element=document.createElement('menupopup');
  Element.setAttribute('id','HostNames-MenuPopUp');
  //  Insert the MenuPopUp into the MenuList
  MenuList.appendChild(Element);
  //  Get all the currently configured HostNames
  var HostNames=this.Configurator.GetHosts(Reason);
  if (HostNames == null){
    //  There are no HostNames configured
    return true;
  };
/*
  //  Delete all the current MenuItems
  var MenuItems=Element.childNodes;
  for (var j=0;j<MenuItems.length;j++)
    Element.removeChild(MenuItems[j]);
  var MenuItems=Element.childNodes;
  alert('after delete ' + MenuItems.length);

*/
  //  Create a MenuItem for each HostName
  for (var j=0;j<HostNames.length;j++){
    var MenuItem=document.createElement("menuitem");
    MenuItem.setAttribute('label',HostNames[j]);
    if (SelectHostName && (HostName == HostNames[j]))
      MenuItem.setAttribute('selected','true');
    Element.appendChild(MenuItem);
  };
  return true;
}// of  T_HotLine.prototype.RebuildHostNamesMenuList

T_HotLine.prototype.DoRebuildRecorderTree = function (event){
  //  Called from the Context Menu
  //  event.currentTarget=='menuitem'
  this.RebuildRecorderTree();
}// of T_HotLine.prototype.DoRebuildRecorderTree

T_HotLine.prototype.RebuildRecorderTree = function (){
  var RecorderTree=document.getElementById('Recorder-Tree');
  RecorderTree.builder.rebuild();
}// of T_HotLine.prototype.RebuildRecorderTree

T_HotLine.prototype.RebuildHostsTree = function (){
  var HostsTree=document.getElementById('Hosts-Tree');
  HostsTree.builder.rebuild();
}// of T_HotLine.prototype.RebuildHostsTree

T_HotLine.prototype.RefreshDataSource = function (Element){
  //  This was an attempt to reload the DataSource after the StyleSheet had been changed
  //  to force an update of the RDF Tree Styles, but it doesn't work
  var Sources=Element.database.GetDataSources();
  if (Sources.hasMoreElements()){
    do {
      var DataSource=Sources.getNext();
      DataSource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
      DataSource.Flush();
      DataSource.Refresh(true);
    } while (Sources.hasMoreElements());
  };
}// of T_HotLine.prototype.RefreshDataSource

//**********************************************************
//            Host PopUp Menus                             *
//**********************************************************

T_HotLine.prototype.GetHostsTreeHostName = function (){
  //  This is a utility function which returns the
  //  HostName currently selected in the Hosts Tree,
  //  when a Right Click PopUpMenu is requested.
  var Element=document.getElementById('Hosts-Tree');
  var Index=Element.currentIndex;
  if (Index < 0)
    return null;
  var HostName=Element.view.getCellText(Index,Element.columns[0]);
  if (Empty(HostName))
    return null;
  return HostName;
}// of T_HotLine.prototype.GetHostsTreeHostName

T_HotLine.prototype.DoConnectPopUp = function (){
  var Reason=new T_Reason();
  var HostName=this.GetHostsTreeHostName();
  if (HostName == null)
    return;
  if (!this.DoAutoConnect(HostName,Reason))
    this.Display(HostName,Reason.Text);
}// of T_HotLine.prototype.DoConnectPopUp

T_HotLine.prototype.DoDisconnectPopUp = function (){
  var Reason=new T_Reason();
  var HostName=this.GetHostsTreeHostName();
  if (HostName == null)
    return;
  var Host = this.HostList[HostName];
  if (Host!=undefined){
    if (!Host.ShutDown(Reason))
      this.Display(HostName,Reason.Text);
  };
}// of T_HotLine.prototype.DoDisconnectPopUp

T_HotLine.prototype.DoConfigurePopUp = function (){
  var HostName=this.GetHostsTreeHostName();
  if (HostName == null)
    return;
  this.OnSelectHost(HostName);
  this.HideConfigurator(HostName,false);  
}// of T_HotLine.prototype.DoConfigurePopUp

T_HotLine.prototype.DoClearStatusPopUp = function (){
  var Reason=new T_Reason();
  var HostName=this.GetHostsTreeHostName();
  if (HostName == null)
    return;
  //  Delete the Status Events for this Host
  var RichListBox=document.getElementById('Status-RichListBox');
  if (!this.DeleteHostRichListItems(HostName,RichListBox,Reason))
    this.Display(HostName,Reason.Text);
}// of T_HotLine.prototype.DoClearStatusPopUp

T_HotLine.prototype.DoClearRecorderPopUp = function (){
  var Reason=new T_Reason();
  var HostName=this.GetHostsTreeHostName();
  if (HostName == null)
    return;
  //  Delete the Recorder Events for this Host
  //  Not yet implemented.
}// of T_HotLine.prototype.DoClearRecorderPopUp

T_HotLine.prototype.DoAcknowledgePopUp = function (){
  var Reason=new T_Reason();
  var HostName=this.GetHostsTreeHostName();
  if (HostName == null)
    return;
  var RichListBox=document.getElementById('Alert-RichListBox');
  if (!this.DeleteHostRichListItems(HostName,RichListBox,Reason))
    this.Display(HostName,Reason.Text);
}// of T_HotLine.prototype.DoAcknowledgePopUp

T_HotLine.prototype.DoCleanUp = function (HostName,Reason){
  //  Discard Expired Recorder Messages for this Host
  var HoursToKeep=new T_ByReference();
  //  Host must be configured
  var Host = this.GetHotLineHost(HostName,false,Reason);
  if (Host == null)
    return false;
  //  Get the current HoursToKeep value
  if (!this.Configurator.GetHoursToKeep(HostName,HoursToKeep,Reason))
    HoursToKeep.Value=T_HotLineConstant.DefaultHoursToKeep;
  if (!Host.DiscardExpiredRecords(HoursToKeep.Value,Reason))
    return false;
  return true;
}// of T_HotLine.prototype.DoCleanUp

T_HotLine.prototype.DoCleanUpPopUp = function (){
  //  Discard Expired Recorder Messages for this Host
  var Reason=new T_Reason();
  var HostName=this.GetHostsTreeHostName();
  if (!this.DoCleanUp(HostName,Reason))
    this.Display(HostName,Reason.Text);
}// of T_HotLine.prototype.DoCleanUpPopUp

//**********************************************************
//           Deleting Host Configuration                   *
//**********************************************************

T_HotLine.prototype.OnCommandDeleteHost = function (){
  //  Delete Button Pressed
  var Reason = new T_Reason();
  var HostName=this.GetSelectedHostName();
  //  ShutDown the Host if it is Active
  var Host = this.HostList[HostName];
  if (Host!=undefined){
    if (Host.ItsConnected){
       this.Display(HostName,HostName + ' was not deleted because it is connected');
       return;
    };
    //  Delete the entry from the associative array
    delete this.HostList[HostName];
  };
  //  Delete the Host from the Configuration
  if (!this.Configurator.DeleteHost(HostName,Reason)){
    this.Display(HostName,Reason.Text);
    return;
  };
  //  Force a Rebuild of the MenuList. Changing the RDF DataSource
  //  should be sufficient, however, because the GUI realizes that
  //  the menu is not visible, it does not redraw it.

  //  var Element=document.getElementById('HostNames-MenuList');
  //  Element.builder.rebuild();
  if (!this.RebuildHostNamesMenuList('',Reason))
    this.Display(HostName,Reason.Text);
  //  Initialize the Selected HostName
  this.SetSelectedHostName('');
  this.Display(HostName,
               'Deleted '+HostName + ' from ' + 
                           this.Configurator.ConfigurationFileName());
  if (!this.RebuildStyleSheets(Reason))
    this.Display(HostName,Reason.Text);
}// of T_HotLine.prototype.OnCommandDeleteHost

T_HotLine.prototype.InitializeHelpButton = function (){
  //  Set the Help Button to match the State of the Help Box
  var HelpGroupBox=document.getElementById('Help-GroupBox');
  this.ChangeHelpButton(HelpGroupBox.collapsed);
}// of  T_HotLine.prototype.InitializeHelpButton

T_HotLine.prototype.DoHelpButton = function (Collapsed){
  //  The Help Button is only visible when the Help GroupBox
  //  is visible. Pressing the Help Button collapses the Help
  //  Panel and makes the Recorder Panel visible.
  //  Called without a parameter, the current state is toggled,
  //  otherwise set to the value in Initialize.
  var HelpGroupBox=document.getElementById('Help-GroupBox');
  if (arguments.length==0)
    HelpGroupBox.collapsed=!HelpGroupBox.collapsed;
  else
    HelpGroupBox.collapsed=Collapsed;
  var RecorderVBox=document.getElementById('Recorder-VBox');
  RecorderVBox.collapsed=(!HelpGroupBox.collapsed);
  this.ChangeHelpButton(HelpGroupBox.collapsed);
}// of T_HotLine.prototype.DoHelpButton

T_HotLine.prototype.DoSaveButton = function (){
  //  Saves the Configuration Files to the Profile Directory
  var Reason=new T_Reason();
  if (!this.Configurator.SaveConfiguration(Reason))
    this.Display('',Reason.Text);
}// of T_HotLine.prototype.DoSaveButton

T_HotLine.prototype.DoRestoreButton = function (){
  //  Restores the Configuration Files from the Profile Directory
  var Reason=new T_Reason();
  if (!this.Configurator.RestoreConfiguration(Reason))
    this.Display('',Reason.Text);
}// of T_HotLine.prototype.DoRestoreButton

T_HotLine.prototype.DoTestButton = function (){
  //  Test a New Feature
  var Reason=new T_Reason();
  Reason.Text='Feature Not yet Implemented';
  this.Display('',Reason.Text);
}// of T_HotLine.prototype.DoRestoreButton

T_HotLine.prototype.ChangeHelpButton = function (Hidden){
  //  Changes the Help Button Text. Hidden is
  //  set if the help panel is not visible.
  var HelpButton=document.getElementById('Help-Button');
  if (Hidden){
    HelpButton.label=HotLineConstant.OpenHelp();
    HelpButton.tooltipText=HotLineConstant.OpenHelp();
  }
  else{
    HelpButton.label=HotLineConstant.CloseHelp();
    HelpButton.tooltipText=HotLineConstant.CloseHelp();
  };

}// of T_HotLine.prototype.ChangeHelpButton

T_HotLine.prototype.DoExitButton = function (){
  //  Close all connections and Exit HotLine
  var Reason=new T_Reason();
  window.close();
}// of T_HotLine.prototype.DoExitButton

T_HotLine.prototype.DoClearMessages = function (){
  //  Clear the Messages Panel
  var Reason=new T_Reason();
  if (!this.Status.Clear(Reason))
    this.Display('',Reason.Text);
}// of T_HotLine.prototype.DoClearMessages

T_HotLine.prototype.DoChangeConfiguratorBox = function (Hide){
  //  Called when the Configurator Button has been pressed.
  //  Called without a parameter, the current state is toggled,
  //  otherwise set to the value in Initialize.
  var Box=document.getElementById('HostConfiguration-VBox');
  //  Seems to be a problem with hidden when repeatedly making the
  //  Configurator hidden and visible. The screen takes longer and 
  //  longer to refresh and is using most of the processor.
  if (this.UseCollapsed){
    if (arguments.length==0)
      Box.collapsed=!Box.collapsed
    else
      Box.collapsed=Hide;
    this.ChangeConfiguratorButton(Box.collapsed);
  }
  else{
    if (arguments.length==0)
      Box.hidden=!Box.hidden
    else
      Box.hidden=Hide;
    this.ChangeConfiguratorButton(Box.hidden);
  };
}// of T_HotLine.prototype.DoChangeConfiguratorBox

T_HotLine.prototype.ChangeConfiguratorButton = function (Hidden){
  //  Changes the Configurator Button Text. Hidden is
  //  set if the Configurator panel is not visible.
  var ConfiguratorButton=document.getElementById('Change-Configurator-Button');
  if (Hidden){
    ConfiguratorButton.label=HotLineConstant.OpenConfigurator()
    ConfiguratorButton.tooltipText=HotLineConstant.OpenConfigurator()
  }
  else{
    ConfiguratorButton.label=HotLineConstant.CloseConfigurator();
    ConfiguratorButton.tooltipText=HotLineConstant.CloseConfigurator();
  };
  var ClearButton=document.getElementById('ClearMessages-Button');
  ClearButton.hidden=Hidden;
}// of T_HotLine.prototype.ChangeConfiguratorButton

//**********************************************************
//               Attributes                                *
//**********************************************************

T_HotLine.prototype.ChangeStyle = function (HostName,Colour,FontFamily,TextSize,Reason){
  //  Delete the StyleSheet
  var StyleSheetURI=this.Configurator.StyleSheetURI(HostName);
  this.Display(HostName,'Deleting StyleSheet ' + StyleSheetURI + ' for ' + HostName);
  if (!T_StyleSheet.DeleteDOMStyleSheet(StyleSheetURI,true,Reason))
    return false;
  //  ReWrite the StyleSheet
  if (!this.Configurator.CreateStyleSheet(HostName,Colour,FontFamily,TextSize,Reason))
    return false;
  //  Add the StyleSheet 
  this.Display(HostName,'Adding StyleSheet ' + StyleSheetURI + ' for ' + HostName);
  if (!T_StyleSheet.AddDOMStyleSheet(StyleSheetURI,true,Reason))
    return false;
  //  Reload the Skin. The Trees are rebuilt whenever
  //  the skin is reloaded.
  if (!this.RebuildStyleSheets(Reason)) 
    return false;
  return true;
}// of  T_HotLine.prototype.ChangeStyle

//**********************************************************
//             Comment Attribute                           *
//**********************************************************

T_HotLine.prototype.OnChangeComment = function (Comment){
  //  If there is already a Comment for this Host,
  //  and Comment is Empty, then delete the existing
  //  Comment, otherwise replace it.
  var Reason = new T_Reason();
  var HostName=this.GetSelectedHostName();
  if (Empty(HostName))
    return;
  var Element=document.getElementById('Comment-TextBox');
  if (Empty(Element.value)){
    if (this.Configurator.HasComment(HostName,Reason))
      if (!this.Configurator.DeleteComment(HostName,Reason))
        this.Display(HostName,Reason.Text);
  }
  else
    if (!this.Configurator.SetComment(HostName,Element.value,Reason))
      this.Display(HostName,Reason.Text);

}// of T_HotLine.prototype.OnChangeComment

//**********************************************************
//            IPAddress Attribute                          *
//**********************************************************

T_HotLine.prototype.OnChangeIPAddress = function (IPAddress){
  // Configurator IPAddress onchange event
  var Reason = new T_Reason();
  var HostName=this.GetSelectedHostName();
  if (Empty(HostName))
    return;
  if (!this.Configurator.SetIPAddress(HostName,IPAddress,Reason))
    this.Display(HostName,Reason.Text);
}// of T_HotLine.prototype.OnChangeIPAddress

T_HotLine.prototype.OnInputIPAddress = function (IPAddress){
}// of T_HotLine.prototype.OnInputIPAddress

//**********************************************************
//           UserNanme Attribute                           *
//**********************************************************

T_HotLine.prototype.OnChangeUserName = function (UserName){
  // Configurator UserName onchange event            
  var Reason = new T_Reason();
  var Colour = new T_ByReference();
  var HostName=this.GetSelectedHostName();
  if (Empty(HostName))
    return;
  if (!Empty(UserName))
    if (!this.Configurator.SetUserName(HostName,UserName,Reason))
      this.Display(HostName,Reason.Text);
}// of T_HotLine.prototype.OnChangeUserName

T_HotLine.prototype.OnInputUserName = function (IPAddress){
}// of T_HotLine.prototype.OnInputUserName

//**********************************************************
//            Colour Attribute                             *
//**********************************************************

T_HotLine.prototype.OnChangeColour = function (Colour){
  // Configurator Colour onchange event            
  var Reason = new T_Reason();
  var FontFamily = new T_ByReference();
  var TextSize = new T_ByReference();
  var HostName=this.GetSelectedHostName();
  if (Empty(HostName))
    return;
  //  Get the Style Attributes (TextSize..)
  if (!this.Configurator.GetTextSize(HostName,TextSize,Reason)){
    var Element=document.getElementById('Host-TextSize');
    TextSize.Value=Element.value;
  };
  if (!this.Configurator.GetFontFamily(HostName,FontFamily,Reason)){
    var Element=document.getElementById('Host-FontFamily-MenuList');
    FontFamily.Value=Element.label;
  };
  //  Update the Colour
  if (!this.Configurator.SetColour(HostName,Colour,Reason))
    this.Display(HostName,Reason.Text);
  //  Change the Style
  if (!this.ChangeStyle(HostName,Colour,FontFamily.Value,TextSize.Value,Reason))
    this.Display(HostName,Reason.Text);
}// of T_HotLine.prototype.OnChangeColour

//**********************************************************
//           FontFamily Attribute                          *
//**********************************************************

T_HotLine.prototype.OnChangeFontFamily = function (FontFamily){
  // Configurator FontFamily onchange event            
  var Reason = new T_Reason();
  var Colour = new T_ByReference();
  var TextSize = new T_ByReference();
  var HostName=this.GetSelectedHostName();

  if (Empty(HostName))
    return;
  //  Get the Style Attributes (Colour..)
  if (!this.Configurator.GetColour(HostName,Colour,Reason)){
    var Element=document.getElementById('Host-ColorPicker');
    Colour.Value=Element.value;
  };
  if (!this.Configurator.GetTextSize(HostName,TextSize,Reason)){
    var Element=document.getElementById('Host-TextSize');
    TextSize.Value=Element.value;
  };
  //  Update the FontFamily
  if (!this.Configurator.SetFontFamily(HostName,FontFamily,Reason))
    this.Display(HostName,Reason.Text);
  //  Change the Style
  if (!this.ChangeStyle(HostName,Colour.Value,FontFamily,TextSize.Value,Reason))
    this.Display(HostName,Reason.Text);
}// of T_HotLine.prototype.OnChangeFontFamily

//**********************************************************
//           TextSize Attribute                            *
//**********************************************************

T_HotLine.prototype.OnChangeTextSize = function (TextSize){
  // Configurator TextSize onchange event            
  var Reason = new T_Reason();
  var Colour = new T_ByReference();
  var FontFamily = new T_ByReference();
  var HostName=this.GetSelectedHostName();
  if (Empty(HostName))
    return;
  //  Get the Style Attributes (Colour..)
  if (!this.Configurator.GetColour(HostName,Colour,Reason)){
    var Element=document.getElementById('Host-ColorPicker');
    Colour.Value=Element.value;
  };
  if (!this.Configurator.GetFontFamily(HostName,FontFamily,Reason)){
    var Element=document.getElementById('Host-FontFamily-MenuList');
    FontFamily.Value=Element.label;
  };
  //  Update the TextSize
  if (!this.Configurator.SetTextSize(HostName,TextSize,Reason))
    this.Display(HostName,Reason.Text);
  //  Change the Style
  if (!this.ChangeStyle(HostName,Colour.Value,FontFamily.Value,TextSize,Reason))
    this.Display(HostName,Reason.Text);
}// of T_HotLine.prototype.OnChangeTextSize

//**********************************************************
//           AutoConnect Attribute                         *
//**********************************************************

T_HotLine.prototype.OnChangeAutoConnect = function (AutoConnect){
  // AutoConnect is the new value of the attribute 
  // Configurator AutoConnect onchange event            
  var Reason = new T_Reason();
  var HostName=this.GetSelectedHostName();
  if (Empty(HostName))
    return;
  //  Update the AutoConnect Attribute
  if (!this.Configurator.SetAutoConnect(HostName,AutoConnect,Reason))
    this.Display(HostName,Reason.Text);
}// of  T_HotLine.prototype.OnChangeAutoConnect

//**********************************************************
//           HoursToKeep Attribute                         *
//**********************************************************

T_HotLine.prototype.OnChangeHoursToKeep = function (HoursToKeep){
  // HoursToKeep is the new value of the attribute 
  // Configurator HoursToKeep onchange event            
  var Reason = new T_Reason();
  var HostName=this.GetSelectedHostName();
  if (Empty(HostName))
    return;
  //  Update the HoursToKeep Attribute
  if (!this.Configurator.SetHoursToKeep(HostName,HoursToKeep,Reason))
    this.Display(HostName,Reason.Text);
}// of  T_HotLine.prototype.OnChangeHoursToKeep

//**********************************************************
//     TypeName Manual Entry                               *
//**********************************************************

T_HotLine.prototype.DoHideRecorderNames = function (HideButtonId){
  var Element=document.getElementById('Recorder-Names-HBox');
  var HideButton=document.getElementById(HideButtonId);
  if (Element.collapsed){
    Element.collapsed=false;
    HideButton.label=HotLineConstant.HideRecNames();
  }
  else{
    Element.collapsed=true;
    HideButton.label=HotLineConstant.ViewRecNames(); 
  };
}// of T_HotLine.prototype.DoHideRecorderNames

T_HotLine.prototype.ReportTypeNameError = function (Text){
  var StatusLine=document.getElementById('Enter-TypeName-Error-Label');
  StatusLine.collapsed=false;
  StatusLine.value=Text;
}// of T_HotLine.prototype.ReportTypeNameError

T_HotLine.prototype.ResetTypeNameError = function (){
  var StatusLine=document.getElementById('Enter-TypeName-Error-Label');
  StatusLine.collapsed=true;
  StatusLine.value='';
}// of T_HotLine.prototype.ReportTypeNameError

T_HotLine.prototype.ClearTypeNameAndValue = function (){
  var TypeNameElement=document.getElementById('Enter-TypeName-TextBox');
  var TypeValueElement=document.getElementById('Enter-TypeValue-TextBox');
  TypeNameElement.value='NEW-NAME';
  TypeValueElement.value='0';
  this.ResetTypeNameError();
}// of T_HotLine.prototype.ReportTypeNameError

T_HotLine.prototype.OnInputTypeName = function (TypeName){
  var Element=document.getElementById('Enter-TypeName-TextBox');
  var TypeName=(Element.value).toUpperCase();
  var Reason=new T_Reason();
  var ItemRef=new T_ByReference();
  //  Check for valid values
  if (!this.Configurator.ValidTypeName(TypeName,ItemRef,Reason)){
    this.ReportTypeNameError(Reason.Text)
    Element.value=ItemRef.Value;
    return;
  };
  this.ResetTypeNameError();
}// of T_HotLine.prototype.OnInputTypeName

T_HotLine.prototype.OnInputTypeValue = function (TypeValue){
  var Element=document.getElementById('Enter-TypeValue-TextBox');
  var TypeValue=Element.value;
  var Reason=new T_Reason();
  var ItemRef=new T_ByReference();
  //  Check for valid values
  if (!this.Configurator.ValidTypeValue(TypeValue,ItemRef,Reason)){
    this.ReportTypeNameError(Reason.Text)
    Element.value=ItemRef.Value;
    return;
  };
  this.ResetTypeNameError();
}// of T_HotLine.prototype.OnInputTypeValue

T_HotLine.prototype.DoManualAddTypeName = function (){
  var TypeName=(document.getElementById('Enter-TypeName-TextBox').value).toUpperCase();
  var TypeValue=document.getElementById('Enter-TypeValue-TextBox').value;
  var HostName=this.GetSelectedHostName();
  var Reason=new T_Reason();
  var ItemRef=new T_ByReference();
  //  Check for valid values
  if (!this.Configurator.ValidTypeName(TypeName,ItemRef,Reason)){
    this.ReportTypeNameError(Reason.Text);
    return;
  };
  if (!this.Configurator.ValidTypeValue(TypeValue,ItemRef,Reason)){
    this.ReportTypeNameError(Reason.Text);
    return;
  };
  // Add the TypeName to the Host Configuration
  if (!this.Configurator.StoreTypeNameAndValue(HostName,TypeName,TypeValue,Reason)){
    this.ReportTypeNameError(Reason.Text);
    return;
  };
  this.ClearTypeNameAndValue();
}// of T_HotLine.prototype.DoManualAddTypeName

T_HotLine.prototype.DoSelectAllTypeNames = function (Checked){
  //  This function is called by the oncommand event
  //  of the Recorder Filter ALL CheckBox, when the
  //  CheckBox state is changed.
  var Result;
  var Reason = new T_Reason();
  var HostName=this.GetSelectedHostName();
  //  Update the configuration for the Host
  if (!this.Configurator.SetSelectAll(HostName,Checked,Reason)){
    this.Display(HostName,Reason);
    return;
  };
  //  Check if the Host is active and if so rebuild
  //  the Record Filter
  var Host = this.GetHotLineHost(HostName,false,Reason);
  if (Host == null)
    return;
  if (!Host.RebuildRecordFilter(Reason))
     this.Display(HostName,Reason.Text);
}// of T_HotLine.prototype.DoSelectAllTypeNames


T_HotLine.prototype.DoRecNamesUpdate = function (){
  //  Send a request to the Host to transmit the 
  //  current TT REC NAMES list.
  var Reason = new T_Reason();
  var HostName=this.GetSelectedHostName();
  //  Check if the Host is active
  var Host = this.GetHotLineHost(HostName,false,Reason);
  if (Host == null)
    return;
  if (!Host.RefreshTypeNames(Reason)){
    this.Display(HostName,Reason.Text);
    return false;
  };
}// of T_HotLine.prototype.DoRecNamesUpdate

T_HotLine.prototype.DoRecNamesUpDown = function (By){
  //  Move the REC NAMES list in the direction and 
  //  for as many elements indicated by By
 var ArrowScrollBox=document.getElementById('TT-REC-NAMES-ArrowScrollBox');
 ArrowScrollBox.scrollByIndex(By);
}// of T_HotLine.prototype.DoRecNamesUpDown

//**********************************************************
//     TypeNames Drag and Drop Interface                   *
//**********************************************************

T_HotLine.prototype.CanDrop = function (Event){
  Event.preventDefault();
  return false;  
}// of T_HotLine.prototype.CanDrop

T_HotLine.prototype.StartDrag = function (Event){
  var Label=Event.target.getAttribute("label");
  Event.dataTransfer.setData("text/plain",Label);
  Event.dataTransfer.effectAllowed="copy";
  //  Using the mouse, so they have read the message
  this.ClearTypeNameAndValue();
}// of T_HotLine.prototype.StartDrag

T_HotLine.prototype.DragOver = function (Event){
  var DragGroup=Event.target.getAttribute('draggroup');
  var DropEffect=Event.dataTransfer.dropEffect;
  if (DragGroup == 'bin')
    return HotLine.CanDrop(Event);
  var DragSession = HotLine.DragService.getCurrentSession();
  if (DragSession.sourceNode.hasAttribute('typenamevalue'))
    switch (DragGroup){
   case 'record':
   case 'alert':
   case 'status':
   case 'bin':
       return HotLine.CanDrop(Event);
   default:
       break;  
    };
  return true;
}// of T_HotLine.prototype.DragOver

T_HotLine.prototype.DragEnter = function (Event){
  HotLine.CanDrop(Event);
}// of T_HotLine.prototype.DragEnter

T_HotLine.prototype.Drop = function (Event){
  var TypeName;
  var DragSession = HotLine.DragService.getCurrentSession();
    //  Prevent this being handled by FireFox
  Event.preventDefault();
  var Label=Event.dataTransfer.getData('text/plain');
  var HostName=this.GetSelectedHostName();
  //  The typenamevalue and typenamelabel attributes are
  //  configured on elements in the TT REC NAMES Table.
  switch (Event.target.getAttribute('draggroup')){
 case 'record':
     if (DragSession.sourceNode.hasAttribute('typenamevalue')){
       TypeName=DragSession.sourceNode.getAttribute('typenamelabel');
       if (!this.Configurator.MarkAsRecordFilter(HostName,TypeName,Reason))
         this.Display(HostName,Reason.Text); 
       if (!this.DoRecordFilterChanged(HostName,Reason))
         this.Display(HostName,Reason.Text);
       return;
     };
     break;
 case 'alert':
     if (DragSession.sourceNode.hasAttribute('typenamevalue')){
       TypeName=DragSession.sourceNode.getAttribute('typenamelabel');
       if (!this.Configurator.MarkAsAlertFilter(HostName,TypeName,Reason))
         this.Display(HostName,Reason.Text); 
       if (!this.DoAlertFilterChanged(HostName,Reason))
         this.Display(HostName,Reason.Text);
       return;
     };
     break;
 case 'status':
     if (DragSession.sourceNode.hasAttribute('typenamevalue')){
       TypeName=DragSession.sourceNode.getAttribute('typenamelabel');
       if (!this.Configurator.MarkAsStatusFilter(HostName,TypeName,Reason))
         this.Display(HostName,Reason.Text); 
       if (!this.DoStatusFilterChanged(HostName,Reason))
         this.Display(HostName,Reason.Text);
       return;
     };
     break;
 case 'bin':
     if (DragSession.sourceNode.hasAttribute('recordvalue')){
       TypeName=DragSession.sourceNode.getAttribute('recordlabel');
       if (!this.Configurator.UnMarkAsRecordFilter(HostName,TypeName,Reason))
         this.Display(HostName,Reason.Text); 
       if (!this.DoRecordFilterChanged(HostName,Reason))
         this.Display(HostName,Reason.Text);
       return;
     };
     if (DragSession.sourceNode.hasAttribute('alertvalue')){
       TypeName=DragSession.sourceNode.getAttribute('alertlabel');
       if (!this.Configurator.UnMarkAsAlertFilter(HostName,TypeName,Reason))
         this.Display(HostName,Reason.Text); 
       if (!this.DoAlertFilterChanged(HostName,Reason))
         this.Display(HostName,Reason.Text);
       return;
     };
     //  The statusvalue and statuslabel attributes are applied to
     //  the entries in the Status-Filter-GroupBox in the XUL. When
     //  a typename is dropped, in the Bin, it has to be deleted from
     //  the configuration file, and the message filter table has to
     //  be updated.
     if (DragSession.sourceNode.hasAttribute('statusvalue')){
       TypeName=DragSession.sourceNode.getAttribute('statuslabel');
       if (!this.Configurator.UnMarkAsStatusFilter(HostName,TypeName,Reason))
         this.Display(HostName,Reason.Text); 
       if (!this.DoStatusFilterChanged(HostName,Reason))
         this.Display(HostName,Reason.Text);
       return;
     };
     if (DragSession.sourceNode.hasAttribute('typenamevalue')){
       TypeName=DragSession.sourceNode.getAttribute('typenamelabel');
       if (!this.Configurator.DeleteTypeName(HostName,TypeName,Reason))
         this.Display(HostName,Reason.Text); 
       return;
     };
     break;
  };
}// of T_HotLine.prototype.Drop

T_HotLine.prototype.getSupportedFlavours = function (){
    var flavours=new FlavourSet();
    flavours.appendFlavour("text/unicode");
    return flavours;
}// of T_HotLine.prototype.getSupportedFlavours

T_HotLine.prototype.DoRecordFilterChanged = function (HostName,Reason){
  var Host = this.GetHotLineHost(HostName,false,Reason);
  //  If the Host has not been initialized, then return
  //  true since the Filter Tables will be rebuilt then.
  if (Host == null)
    return true;
  return Host.RebuildRecordFilter(Reason);
}// of T_HotLine.prototype.DoRecordFilterChanged

T_HotLine.prototype.DoAlertFilterChanged = function (HostName,Reason){
  var Host = this.GetHotLineHost(HostName,false,Reason);
  if (Host == null)
    return true;
  return Host.RebuildAlertFilter(Reason);
}// of T_HotLine.prototype.DoAlertFilterChanged

T_HotLine.prototype.DoStatusFilterChanged = function (HostName,Reason){
  var Host = this.GetHotLineHost(HostName,false,Reason);
  if (Host == null)
    return true;
  return Host.RebuildStatusFilter(Reason);
}// of T_HotLine.prototype.DoStatusFilterChanged

//**********************************************************
//              Connecting Host                            *
//**********************************************************

T_HotLine.prototype.Connect = function (HostName,UserName,IPAddress,Reason){
  //  Connect to HostName with the specified IPAddress
  var Host;
  Host = this.GetHotLineHost(HostName,true,Reason);
  if (Host == null)
    return false;
  this.Display(HostName,
               'Connect to ' + HostName + ' as ' + UserName + ' using Address ' + IPAddress);
  //  Start the Connection
  if (!Host.Connect(UserName,IPAddress,Reason))
    return false;
  return true;
}// of T_HotLine.prototype.Connect

T_HotLine.prototype.DoConnect = function (){
  //  Connect Button Pressed
  var Reason = new T_Reason();
  var HostName=this.GetSelectedHostName();
  var UserName=document.getElementById("UserName-TextBox");
  var IPAddress=document.getElementById("IPAddress-TextBox");
  
  if (!this.Connect(HostName,UserName.value,IPAddress.value,Reason))
   this.Display(HostName,Reason.Text);
}// of T_HotLine.prototype.DoConnect

T_HotLine.prototype.DoAutoConnect = function (HostName,Reason){
  //  Connect Button Pressed
  var UserName=new T_ByReference();
  var IPAddress=new T_ByReference();
  if (!this.Configurator.GetIPAddress(HostName,IPAddress,Reason))
    return false;
  if (!this.Configurator.GetUserName(HostName,UserName,Reason))
    UserName.Value='';
  return this.Connect(HostName,UserName.Value,IPAddress.Value,Reason);
}// of T_HotLine.prototype.DoAutoConnect

T_HotLine.prototype.DoConnected = function (HostName,ConnectText){
  //  HotLine Session notifying a successful connection
  //  with the TypeName List being supplied in the
  //  ConnectText.
  this.Display(HostName,'Connected ' + ConnectText);
}// of T_HotLine.prototype.DoConnected

T_HotLine.prototype.DoRejected = function (HostName,RejectText){
  //  HotLine Session notifying an unsuccessful connection
  //  with the reason being supplied in the
  //  RejectText.
  this.Display(HostName,RejectText);
}// of T_HotLine.prototype.DoRejected

T_HotLine.prototype.DoDisconnected = function (HostName,DisconnectText){
  //  HotLine Session notifying disconnection
  //  with the Disconnection reason being supplied in the
  //  DisconnectText.
  this.Display(HostName,DisconnectText);
}// of T_HotLine.prototype.DoDisconnected

//**********************************************************
//                   Record                                *
//**********************************************************

T_HotLine.prototype.DoRecord = function (HostName,ClassName,RecorderData){
  //  This function is called by the HotLineHost when
  //  a Recoder message has been selected and the 
  //  log update with its contents. Since the log
  //  is an RDF DataSource, the Recorder Tree is
  //  automatically updated.
  
}// of T_HotLine.prototype.DoRecord

//**********************************************************
//                 RichListBox Functions                   *
//**********************************************************

function HotLineSelectRichListItem(event){
  //  This is a callback for the EventListener attached to
  //  a RichListItem.
  event.stopPropagation();
  switch (event.target.tagName){
case 'button':
case 'image':
    HotLine.SelectDetails(event);
    break;
default:
    HotLine.SelectRichListItem(event);
    break;
  };
}// of HotLineSelectRichListItem

T_HotLine.prototype.SelectRichListItem = function (event){
  //  Called from the global function HotLineSelectRichListItem
  //  (declared above) when the eventlistener fires because a
  //  RichListItem has been clicked.
  var RichListItem=event.currentTarget.cloneNode(true);
  var HostName=RichListItem.getAttribute('hostname');
  var LogTimeStamp=RichListItem.getAttribute('logtimestamp');
  var ClassName=RichListItem.getAttribute('classname');
  var RecorderData=new T_RecorderData(RichListItem.getAttribute('msg'));
  switch (RichListItem.getAttribute('acktype')){
 case HotLineConstant.AckAlert():
    var Postponed=new T_ByReference();
    if (!this.AcknowledgeAlert(HostName,LogTimeStamp,ClassName,RecorderData,
                               Postponed,Reason))
      this.Display(HostName,Reason.Text)
    else
      if (Postponed.Value){
        //  Change the class of the Description 
        var Elements=event.currentTarget.childNodes;
        for(k=0;k<Elements.length;k++)
          if (Elements[k].tagName=='description')
            Elements[k].setAttribute('class',T_HotLineConstant.HostClass(HostName));
        //  Change the class of the RichListItem
        event.currentTarget.setAttribute('class',T_HotLineConstant.StatusRichListItemClass(HostName));
      };
    break;
 case HotLineConstant.AckStatus():
    if (!this.AcknowledgeStatus(HostName,LogTimeStamp,ClassName,RecorderData,Reason))
      this.Display(HostName,Reason.Text);
    break;
  };
}// of T_HotLine.prototype.SelectRichListItem

T_HotLine.prototype.BuildDetailsXUL = function (HostName,XUL,PreambleSize){
alert('HostName XUL='+XUL);
  var Preamble='<?xml version="1.0"?>' + NL +
           '<?xml-stylesheet href="chrome://global/skin/" type="text/css"?>' + NL +
           '<?xml-stylesheet href="' + 
                T_HotLineConstant.ChromeSkin('hotline.css') + '" type="text/css"?>' + NL +
           '<?xml-stylesheet href="' + 
                T_HotLineConstant.ChromeSkin(T_HotLineConstant.StyleSheetFileName(HostName)) + 
                                                         '" type="text/css"?>' + NL +
          '<!DOCTYPE window SYSTEM "' + T_HotLineConstant.ChromeLocale('hotline.dtd') + '">' + NL +
          '<window id="Details-Dialog"' +
                 ' xmlns="http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul"' +
                 ' xmlns:html="http://www.w3.org/1999/xhtml"' +
                 ' buttons="accept"' +
                 ' onload="OnLoad();"' +
                 ' title="&Details-Dialog-Title;"' +
                 ' context="Details-PopUp"' +
                 ' flex="1">'+ NL +
            '<script src="' + T_HotLineConstant.ChromeContent('detailsupport.js') + '"/>' + NL +
            '<script src="' + T_HotLineConstant.ChromeContent('generalsupport.js') + '"/>' + NL +
            '<script src="' + T_HotLineConstant.ChromeContent('hotlineconstants.js') + '"/>' + NL +

            '<script src="' + T_HotLineConstant.ChromeMochiKit('Base.js') + '"/>' + NL +
            '<script src="' + T_HotLineConstant.ChromeMochiKit('Color.js') + '"/>' + NL +
            '<script src="' + T_HotLineConstant.ChromeMochiKit('Iter.js') + '"/>' + NL +
            '<script src="' + T_HotLineConstant.ChromeMochiKit('DOM.js') + '"/>' + NL +
            '<script src="' + T_HotLineConstant.ChromeMochiKit('Format.js') + '"/>' + NL +

            '<script src="' + T_HotLineConstant.ChromePlotKit('Base.js') + '"/>' + NL +
            '<script src="' + T_HotLineConstant.ChromePlotKit('Layout.js') + '"/>' + NL +
            '<script src="' + T_HotLineConstant.ChromePlotKit('Canvas.js') + '"/>' + NL +
            '<script src="' + T_HotLineConstant.ChromePlotKit('SweetCanvas.js') + '"/>' + NL +
            '<script src="' + T_HotLineConstant.ChromePlotKit('SVG.js') + '"/>' + NL +
            '<script src="' + T_HotLineConstant.ChromePlotKit('SweetSVG.js') + '"/>' + NL +

            '<popupset>' +
               '<popup id="Details-PopUp" flex="1">' +
                  '<menuitem id="Details-PopUp-ShowSource"' +
                             ' oncommand="ShowSource();"' +            
                             ' label="&ShowSource-PopUp-Label;"/>' +
               '</popup>' +
             '</popupset>' +
            '<vbox id="Details-Box" flex="1">';
   PreambleSize.Value=Preamble.length;
   return Preamble +
            XUL +
            '</vbox>' +
          '</window>'; 
}// of T_HotLine.prototype.BuildDetailsXUL

T_HotLine.prototype.ParseXUL = function (XUL,Reason){
  //  String XUL, T_Reason Reason
  var Parser=new DOMParser();
  var Document=Parser.parseFromString(XUL,"text/xml");

  var RootTag = Document.documentElement;
  if ((RootTag.tagName == "parsererror") ||
      (RootTag.namespaceURI == "http://www.mozilla.org/newlayout/xml/parsererror.xml")){
    Reason.Text=RootTag.firstChild.nodeValue;
    return false;
  };
  return true;
}// of T_HotLine.prototype.ParseXUL

T_HotLine.prototype.SelectDetails = function (event){
  //  Called from the global function HotLineSelectRichListItem
  //  (declared above) when the eventlistener fires because a
  //  RichListItem Button has been clicked.
  var Reason=new T_Reason();
  var RichListItem=event.currentTarget.cloneNode(true);
  var HostName=RichListItem.getAttribute('hostname');
  var RecorderData=new T_RecorderData(RichListItem.getAttribute('msg'));
  if (!this.ShowDetails(HostName,RecorderData,Reason))
    this.Display(HostName,Reason.Text);
}// of T_HotLine.prototype.SelectDetails

T_HotLine.prototype.ShowDetails = function (HostName,RecorderData,Reason){
  window.openDialog('chrome://hotline/content/details.xul',
                    '',
                //    'chrome,modal,dialog,resizable',
                   'chrome,dialog,resizable',
                    HostName,
                    RecorderData);
  return true;
}// of T_HotLine.prototype.ShowDetails

T_HotLine.prototype.PreFF4ShowDetails = function (HostName,RecorderData,Reason){
  //  Execute the XUL in a Modal Dialog Window
  var PreambleSize=new T_ByReference();
  var XUL=this.BuildDetailsXUL(HostName,RecorderData.XUL(),PreambleSize);
  if (!this.ParseXUL(XUL,Reason)){
    var Match=Reason.Text.match(/.*Column *([0-9]*)/);
    if (Match==null){
      //  Ensure that the XUL does not have any CDATA sections otherwise
      //  the error message xml will be invalid
      var Text=ReplaceCDATA(RecorderData.XUL());
      var SourceText='<description class="' + T_HotLineConstant.HostClass(HostName) + '">' + 
                          '<![CDATA[' + NL +
                           Text + NL +  
                           ']]>' +
                        '</description>';
    }
    else{
      var Column=Match[1]-1;
      //  Ensure that the XUL does not have any CDATA sections otherwise
      //  the error message xml will be invalid
      var Text=ReplaceCDATA(XUL);
      var SourceText='';
      var iStart=0;
      if (PreambleSize.Value < Column){
        SourceText='<description>' + 
                        '<![CDATA[' + Text.slice(0,PreambleSize.Value) + ']]>' +
                     '</description>';
        iStart=PreambleSize.Value;
      }
      else
        iStart=0;
      SourceText+='<description class="' + T_HotLineConstant.HostClass(HostName) + '">' + 
                          '<![CDATA[' + Text.slice(iStart,Column) + ']]>' +
                        '</description>' +
                        '<description>' +
                          '<![CDATA[' + Text.slice(Column,Text.length) + ']]>' +
                    '</description>';
    }
    XUL=this.BuildDetailsXUL(HostName,
                             '<groupbox>' +
                               '<caption label="&Details-ParseError-Caption;"/>' +
                               '<description><![CDATA[' + Reason.Text + ']]></description>' +
                               SourceText +
                             '</groupbox>',
                             PreambleSize);
  };
  window.openDialog('data:application/vnd.mozilla.xul+xml,' + encodeURIComponent(XUL),
                    '',
                    'chrome,modal,dialog,resizable',
                    HostName,
                    RecorderData);
  return true;
}// of T_HotLine.prototype.PreFF4ShowDetails

T_HotLine.prototype.BuildRichListItem = function (AckType,HostName,LogTimeStamp,ClassName,RecorderData,Stale,Reason){
  //  AckType is either AckAlert or AckStatus as defined in HotLineConstants
  //  HostName, LogTimeStamp and ClassName are strings.
  //  RecorderData is an Object.
  var Tagged=RecorderData.ItsTagged();
  var ItsXUL=RecorderData.ItsXUL();
  var RichListItem=document.createElement('richlistitem');
  if (Tagged)
    RichListItem.setAttribute('tag',RecorderData.Tag);
  RichListItem.setAttribute('hostname',HostName);
  RichListItem.setAttribute('logtimestamp',LogTimeStamp);
  RichListItem.setAttribute('classname',ClassName);
  RichListItem.setAttribute('msg',RecorderData.Msg);
  RichListItem.setAttribute('itsxul',ItsXUL);
  RichListItem.setAttribute('class',(Stale ? T_HotLineConstant.StaleRichListItemClass(HostName) :
                                             T_HotLineConstant.StatusRichListItemClass(HostName)));
  RichListItem.setAttribute('acktype',AckType);

  //  XUL information <Image>
  if (ItsXUL){
    var Image=document.createElement('image');
    Image.setAttribute('src','chrome://hotline/content/details.png');
    RichListItem.appendChild(Image);
  };
  //  Call an outerblock function which can access HotLine
  RichListItem.addEventListener('click',HotLineSelectRichListItem, true);

  //  Setting the style here has the side effect of removing all other
  //  styling, and so avoids the style change when it is selected.
  //  This should be changed later on to be part of the skin.
  RichListItem.setAttribute('style','background:white');

  //  Add a <description> element
  var Description=document.createElement('description');
  Description.setAttribute('class',(Stale ? T_HotLineConstant.StaleDescriptionClass(HostName) :
                                            T_HotLineConstant.HostClass(HostName)));
  var TimeStamp=RecorderData.TimeStamp();
  var Content='';
  if (ItsXUL)
    Content=RecorderData.Abstract()
  else
    if (Tagged)
      Content=RecorderData.TaggedText
    else
      Content=RecorderData.Text();
  var Text = HotLineConstant.ShortTimeStamp(TimeStamp.getDay(),
                                             TimeStamp.getHours(),
                                             TimeStamp.getMinutes()) +
              ' ' + Content;
  //  If a TextNode is used, the text is wrapped and multiple
  //  spaces are eliminated, but this means that fixed format
  //  format Status messages from Recorder do not line up.
  if (false){
    var TextLine = document.createTextNode(Text);
    Description.appendChild(TextLine);
  }
  else
    Description.setAttribute('value',Text);
  Description.setAttribute('flex','1');
  RichListItem.appendChild(Description);

  //  The details button has been replaced by the information icon
  /* (false){
    //  Create a Button to get the Details
    var Button=document.createElement('button');
    Button.setAttribute('label','Details');
    Button.setAttribute('class',T_HotLineConstant.DetailsButtonClass(HostName));
    RichListItem.appendChild(Button);
  };*/

  //  Note: If the mouse hovers over the Description element region first,
  //        a litle square is shown. If it hovers over the RichListElement
  //        it is displayed correctly.If the flex attribute is set on the
  //        Description, then it displays as a little square, since there
  //        is no exposed RichListItem. It seems to provoke a FireFox crash
  //        so it has been disabled.
  //  RichListItem.tooltipText=LogTimeStamp + ' ' + RecorderData.Hint(ClassName);

  return RichListItem;
}// of T_HotLine.prototype.BuildRichListItem

T_HotLine.prototype.AddRichListItem = function (AckType,HostName,RichListBox,LogTimeStamp,
                                                 ClassName,RecorderData,Stale,Reason){
  //  AckType is either AckAlert or AckSTatus as defined in HotLineConstants
  var RichListItem=this.BuildRichListItem(AckType,HostName,LogTimeStamp,ClassName,RecorderData,Stale,Reason);
  if (RichListItem==null)
    return false;
  RichListBox.appendChild(RichListItem);
  return true;
}// of T_HotLine.prototype.AddRichListItem

T_HotLine.prototype.ItsTagged = function (Who,RecorderData,Reason){
  if (RecorderData.ItsTagged())
    return true;
  Reason.Text='Ignored UnTagged Recorder Data (' + Who + ')';
  return false;
}// of T_HotLine.prototype.ItsTagged

T_HotLine.prototype.ModifyTaggedRichListItems = function (AckType,HostName,RichListBox,LogTimeStamp,
                                                            ClassName,RecorderData,Stale,Reason){
  //  Replace the first RichListItem in the RichListBox which has a HostName
  //  attribute which matches the given HostName and a Tag attribute which
  //  matches the Tag, and delete any other matching entries.
  if (!this.ItsTagged('Modify',RecorderData,Reason))
    return false;
  var NewRichListItem=this.BuildRichListItem(AckType,HostName,LogTimeStamp,
                                             ClassName,RecorderData,
                                             Stale,Reason);
  if (NewRichListItem==null)
    return false;
  var RichListItems=RichListBox.children;
  if (RichListItems.length > 0)
    for (var j=RichListItems.length-1;j>=0;j--){
      var RichListItem=RichListItems[j];
      if ((RichListItem.getAttribute('hostname')==HostName) && 
          (RichListItem.getAttribute('tag')==RecorderData.Tag)){
        if (NewRichListItem==null)
          RichListBox.removeChild(RichListItem)
        else{
          RichListBox.replaceChild(NewRichListItem,RichListItem);
          NewRichListItem=null;
        };//else
      };//if 
    };//for
  if (NewRichListItem!=null)
    RichListBox.appendChild(NewRichListItem);
  return true;
}// of T_HotLine.prototype.ModifyTaggedRichListItems

T_HotLine.prototype.DeleteTaggedRichListItems = function (HostName,RichListBox,RecorderData,Reason){
  //  Delete any RichListItem in the RichListBox which has a
  //  HostName attribute which matches the given HostName and
  //  a Tag attribute which matches the Tag.
  if (!this.ItsTagged('Delete',RecorderData,Reason))
    return false;
  var RichListItems=RichListBox.children;
  if (RichListItems.length > 0){
    for (var j=RichListItems.length-1;j>=0;j--){
      var RichListItem=RichListItems[j];
      if ((RichListItem.getAttribute('hostname')==HostName) && 
          (RichListItem.getAttribute('tag')==RecorderData.Tag))
        RichListBox.removeChild(RichListItem);
    };
  };
  return true;
}// of T_HotLine.prototype.DeleteTaggedRichListItems

T_HotLine.prototype.AgeRichListItems = function (RichListBox,ReactionTimeInSeconds,Reason){
  //  Search the RichListItems for any entries which have become
  //  stale, and change their class so that a different style
  //  can be applied.
  var RichListItems=RichListBox.children;
  var Now=new Date();
  if (RichListItems.length > 0){
    for (var j=RichListItems.length-1;j>=0;j--){
      var RichListItem=RichListItems[j];
      var HostName=RichListItem.getAttribute('hostname');
      var LogTimeStamp=FormattedYYYYMMDDHHMMSSNNNToDate(RichListItem.getAttribute('logtimestamp'));
      var Age=Math.floor((Now.getTime()-LogTimeStamp.getTime())/1000);
      if (Age > ReactionTimeInSeconds){
        //  Change the class of the Description 
        var Elements=RichListItem.childNodes;
        for(k=0;k<Elements.length;k++)
          if (Elements[k].tagName=='description')
            Elements[k].setAttribute('class',T_HotLineConstant.StaleDescriptionClass(HostName));
        //  Change the class of the RichListItem
        RichListItem.setAttribute('class',T_HotLineConstant.StaleRichListItemClass(HostName));
      };
    };
  };
  return true;
}// of T_HotLine.prototype.AgeRichListItems

T_HotLine.prototype.DeleteHostRichListItems = function (HostName,RichListBox,Reason){
  //  Delete any RichListItem in the RichListBox which has a
  //  HostName attribute which matches the given HostName
  var RichListItems=RichListBox.children;
  if (RichListItems.length > 0){
    for (var j=RichListItems.length-1;j>=0;j--){
      var RichListItem=RichListItems[j];
      if (RichListItem.getAttribute('hostname')==HostName)
        RichListBox.removeChild(RichListItem);
    };
  };
  return true;
}// of T_HotLine.prototype.DeleteHostRichListItems

T_HotLine.prototype.DeleteHostRichListItem = function (HostName,LogTimeStamp,RichListBox,Reason){
  //  Delete any RichListItem in the RichListBox which has a
  //  HostName and LogTimeStamp attribute matching those supplied.
  //  There should only be one matching entry.
  var Count=0;
  var RichListItems=RichListBox.children;
  if (RichListItems.length > 0){
    for (var j=RichListItems.length-1;j>=0;j--){
      var RichListItem=RichListItems[j];
      if ((RichListItem.getAttribute('hostname')==HostName)&&
           (RichListItem.getAttribute('logtimestamp')==LogTimeStamp)){
        RichListBox.removeChild(RichListItem);
        Count++;
      };
    };
  };
  if (Count == 0){
    Reason.Text=RichListBox.id + ' No Matching Entry for ' + HostName + ' logged at ' + LogTimeSTamp;
    return false;
  };
  return true;
}// of T_HotLine.prototype.DeleteHostRichListItem

//**********************************************************
//                   Alert                                 *
//**********************************************************

T_HotLine.prototype.TestAge = function (){
  var Reason=new T_Reason();
  var RichListBox=document.getElementById('Alert-RichListBox');
  if (!this.AgeRichListItems(RichListBox,0,Reason))
    alert(Reason.Text);
}// of T_HotLine.prototype.TestAge

T_HotLine.prototype.AgeAlertRichListItems = function (Reason){
  var RichListBox=document.getElementById('Alert-RichListBox');
  return this.AgeRichListItems(RichListBox,T_HotLineConstant.AlertReactionTimeInSeconds,Reason);
}// of T_HotLine.prototype.AgeAlertRichListItems

T_HotLine.prototype.DoAlert = function (HostName,LogTimeStamp,ClassName,RecorderData,Stale){
  //  This function is called by the HotLineHost when
  //  an Alert message has been selected.
  //  The Action is either + or -. The Tag is the key used
  //  to identify an entry.
  var Reason=new T_Reason();
  var Tagged=RecorderData.ItsTagged();
  var Cancel=(Tagged ? (RecorderData.Action == '-') : false);
  var RichListBox=document.getElementById('Alert-RichListBox');
  //  Handle UnTagged Alert Messages
  if (!Tagged){
    //  Add it to the Alert Panel
    if (!this.AddRichListItem(HotLineConstant.AckAlert(),HostName,RichListBox,LogTimeStamp,
                              ClassName,RecorderData,Stale,Reason))
       this.Display(HostName,Reason.Text);
    return;
  };

  //  Tagged Alert. Delete any RichListItem with a matching Tag
  if (!this.DeleteTaggedRichListItems(HostName,RichListBox,RecorderData,Reason)){
    this.Display(HostName,Reason.Text);
    return;
  };
  //  Add a RichListItem for this RecorderData
  if (!Cancel)
    if (!this.AddRichListItem(HotLineConstant.AckAlert(),HostName,RichListBox,LogTimeStamp,
                              ClassName,RecorderData,Stale,Reason)){
      this.Display(HostName,Reason.Text);
      return;
    };
}// of T_HotLine.prototype.DoAlert

T_HotLine.prototype.AcknowledgeAlert = function (HostName,LogTimeStamp,ClassName,RecorderData,
                                                   Postponed,Reason){
  //  T_ByReference Postponed
  var Content,Tag='';
  var Acknowledgement=new T_Acknowledgement();
  if (RecorderData.ItsXUL())
     Content=RecorderData.Abstract()
  else
     if (RecorderData.ItsTagged()){
       Tag=RecorderData.Tag;
       Content=RecorderData.TaggedText;
     }
     else
       Content=RecorderData.Text();
  var LocalTime=FormattedYYYYMMDDHHMMSSNNNToDate(LogTimeStamp);
  if (!Acknowledgement.Initialize(HotLineConstant.AckAlert(),
                                  true,
                                  HostName,LocalTime,ClassName,
                                  RecorderData.Opal(ClassName),
                                  RecorderData.TimeStamp(),
                                  Content,
                                  Tag,
                                  Reason))
    return false;
  if (!DoAcknowledge(Acknowledgement,Reason))
    return false;
  if (Acknowledgement.Acknowledged){
    //  Delete this item from the Alert Table
    var RichListBox=document.getElementById('Alert-RichListBox');
    if (!this.DeleteHostRichListItem(HostName,LogTimeStamp,RichListBox,Reason))
      return false;
    //  Delete the entry from the List of outstanding Alerts
  };
  //  Leave the acknowledgement until later
  Postponed.Value=!Acknowledgement.Acknowledged;
  return true;
}// of T_HotLine.prototype.AcknowledgeAlert

//**********************************************************
//                     Status                              *
//**********************************************************

T_HotLine.prototype.AgeStatusRichListItems = function (Reason){
  var RichListBox=document.getElementById('Status-RichListBox');
  return this.AgeRichListItems(RichListBox,T_HotLineConstant.StatusReactionTimeInSeconds,Reason);
}// of T_HotLine.prototype.AgeStatusRichListItems

T_HotLine.prototype.DoStatus = function (HostName,LogTimeStamp,ClassName,RecorderData,Stale){
  //  This function is called by the HotLineHost when
  //  a Status message has been selected. If a Tag is attached, it
  //  is used as the key to identify an entry, so it can be deleted
  //  or modified. If there is no Tag, the entry is added.
  var Reason=new T_Reason();
  var Tagged=RecorderData.ItsTagged();
  var Cancel=(Tagged ? (RecorderData.Action == '-') : false);
  var RichListBox=document.getElementById('Status-RichListBox');

  //  Handle UnTagged Status Messages
  if (!Tagged){
    //  Add it to the Status Panel
    if (!this.AddRichListItem(HotLineConstant.AckStatus(),HostName,RichListBox,LogTimeStamp,
                              ClassName,RecorderData,Stale,Reason))
      this.Display(HostName,Reason.Text);
    return;
  };
  //  Tagged Status Messages (A Tag Action of '-' Cancels)
  if (Cancel){
    //  Delete any RichListItem with a matching Tag
    if (!this.DeleteTaggedRichListItems(HostName,RichListBox,RecorderData,Reason)){
      this.Display(HostName,Reason.Text);
      return;
    };
  }
  else
    if (!this.ModifyTaggedRichListItems(HotLineConstant.AckStatus(),HostName,RichListBox,
                                        LogTimeStamp,ClassName,RecorderData,Stale,
                                        Reason)){
      this.Display(HostName,Reason.Text);
      return;
    };
}// of T_HotLine.prototype.DoStatus

T_HotLine.prototype.AcknowledgeStatus = function (HostName,LogTimeStamp,ClassName,RecorderData,Reason){
  var Content,Tag='';
  var Acknowledgement=new T_Acknowledgement();
  if (RecorderData.ItsXUL())
     Content=RecorderData.Abstract()
  else
     if (RecorderData.ItsTagged()){
       Tag=RecorderData.Tag;
       Content=RecorderData.TaggedText;
     }
     else
       Content=RecorderData.Text();
  var LocalTime=FormattedYYYYMMDDHHMMSSNNNToDate(LogTimeStamp);
  if (!Acknowledgement.Initialize(HotLineConstant.AckStatus(),
                                  false,
                                  HostName,LocalTime,ClassName,
                                  RecorderData.Opal(ClassName),
                                  RecorderData.TimeStamp(),
                                  Content,
                                  Tag,
                                  Reason))
    return false;
  if (!DoAcknowledge(Acknowledgement,Reason))
    return false;
  //  There is nothing to acknowledge for Status Items
  return true; 
}// of T_HotLine.prototype.AcknowledgeStatus

//**********************************************************
//                Recorder Tree Selection                  *
//**********************************************************

T_HotLine.prototype.DoTreeSelection = function (event){
  //  Left Mouse Click. Right click is the Context Menu
  if (event.button != T_HotLineConstant.LeftMouseButton)
     return;

  var Reason=new T_Reason();
  var Tree=document.getElementById('Recorder-Tree');
  var Index=Tree.currentIndex;
  //  Determine the Mouse Hit
  var Row = new Object();
  var Column= new Object();
  var ChildElement = new Object();
  Tree.treeBoxObject.getCellAt(event.pageX,event.pageY,Row,Column,ChildElement);
  var TreeColumn=Column.value;
  //  Get the Data
  var Builder=Tree.builderView;
  var Resource=Builder.getResourceAtIndex(Index);
  //  Find the Host that owns this Msg
  var HostName=Tree.view.getCellText(Index,Tree.columns[T_HotLineConstant.HostNameTreeColIndex]);
  var Host = this.GetHotLineHost(HostName,true,Reason);
  if (Host == null){
    this.Display(HostName,Reason.Text);
    return;
  };
  //  Get the RecorderData for this Resource from the Host DataSource
  var RecorderData=Host.GetRecorderData(Resource,Reason);
  if (RecorderData==null){
    this.Display(HostName,Reason.Text);
    return;
  };
  if (ChildElement.value == 'image'){
    //  Clicked on the Information Image (the only image)
    //  If there is no XUL, then finished
    if (!RecorderData.ItsXUL())
      return;
    //  Render onto the Screen that which is XUL
    if (!this.ShowDetails(HostName,RecorderData,Reason))
      this.Display(HostName,Reason.Text);
  };
  //  otherwise text, cell or twisty (see getCellAt)
}// of T_HotLine.prototype.DoTreeSelection


//  T_HotLine Class Items

var HotLine = new T_HotLine();

//**********************************************************
//                T_HotLineHost                            *
//  This type represents a HotLine Host. The persistent    *
//  properties of the host are stored in the Hosts.rdf     *
//  DataSource.                                            *
//  When a Connect request is made, a HotLine Session      *
//  is created and initialized and a connection attempt    *
//  is made.                                               *
//  If the Connection fails, the Session is destroyed.     *
//  If a Session already exists when a Connect is          *
//  requested, the Connect request rejected.               *
//**********************************************************

//  T_HotLineHost Properties
function T_HotLineHost(HotLine){
  this.HotLine=HotLine;
  this.Configurator=this.HotLine.Configurator;
  this.ItsConnected=false;
}// of T_HotLineHost

//  T_HotLineHost Methods 
T_HotLineHost.prototype.Initialize = function (HostName,ElementName,Reason){
  //  Initialize this Host. 
  if (!this.Configurator.FindHost(HostName,Reason))
    return false; 
  this.HostName=HostName;
  this.ChangeConnectionState(false,'Initializing ' + HostName);
  //  A Session will be created when the Connect is requested
  this.Session=null;
  //  Load the StyleSheet for the Host
  var StyleSheet=this.Configurator.StyleSheetURI(HostName);
  this.Display('Adding StyleSheet ' + StyleSheet + ' for ' + HostName);
  if (!T_StyleSheet.AddDOMStyleSheet(StyleSheet,true,Reason))
    return false;
  //  Rebuild the Skins to load the StyleSheet. This reloads the skins
  //  for FireFox and any extensions also, so it is a rather heavy handed
  //  approach.
  if (!this.HotLine.RebuildStyleSheets(Reason)) 
    return false;
  //  Load the Recorder File 
  this.File=new T_RecorderFile(this);
  if (!this.File.Initialize(HostName,'Recorder',Reason))
    return false;
  this.File.AddToDataSources(ElementName,true);
  //  Create the Record Filter
  this.RecordFilter=new T_Filter(this);
  if (!this.RecordFilter.Initialize(T_HotLineConstant.RecordFilter,Reason))
    return false;
  //  Create the Alert Filter
  this.AlertFilter=new T_Filter(this);
  if (!this.AlertFilter.Initialize(T_HotLineConstant.AlertFilter,Reason))
    return false;
  //  Create the Status Filter
  this.StatusFilter=new T_Filter(this);
  if (!this.StatusFilter.Initialize(T_HotLineConstant.StatusFilter,Reason))
    return false;
  //  Set the SelectAll attribute
  var SelectAllAtt = new T_ByReference();
  this.SelectAll=(this.Configurator.GetSelectAll(this.HostName,SelectAllAtt,Reason) && 
                  SelectAllAtt.Value);
  if (!this.RebuildTypeNamesTable(Reason))
    return false;
  this.Display('Initialized Host ' + HostName);
  return true;
}// of T_HotLineHost.prototype.Initialize

T_HotLineHost.prototype.ChangeConnectionState = function(Connected,Text){
  var Reason=new T_Reason();
  this.ItsConnected=Connected;
  Text=(new Date()).toString() + ' (' + Text + ')';
  if (!this.Configurator.SetConnected(this.HostName,Connected,Reason))
    this.Display(Reason.Text);
  if (Connected){
    if (!this.Configurator.SetConnect(this.HostName,Text,Reason))
      this.Display(Reason.Text);
  }
  else{
    if (!this.Configurator.SetDisconnect(this.HostName,Text,Reason))
      this.Display(Reason.Text);
  };
}// of T_HotLineHost.prototype.ChangeConnectionState

T_HotLineHost.prototype.ShutDown = function(Reason){
  if (this.Session == null){
    if (!this.File.Finalize(Reason))
       return false;
    return true;
  };
  //  Build a Record Message to highlight the discontinuity
  var Now=new Date();
  var Text=' ********** ' + this.HostName + ' Shutdown on ' +  Now.toLocaleString() + ' **********';
  var LogTimeStamp=T_RDF.TimeStamp();
  var ClassName=T_RecorderData.NotificationIdV;
  var RecorderData=new T_RecorderData(
          T_RecorderData.Build(T_RecorderData.NotificationV,
                                T_RecorderData.NotificationWhenIdV,
                                T_RecorderData.NotificationVerbV,
                                Text));
  this.File.Store(ClassName,RecorderData,LogTimeStamp);
  if (!this.File.Finalize(Reason))
     return false;
  if (!this.Session.ShutDown(Reason))
    return false;
  this.DestroySession();
  return true;
}// of T_HotLineHost.prototype.ShutDown

T_HotLineHost.prototype.Display = function (Text){
  this.HotLine.Display(this.HostName,Text);
}// of T_HotLineHost.prototype.Display

//**********************************************************
//              StyleSheets                                *
//**********************************************************

T_HotLineHost.prototype.DisplayStyleSheets = function (Heading){
  var Data=T_StyleSheet.GetDOMStyleSheetData();
  this.Display(Heading);
  for (var j=0;j<Data.length;j++)
    this.Display(Data[j]);
}// of T_HotLineHost.prototype.DisplayStyleSheets

//**********************************************************
//                TypeMask                                 *
//**********************************************************

T_HotLineHost.prototype.RebuildRecordFilter = function (Reason){
  if (!this.RecordFilter.Rebuild(Reason))
    return false;
  return this.UpdateTypeMask(Reason);
}// of T_HotLineHost.prototype.RebuildRecordFilter

T_HotLineHost.prototype.RebuildAlertFilter = function (Reason){
  if (!this.AlertFilter.Rebuild(Reason))
    return false;
  return this.UpdateTypeMask(Reason);
}// of T_HotLineHost.prototype.RebuildAlertFilter

T_HotLineHost.prototype.RebuildStatusFilter = function (Reason){
  if (!this.StatusFilter.Rebuild(Reason))
    return false;
  return this.UpdateTypeMask(Reason);
}// of T_HotLineHost.prototype.RebuildStatusFilter

T_HotLineHost.prototype.RebuildTypeNamesTable = function (Reason){
  this.TypeNamesTable=new Array();
  return this.Configurator.BuildTypeNamesTable(this.HostName,this.TypeNamesTable,Reason);
}// of T_HotLineHost.prototype.RebuildTypeNamesTable

T_HotLineHost.prototype.FindTypeName = function (ClassID){
  //  This function is used to search the TypeNames table
  //  for a TypeName with a matching ClassID. It is used
  //  to find the TypeNAme when ALL is selected. If the
  //  ClassName is not found, 'Unknown' is returned.
  var ClassName=null;
  if ((ClassID >= 0) && (ClassID < this.TypeNamesTable.length))
    ClassName=this.TypeNamesTable[ClassID];
  if ((ClassName == undefined) || (ClassName == null))
    return 'Unknown';
  return ClassName;
}// of T_HotLineHost.prototype.FindTypeName

T_HotLineHost.prototype.BuildTypeMask = function (All){
  //  This function returns a T_Word48 object in which
  //  each bit represents a RECORD case which should
  //  be sent to this HotLine.
  var TypeMask = new T_Word48();
  if (All)
    TypeMask.SetAllBits()
  else{
    this.RecordFilter.SetTypeMaskBits(TypeMask);
    this.AlertFilter.SetTypeMaskBits(TypeMask);
    this.StatusFilter.SetTypeMaskBits(TypeMask);
  };
  return TypeMask;
}// of T_HotLineHost.prototype.BuildTypeMask

T_HotLineHost.prototype.UpdateTypeMask = function (Reason){
  //  Refresh the SelectAll value from the Host Configuration
  var SelectAllAtt = new T_ByReference();
  this.SelectAll=(this.Configurator.GetSelectAll(this.HostName,SelectAllAtt,Reason) && SelectAllAtt.Value);
  //  Rebuild the TypeMask
  var TypeMask = this.BuildTypeMask(this.SelectAll);
  //  Save the new TypeMask as a ReadOnly Host Attribute
  if (!this.Configurator.SetTypeMask(this.HostName,TypeMask.ToHex(),Reason))
    this.Display(Reason.Text);
  //  If its not connected, it will be updated
  //  during host initialization.
  if (!this.ItsConnected)
    return true;
  //  Send the new TypeMask to the MCP
  return this.Session.SendTypeMask(TypeMask,Reason);
}// of T_HotLineHost.prototype.UpdateTypeMask

T_HotLineHost.prototype.RefreshTypeNames = function (Reason){
  //  Refresh the TypeNames by sending a request to
  //  Recorder.
  return this.Session.RequestTypeNames(Reason);
}// of T_HotLineHost.prototype.RefreshTypeNames

T_HotLineHost.prototype.RequestInformation = function (Which,Reason){
  //  Send a Request for the Information specified
  //  by Which to Recorder.
  return this.Session.RequestInformation(Which,Reason);
}// of T_HotLineHost.prototype.RequestInformation

T_HotLineHost.prototype.GetTimeOfDay = function (Reason){
  return this.RequestInformation(T_GetInformation.GetTimeOfDayV,Reason);
}// of T_HotLineHost.prototype.GetTimeOfDay

T_HotLineHost.prototype.ReceivedTypeNames = function (TypeNames){
  var Reason=new T_Reason();
  //  Merge the TypeNames with those stored for the Host
  if (!this.Configurator.MergeTypeNames(this.HostName,TypeNames,Reason))
    this.Display(Reason.Text);
  //  Build the TypeNames Table. This Table contains all the
  //  known TypeNames, so that if ALL is specified, the
  //  TypeName can be found using the ClassID.
  if (!this.RebuildTypeNamesTable(Reason))
    this.Display(Reason.Text);
}// of T_HotLineHost.prototype.ReceivedTypeNames

T_HotLineHost.prototype.ReceivedInformation = function (Which,Data,Text){
  //  Integer Which (such as T_GetInformation.GetTimeOfDayV)
  //  T_Word48 Data, String Text

  //  The information is not yet used, it is just sent
  //  for the Heartbeat (keep alive).

} // of T_HotLineHost.prototype.ReceivedInformation

T_HotLineHost.prototype.AutoConfigureFilters = function (Reason) {
  //  If there are no filters defined, automatically
  //  add META_RECORD, META_STATUS and META_ALERT if
  //  they are defined.

  //  Search the TypeNames Array for the Standard Names 
  for (var j in this.TypeNamesTable){
    if (this.TypeNamesTable[j]=='META_RECORD'){
      if (!(this.Configurator.MarkAsRecordFilter(this.HostName,this.TypeNamesTable[j],Reason) &&
            this.RebuildRecordFilter(Reason)))
        this.Display(Reason.Text); 
    };
    if (this.TypeNamesTable[j]=='META_STATUS'){
      if (!(this.Configurator.MarkAsStatusFilter(this.HostName,this.TypeNamesTable[j],Reason) &&
            this.RebuildStatusFilter(Reason)))
        this.Display(Reason.Text); 
    };
    if (this.TypeNamesTable[j]=='META_ALERT'){
      if (!(this.Configurator.MarkAsAlertFilter(this.HostName,this.TypeNamesTable[j],Reason) &&
            this.RebuildAlertFilter(Reason)))
        this.Display(Reason.Text); 
    };
  };
  return true;
}// of T_HotLineHost.prototype.AutoConfigureFilters

//**********************************************************
//         Connection and Disconnection                    *
//**********************************************************

T_HotLineHost.prototype.CreateSession = function (){
  this.Session=new T_HotLineSession(this);
}// T_HotLineHost.prototype.CreateSession

T_HotLineHost.prototype.DestroySession = function (){
  this.Session=null;
}// T_HotLineHost.prototype.DestroySession

T_HotLineHost.prototype.Connect = function (UserName,Address,Reason){
  //  This function attempts to connect to the
  //  HostName with the given UserName and Address.
  if (this.Session != null){
    Reason.Text=this.HostName + ' Session is already Active';
    return false;
  };
  this.CreateSession();
  //  Attempt to Connect Session
  this.Display('HotLineHost Connect to ' + Address);
  this.Address=Address;
  if (!this.Session.Initialize(UserName,Address,Reason)){
    this.DestroySession();
    return false;
  };
  return true;
}// of T_HotLineHost.prototype.Connect

T_HotLineHost.prototype.Started = function (Text){
  this.Display(Text);
}// of T_HotLineHost.prototype.Started

T_HotLineHost.prototype.Connected = function (ConnectText){
  var Reason = new T_Reason();
  //  Mark as Connected
  this.ChangeConnectionState(true,ConnectText);
  //  Merge the TypeNames from the Connection with those
  //  stored for the Host
  if (!this.Configurator.MergeTypeNames(this.HostName,ConnectText,Reason))
    this.Display(Reason.Text);
  //  Build the TypeNames Table This Table contains all the
  //  known TypeNames, so that if ALL is specified, the
  //  TypeName can be found using the ClassID.
  if (!this.RebuildTypeNamesTable(Reason))
    this.Display(Reason.Text);
  //  Auto Configure Filters
  if ((this.RecordFilter.Count==0) &&
       (this.AlertFilter.Count==0) &&
       (this.StatusFilter.Count==0) &&
       (!this.SelectAll))
    if (!this.AutoConfigureFilters(Reason))
      this.Display(Reason.Text);
  //  Update the TypeMask so it reflects the current
  //  Host configuration
  if (!this.UpdateTypeMask(Reason))
    this.Display(Reason.Text);
  //  Build a Record Message to highlight the discontinuity
  //  Note that the message starts with a blank to avoid
  //  ambiguity with Tagged Messages.
  var Now=new Date();
  var Text=' ********** ' + this.HostName + ' Reconnected on ' +  Now.toLocaleString() + ' **********';
  var LogTimeStamp=T_RDF.TimeStamp();
  var ClassName=T_RecorderData.NotificationIdV;
  var RecorderData=new T_RecorderData(
          T_RecorderData.Build(T_RecorderData.NotificationV,
                                T_RecorderData.NotificationWhenIdV,
                                T_RecorderData.NotificationVerbV,
                                Text));
  this.File.Store(ClassName,RecorderData,LogTimeStamp);
  //  Forward the notification to global HotLine
  this.HotLine.DoConnected(this.HostName,ConnectText);
}// of T_HotLineHost.prototype.Connected

T_HotLineHost.prototype.Rejected = function (RejectText){
  var Reason = new T_Reason();
  this.HotLine.DoRejected(this.HostName,RejectText);
  if (!this.Session.ShutDown(Reason))
    this.Display(Reason.Text);
  this.DestroySession();
}// of T_HotLineHost.prototype.Rejected

T_HotLineHost.prototype.Disconnected = function (DisconnectText){
  this.ChangeConnectionState(false,DisconnectText);
  this.HotLine.DoDisconnected(this.HostName,DisconnectText);
  this.DestroySession();
}// of T_HotLineHost.prototype.Disconnected

T_HotLineHost.prototype.OneMinuteBurden = function (Reason){
  //  If not connected, there is nothing to do
  if (!this.ItsConnected)
    return true;
  //  Check whether the time since tha last message was
  //  received exceeds the Heartbeat Interval.
  var Now=new Date();
  //  If it is connected, there must be a LastReceiveDate
  var Then=this.Session.LastReceiveDate;
  var Age=Math.floor((Now.getTime()-Then.getTime())/1000);

  if (Age > T_HotLineConstant.HeartbeatIntervalInSeconds){
    var Text='+Heartbeat: Last Message Received at ' +
              Then.toLocaleString() + '. Disconnect ' + this.HostName;
    var LogTimeStamp=T_RDF.TimeStamp();
    var ClassName=T_RecorderData.HeartBeatIdV;
    var RecorderData=new T_RecorderData(
            T_RecorderData.Build(T_RecorderData.HeartBeatV,
                                  T_RecorderData.HeartBeatWhenIdV,
                                  T_RecorderData.HeartBeatVerbV,
                                  Text));
    var Cancel=(RecorderData.ItsTagged() ? (RecorderData.Action == '-') : false);
    if (!Cancel)
      this.File.Store(ClassName,RecorderData,LogTimeStamp);
    this.HotLine.DoAlert(HostName,LogTimeStamp,ClassName,RecorderData,true);
    if (!this.Session.ShutDown(Reason))
      return false;
    this.DestroySession();
  }
  else{
    if (!this.GetTimeOfDay(Reason))
      return false;
  };
  return true;
}// of  T_HotLineHost.prototype.OneMinuteBurden

//**********************************************************
//              Recording                                  *
//**********************************************************

T_HotLineHost.prototype.Record = function (RecorderData){
  var LogTimeStamp = new T_ByReference();
  var ClassID = RecorderData.ClassID();
  //  Check the Recorder Table
  var ClassName = this.RecordFilter.GetTypeName(ClassID);
  //  If SelectAll is set, check if the ClassID is
  //  listed in the Status or Alert Filter Tables.
  //  If so, do not record them. This prevents the
  //  Recorder log filling up with Status and Alert
  //  messages.
  if ((ClassName != null) || 
       (this.SelectAll && (this.StatusFilter.GetTypeName(ClassID) == null) &&
                          (this.AlertFilter.GetTypeName(ClassID) == null))   ){
    if (ClassName == null)
      ClassName=this.FindTypeName(ClassID);
    this.File.Store(ClassName,RecorderData,LogTimeStamp);
    this.HotLine.DoRecord(this.HostName,ClassName,RecorderData); 
  }
  else
    LogTimeStamp.Value=T_RDF.TimeStamp();
  //  Check the Alert Filter Table
  ClassName = this.AlertFilter.GetTypeName(ClassID);
  if (ClassName != null){
    var Cancel=(RecorderData.ItsTagged() ? (RecorderData.Action == '-') : false);
    if (!Cancel)
      this.File.Store(ClassName,RecorderData,LogTimeStamp);
    this.HotLine.DoAlert(this.HostName,LogTimeStamp.Value,ClassName,RecorderData,false);
  }
  //  Check the Status Table. In the first version of HotLine,
  //  a Status message was an alert with a '*' as a tag. In the
  //  FireFox version, a separate filter is defined.
  ClassName = this.StatusFilter.GetTypeName(ClassID);
  if (ClassName != null)
    this.HotLine.DoStatus(this.HostName,LogTimeStamp.Value,ClassName,RecorderData,false);
}// of T_HotLineHost.prototype.Record

T_HotLineHost.prototype.Notify = function (NotifyText){
  this.Display(NotifyText);
}// of T_HotLineHost.prototype.Notify

//**********************************************************
//                  Details                                *
//**********************************************************

T_HotLineHost.prototype.GetRecorderData = function (Resource,Reason){
  return this.File.GetRecorderData(Resource,Reason);
}// of T_HotLineHost.prototype.GetRecorderData

//**********************************************************
//                CleanUp                                  *
//**********************************************************

T_HotLineHost.prototype.DiscardExpiredRecords = function (HoursToKeep,Reason){
  return this.File.ExpireRecords(HoursToKeep,Reason);
}// of T_HotLineHost.prototype.DiscardExpiredRecords

//**********************************************************
//                 Start HotLine                           *
//  HotLineOpened is called from the onload event which    *
//  happens after the HotLine window has been constructed. *
//  HotLineClosed is called from the onunload event which  *
//  happens after the HotLine window has been closed.      *
//**********************************************************
var HotLineOpen = false;

function SetIcon(IconURL){
  //  This function sets the icon on the Tab when HotTabbing
  //  There has to be a better way.
  var WindowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                                  .getService(Components.interfaces.nsIWindowMediator);
  var Windows=WindowMediator.getEnumerator('');
  while(Windows.hasMoreElements()) {
    var NextWindow = Windows.getNext();
    if (NextWindow.gBrowser){
      var Tabs=NextWindow.gBrowser.tabContainer.childNodes;
      for (var j=0;j<Tabs.length;j++)
        if (Tabs[j].ItsHotLine)
          NextWindow.gBrowser.setIcon(Tabs[j],IconURL);
    };
  };
}//  SetIcon

function HotLineOpened(event){
  //  The HotLineOpen flag is used because when HotLine is initiated
  //  by AddTab, the onload event fires twice, once with 
  //  eventPhase=AT_TARGET(2) and then with eventPhase=BUBBLING_PHASE(3).
  if (HotLineOpen){
     return;
  };
  var Reason = new T_Reason();
  var Error = function (Why){
                alert(Why);
                window.close();
                return false;
              };
  if (!HotLineConstant.Initialize(Reason))
    return Error(Reason.Text);
  SetIcon('chrome://hotline/content/hotline.png');
  if (!Status.Initialize(Reason)) 
    return Error('Status Not Initialized ' + Reason.Text);
  Status.Display('','Initializing HotLine');
  if (!Configurator.Initialize(HotLineConstant,Status,Reason))
    Status.Display('',Reason.Text) ;
  if (!HotLine.Initialize(Status,Configurator,Reason))
    Status.Display('',Reason.Text);
  HotLineOpen=true;
}// of HotLineOpened

function HotLineClosed(){
  var Reason = new T_Reason();
  if (!HotLine.ShutDown(Reason))
    alert('HotLine Closed ' + Reason.Text);
  //  Save the Configuration Files
  if (!this.Configurator.Finalize(Reason))
    alert(Reason.Text);
  HotLineOpen=false;
}// of HotLineClosed 


