/***********************************************************
 *                   HotLineConstants                      *
 * This file contains HotLine Constants which are used     *
 * within the HotLine code but are not exposed to the      *
 * user interface and therefore need not be                *
 * internationalized.                                      *
 *                                                         *
 *                      History                            *
 * 13/04/2006  Initial Version                             *
 * 08/05/2006  Change StyleSheet from .css to .style       *
 * 23/05/2006  HotLine Version 4.0                         *
 *             Implement HoursToKeep attribute             *
 * 25/05/2006  Implement ShortTimeStamp                    *
 * 29/05/2006  Add DirPermissions                          *
 * 07/06/2006  Add Heartbeat constants                     *
 * 19/06/2006  Add Details                                 *
 * 04/07/2006  Add ChromePlotKit                           *
 ***********************************************************/


//**********************************************************
//                T_HotLineConstant                        *
//  This type is used to manage the stringbundle stored    *
//  in hotlineconstants.properties.                        *
//**********************************************************

//  T_HotLineConstant Properties
function T_HotLineConstant(){
}// of T_HotLineConstant

//  T_HotLineConstants Methods
T_HotLineConstant.prototype.Initialize = function (Reason){
  this.StringBundle=document.getElementById('HotLineConstants');
  this.DayName=new Array();
  //  Index matches Date.getDay()
  this.DayName[0]=this.GetDayOfWeek('Sunday');
  this.DayName[1]=this.GetDayOfWeek('Monday');
  this.DayName[2]=this.GetDayOfWeek('Tuesday');
  this.DayName[3]=this.GetDayOfWeek('Wednesday');
  this.DayName[4]=this.GetDayOfWeek('Thursday');
  this.DayName[5]=this.GetDayOfWeek('Friday');
  this.DayName[6]=this.GetDayOfWeek('Saturday');

  //  Get the Preferences

  var HotLinePreferences=new T_Preferences('extensions.Metalogic.HotLine',false);
  this.Monitoring=HotLinePreferences.GetBoolean('HotMonitoring');
  this.Tabbing=HotLinePreferences.GetBoolean('HotTabbing');

  //  If the dom.max_script_run_time is less than 20 seconds
  //  a 'Warning:Unresponsive Script' alert is displayed and
  //  suspends HotLine when a flurry of messages is received
  //  from the MCP. Set it to 21, to give us a hint that it
  //  was us who changed it.
  var SystemPreferences=new T_Preferences('dom',false);
  if (SystemPreferences.GetInteger('max_script_run_time') < 20)
     SystemPreferences.SetInteger('max_script_run_time',21);

  //  Get the DataPath from the Preferences. The DataPath is the directory
  //  which contains the Hosts.rdf and the <hostname>.log for each of the 
  //  configured Hosts. 
  
  this.DataPath=HotLinePreferences.GetFile('DataPath');
  if (!this.DataPath){
    //  If the DataPath has not been specified then ask for the DataPath
    this.DataPath=this.SelectDirectory('Data Directory');
    if (!this.DataPath){
      Reason.Text='HotLine closing because the Data Directory is not defined';
      return false;
    };
    HotLinePreferences.SetFile('DataPath',this.DataPath);
  };
  
  return true;
}// of T_HotLineConstant.prototype.Initialize

T_HotLineConstant.prototype.OpenConfigurator = function (){
  return this.StringBundle.getString('OpenConfigurator');
}// of  T_HotLineConstant.prototype.OpenConfigurator

T_HotLineConstant.prototype.CloseConfigurator = function (){
  return this.StringBundle.getString('CloseConfigurator');
}// of  T_HotLineConstant.prototype.CloseConfigurator

T_HotLineConstant.prototype.OpenHelp = function (){
  return this.StringBundle.getString('OpenHelp');
}// of  T_HotLineConstant.prototype.OpenHelp

T_HotLineConstant.prototype.CloseHelp = function (){
  return this.StringBundle.getString('CloseHelp');
}// of  T_HotLineConstant.prototype.CloseHelp

T_HotLineConstant.prototype.HideRecNames = function (){
  return this.StringBundle.getString('HideRecNames');
}// of  T_HotLineConstant.prototype.HideRecNames

T_HotLineConstant.prototype.ViewRecNames = function (){
  return this.StringBundle.getString('ViewRecNames');
}// of  T_HotLineConstant.prototype.ViewRecNames

T_HotLineConstant.prototype.AckAlert = function (){
  return this.StringBundle.getString('AckAlert');
}// of  T_HotLineConstant.prototype.AckAlert

T_HotLineConstant.prototype.AckStatus = function (){
  return this.StringBundle.getString('AckStatus');
}// of  T_HotLineConstant.prototype.AckStatus

T_HotLineConstant.prototype.GetDayOfWeek = function (Key){
  //  {Key:Sunday,Monday,Tuesday,Wednesday,Thursday,Friday,Saturday}
  return this.StringBundle.getString(Key);
}// of  T_HotLineConstant.prototype.GetDayOfWeek

T_HotLineConstant.prototype.DayOfWeek = function (getDay){
  //  getDay 0-Sunday..6-Saturday
  return this.DayName[getDay];
}// of  T_HotLineConstant.prototype.DayOfWeek

T_HotLineConstant.prototype.ShortDayOfWeek = function (getDay){
  //  getDay 0-Sunday..6-Saturday
  var Day=this.DayName[getDay];
  return Day.slice(0,3);
}// of  T_HotLineConstant.prototype.ShortDayOfWeek

T_HotLineConstant.prototype.ShortTimeStamp = function (Day,Hours,Minutes){
  //  {Day 0-Sunday..6-Saturday
  return this.ShortDayOfWeek(Day) + ' ' + 
          (Hours <= 9 ? '0' : '') + Hours + ':' + 
          (Minutes <= 9 ? '0' : '') + Minutes;
}// of  T_HotLineConstant.prototype.ShortTimeStamp

T_HotLineConstant.prototype.SelectDirectory = function (Caption){
   // Example: SelectDirectory('Configuration Data')
   const nsIFilePicker = Components.interfaces.nsIFilePicker;
   var FilePicker = Components.classes["@mozilla.org/filepicker;1"]
 	                                        .createInstance(nsIFilePicker);
   FilePicker.init(window,Caption,nsIFilePicker.modeGetFolder);
   var Result = FilePicker.show();
   if (Result == nsIFilePicker.returnOK) 
     return FilePicker.file;
   return null;
}// of  T_HotLineConstant.prototype.SelectDirectory

//  T_HotLineConstant Class Items 

T_HotLineConstant.ExtensionTitle        = 'HotLine';
T_HotLineConstant.ExtensionName         = 'hotline';
T_HotLineConstant.BackupDirectory       = 'hotbackup';
T_HotLineConstant.OldBackupDirectory    = 'coldbackup';
T_HotLineConstant.DirPermissions        = 0777;
T_HotLineConstant.ExtensionDomain       = 'www.metalogic.eu.com';
T_HotLineConstant.ExtensionID           = T_HotLineConstant.ExtensionName + '@' + T_HotLineConstant.ExtensionDomain;
T_HotLineConstant.HostsConfigFile       = 'hosts.rdf';
T_HotLineConstant.RDFNameSpace          = 'http://www.metalogic.lu/hotline/rdf';
T_HotLineConstant.RecordFilter          = 'recordfilter';
T_HotLineConstant.AlertFilter           = 'alertfilter';
T_HotLineConstant.StatusFilter          = 'statusfilter';

T_HotLineConstant.DefaultColour         = '#FFFF00';
T_HotLineConstant.DefaultFontFamily     = 'monospace';
T_HotLineConstant.DefaultTextSize       = 10;               // pt

T_HotLineConstant.DefaultHoursToKeep    = 24;

//  Properties
T_HotLineConstant.DetailsProperty       = 'details';


//  The Index of the HostName Cell in the Recorder Tree. Used to obtain the
//  HostName that owns the REsource when getting Details.
T_HotLineConstant.HostNameTreeColIndex  = 1;

//  This is the the number of seconds after which the class of a status message 
//  or alert is changed from 'fresh' to 'stale', so that a style change can warn the
//  observer of an uncorrected condition.

T_HotLineConstant.AlertReactionTimeInSeconds  = (2*60);
T_HotLineConstant.StatusReactionTimeInSeconds = (3*60);

//  This is the number of seconds after which if no message has been
//  received, a Hearbeat Alert will be generated.
 
T_HotLineConstant.HeartbeatIntervalInSeconds  = (5*60);

//  Standard Constants

T_HotLineConstant.LeftMouseButton             = 0;  //  See onclick function
T_HotLineConstant.RightMouseButton            = 2;

T_HotLineConstant.Chrome = function(Path){
  //  Chrome('locale/en-US/overlay.dtd') returns
  //     chrome://hotline/locale/en-US/overlay.dtd
  return 'chrome://'+T_HotLineConstant.ExtensionName + '/' + Path;
} // of T_HotLineConstant.Chrome

T_HotLineConstant.ChromeContent = function(FileName){
  //  ChromeContent('alpha.log') returns
  //     chrome://hotline/content/alpha.log
  return T_HotLineConstant.Chrome('content') + '/' + FileName;
} // of T_HotLineConstant.ChromeContent

T_HotLineConstant.ChromeSkin = function(FileName){
  //  ChromeSkin('alpha.css') returns
  //     chrome://hotline/skin/alpha.css
  return T_HotLineConstant.Chrome('skin') + '/' + FileName;
} // of T_HotLineConstant.ChromeSkin

T_HotLineConstant.ChromeLocale = function(FileName){
  //  ChromeSkin('hotline.dtd') returns
  //     chrome://hotline/locale/hotline.dtd
  return T_HotLineConstant.Chrome('locale') + '/' + FileName;
} // of T_HotLineConstant.ChromeLocale

T_HotLineConstant.ChromePlotKit = function(FileName){
  //  ChromePlotKit('Base.js') returns
  //     chrome://hotline/content/plotkit/Base.js
  return T_HotLineConstant.Chrome('content') + '/plotkit/' + FileName;
} // of T_HotLineConstant.ChromePlotKit

T_HotLineConstant.ChromeMochiKit = function(FileName){
  //  ChromeMochiKit('Base.js') returns
  //     chrome://hotline/content/mochikit/Base.js
  return T_HotLineConstant.Chrome('content') + '/mochikit/' + FileName;
} // of T_HotLineConstant.ChromeMochiKit

T_HotLineConstant.StyleSheetFileName = function(HostName){
  //  StyleSheetFileName('alpha') returns
  //     alpha.style
  //  Note: .css is not used to avoid collision
  //        with hotline stylesheets
  return HostName + '.style';
} // of T_HotLineConstant.StyleSheetFileName

T_HotLineConstant.LogFileName = function(HostName){
  //  LogFileName('alpha') returns
  //     alpha.log
  return HostName + '.log';
} // of T_HotLineConstant.LogFileName

T_HotLineConstant.HostClass = function(HostName){
  //  HostClass('alpha') returns
  //     Host-alpha
  return 'Host-' + HostName;
} // of T_HotLineConstant.HostClass

T_HotLineConstant.DetailsButtonClass = function(HostName){
  //  DetailsButtonClass('alpha') returns
  //     Details-alpha
  return 'Details-Button-' + HostName;
} // of T_HotLineConstant.DetailsButtonClass

T_HotLineConstant.AttributeNameClass = function(HostName){
  //  AttributeNamesClass('alpha') returns
  //     Attribute-Names-alpha
  return 'Attribute-Name-' + HostName;
} // of T_HotLineConstant.AttributeNameClass

T_HotLineConstant.StatusRichListItemClass = function(HostName){
  //  StatusItemClass('alpha') returns
  //     Host-alpha-Status
  return 'Host-' + HostName + '-Status';
} // of T_HotLineConstant.StatusRichListItemClass

T_HotLineConstant.StaleDescriptionClass = function(HostName){
  //  StaleDescriptionClass('alpha') returns
  //     Host-alpha-Stale-Description
  return 'Host-' + HostName + '-Stale-Description';
} // of T_HotLineConstant.StaleDescriptionClass

T_HotLineConstant.StaleRichListItemClass = function(HostName){
  //  StaleRichListItemClass('alpha') returns
  //     Host-alpha-Stale-RichListItem
  return 'Host-' + HostName + '-Stale-RichListItem';
} // of T_HotLineConstant.StaleRichListItemClass

var HotLineConstant=new T_HotLineConstant();

