/***********************************************************
 *                   HotLineSupport                        *
 * This file contains JavaScript types and functions which *
 * support the HotLine extension.                          *
 *                                                         *
 * Uses                                                    *
 *   generalsupport.js                                     *
 *   mcpsupport.js                                         *
 *   hotlineconstants.js                                   *
 *                                                         *
 *                      History                            *
 * 20/02/2006  Initial Version                             *
 * 16/05/2006  Add T_RecorderData Hint                     *
 * 23/05/2006  HotLine Version 4.0                         *
 * 31/05/2006  Add ShortTime to Recorder File              *
 *             Implement T_GetTypeNames                    *
 * 02/06/2006  Add T_RecorderData.Abstract()               *
 * 07/06/2006  Add T_GetInformation                        *
 *             Implemented T_RecorderData.Build            *
 * 12/06/2006  Flush RecorderFile in Finalize not Store    *
 * 13/06/2006  Replace Debug by HotLineConstant.Monitoring *
 * 19/06/2006  Add Recorder XUL Details                    *
 ***********************************************************/

//**********************************************************
//                T_RequestHeader                          *
//  A T_RequestHeader is embedded in a HotLine Request.    *
//**********************************************************

//  T_RequestHeader Properties
function T_RequestHeader(){
  if (arguments.length==0)
    this.Msg=''
  else
    this.Msg=Msg;
}// of T_RequestHeader

T_RequestHeader.GetTypeNamesV              = 10;      // 0x0A
T_RequestHeader.SetTypeMaskV               = 11;      // 0x0B
T_RequestHeader.GetInformationV            = 12;      // 0x0C
T_RequestHeader.RecorderDataV              = 20;      // 0x14

//  T_RequestHeader Methods

T_RequestHeader.prototype.Load = function(Msg){
  this.Msg=Msg;
  return true;
}// of T_RequestHeader.prototype.Load

T_RequestHeader.prototype.HdrVersion = function(){
  var Word48 = new T_Word48(this.Msg,0)
  return Word48.ToString();
}// of T_RequestHeader.prototype.HdrVersion

T_RequestHeader.prototype.RqCode = function(){
  return T_Rq.GetInt16(this.Msg,6);
}// of T_RequestHeader.prototype.RqCode

T_RequestHeader.prototype.More = function(){
  return T_Rq.GetBoolean(this.Msg,8);
}// of T_RequestHeader.prototype.More

T_RequestHeader.prototype.Error = function(){
  return T_Rq.GetBoolean(this.Msg,9);
}// of T_RequestHeader.prototype.Error

T_RequestHeader.prototype.Reason = function(){
  // Remove When Tested 25/Apr/2006
  var Index=this.GetDescriptorIndex(this.Msg,10);
  var Length=this.GetDescriptorLength(this.Msg,10);
  return EBCDICToASCII.Translate(this.Msg.slice(Index,Index+Length));
}// of T_RequestHeader.prototype.Reason

//  T_RequestHeader Class Items

T_RequestHeader.Version = String.fromCharCode(0x81,0x3C,0x3D,0x4E,0x86,0x2D);

T_RequestHeader.PutDescriptor = function(Index,Length) {
  return Int16ToString(Index) +
          Int16ToString(Length);
} // of T_RequestHeader.PutDescriptor
  
T_RequestHeader.GetDescriptorIndex = function(Msg,Index) {
  return T_Rq.GetInt16(Msg,Index);
} // of T_RequestHeader.GetDescriptorIndex
  
T_RequestHeader.GetDescriptorLength = function(Msg,Index) {
  return T_Rq.GetInt16(Msg,Index+2);
} // of T_RequestHeader.GetDescriptorLength
  
T_RequestHeader.Build = function(RqCode,More,Error,Reason){
  var VariablePart='';
  var FixedPartSize = 6+2+1+1+4;
  var Index;
  var FixedPart = T_RequestHeader.Version +
                   Int16ToString(RqCode) +
                   BooleanToString(More) +
                   BooleanToString(Error);
  Index=FixedPartSize;
  FixedPart+=T_RequestHeader.PutDescriptor(Index,Reason.length);
  VariablePart+=ASCIIToEBCDIC.Translate(Reason);

  return FixedPart+VariablePart;
}// of T_RequestHeader.Build

//**********************************************************
//                T_SetTypeMask                            *
//  This is a HotLine Request sent to the HotLineAgent     *
//  and is the Data in a T_DataRq.                         *
//**********************************************************

//  T_SetTypeMask Properties
function T_SetTypeMask(){
}
//  T_SetTypeMask Methods

//  T_SetTypeMask Class Items

T_SetTypeMask.FixedPartSize = 18;

T_SetTypeMask.Build = function(Tag,Header,TypeMask){
  //  TypeMask is a T_Word48 reference
  var VariablePart='';
  var Index;
  //  RqLength & Tag
  var FixedPart = Int32ToString(T_SetTypeMask.FixedPartSize+Header.length) +
                   Int32ToString(Tag);
  //  Header
  Index=T_SetTypeMask.FixedPartSize;
  FixedPart+=T_RequestHeader.PutDescriptor(Index,Header.length);
  VariablePart+=Header;
  // TypeMask 
  FixedPart+=TypeMask.ToString();
  return FixedPart+VariablePart;
}// of T_SetTypeMask.Build

//**********************************************************
//                T_GetTypeNames                           *
//  This is a HotLine Request sent to the HotLineAgent     *
//  and is the Data in a T_DataRq.                         *
//**********************************************************

//  T_GetTypeNames Properties
function T_GetTypeNames(Msg,Index){
  this.Msg=Msg;
  this.Base=Index;
}// of T_GetTypeNames

//  T_GetTypeNames Methods

T_GetTypeNames.prototype.Assign = function(Msg,Index){
  this.Msg=Msg;
  this.Base=Index;
  return Index;
}// of T_GetTypeNames.prototype.Assign

T_GetTypeNames.prototype.RqLength = function(){
  return T_Rq.GetInt32(this.Msg,this.Base);
}// of T_GetTypeNames.prototype.RqLength

T_GetTypeNames.prototype.Tag = function(){
  return T_Rq.GetInt32(this.Msg,this.Base+4);
}// of T_GetTypeNames.prototype.Tag

T_GetTypeNames.prototype.RequestHeader = function(){
  var Index,Length;
  Index=T_GetTypeNames.FixedPartSize +
        T_RequestHeader.GetDescriptorIndex(this.Msg,this.Base+8);
  Length=T_RequestHeader.GetDescriptorLength(this.Msg,this.Base+8);
  return this.Msg.slice(this.Base+Index,this.Base+Index+Length);
}// of T_GetTypeNames.prototype.RequestHeader

T_GetTypeNames.prototype.MaxNames = function(){
  return T_Rq.GetInt32(this.Msg,this.Base+12);
}// of T_GetTypeNames.prototype.MaxNames

T_GetTypeNames.prototype.TypeNames = function(){
  var Index,nBytes;
  Index=T_GetTypeNames.FixedPartSize +
        T_RequestHeader.GetDescriptorIndex(this.Msg,this.Base+16);
  nBytes=T_RequestHeader.GetDescriptorLength(this.Msg,this.Base+16);
  return EBCDICToASCII.Translate(this.Msg.slice(this.Base+Index,this.Base+Index+nBytes));
}// of T_GetTypeNames.prototype.TypeNames

//  T_GetTypeNames Class Items

T_GetTypeNames.FixedPartSize = 20;

T_GetTypeNames.Build = function(Tag,Header,MaxNames,TypeNames){
  //  Integer MaxNames, String TypeNames
  var VariablePart='';
  var Index;
  //  RqLength & Tag
  var FixedPart = Int32ToString(T_GetTypeNames.FixedPartSize+Header.length+TypeNames.length) +
                   Int32ToString(Tag);
  //  Header
  Index=T_GetTypeNames.FixedPartSize;
  FixedPart+=T_RequestHeader.PutDescriptor(Index,Header.length);
  VariablePart+=Header;
  Index+=Header.length;
  //  MaxNames 
  FixedPart+=Int32ToString(MaxNames);
  //  TypeNames
  FixedPart+=T_RequestHeader.PutDescriptor(Index,TypeNames.length);
  VariablePart+=TypeNames;
  Index+=TypeNames.length;
  return FixedPart+VariablePart;
}// of T_GetTypeNames.Build

//**********************************************************
//                T_GetInformation                         *
//  This is a HotLine Request sent to the HotLineAgent     *
//  and is the Data in a T_DataRq.                         *
//                (Untested)                               *
//**********************************************************

//  T_GetInformation Properties
function T_GetInformation(Msg,Index){
  this.Msg=Msg;
  this.Base=Index;
}// of T_GetInformation

T_GetInformation.GetTimeOfDayV            = 1;       // 0x01

//  T_GetInformation Methods

T_GetInformation.prototype.Assign = function(Msg,Index){
  this.Msg=Msg;
  this.Base=Index;
  return Index;
}// of T_GetInformation.prototype.Assign

T_GetInformation.prototype.RqLength = function(){
  return T_Rq.GetInt32(this.Msg,this.Base);
}// of T_GetInformation.prototype.RqLength

T_GetInformation.prototype.Tag = function(){
  return T_Rq.GetInt32(this.Msg,this.Base+4);
}// of T_GetInformation.prototype.Tag

T_GetInformation.prototype.RequestHeader = function(){
  var Index,Length;
  Index=T_GetInformation.FixedPartSize +
        T_RequestHeader.GetDescriptorIndex(this.Msg,this.Base+8);
  Length=T_RequestHeader.GetDescriptorLength(this.Msg,this.Base+8);
  return this.Msg.slice(this.Base+Index,this.Base+Index+Length);
}// of T_GetInformation.prototype.RequestHeader

T_GetInformation.prototype.Which = function(){
  return T_Rq.GetInt32(this.Msg,this.Base+12);
}// of T_GetInformation.prototype.Which

T_GetInformation.prototype.Data = function(){
  return new T_Word48(this.Msg,this.Base+16);
}// of T_GetInformation.prototype.Data

T_GetInformation.prototype.Text = function(){
  var Index,nBytes;
  Index=T_GetInformation.FixedPartSize +
        T_RequestHeader.GetDescriptorIndex(this.Msg,this.Base+22);
  nBytes=T_RequestHeader.GetDescriptorLength(this.Msg,this.Base+22);
  return EBCDICToASCII.Translate(this.Msg.slice(this.Base+Index,this.Base+Index+nBytes));
}// of T_GetInformation.prototype.Text

//  T_GetInformation Class Items

T_GetInformation.FixedPartSize = 26;

T_GetInformation.Build = function(Tag,Header,Which,Data,Text){
  //  Integer Which,T_Word48 Data,String Text
  var VariablePart='';
  var Index;
  //  RqLength & Tag
  var FixedPart = Int32ToString(T_GetInformation.FixedPartSize+Header.length+Text.length) +
                   Int32ToString(Tag);
  //  Header
  Index=T_GetInformation.FixedPartSize;
  FixedPart+=T_RequestHeader.PutDescriptor(Index,Header.length);
  VariablePart+=Header;
  Index+=Header.length;
  //  Which
  FixedPart+=Int32ToString(Which);
  //  Data
  FixedPart+=Data.ToString();
  //  Text
  FixedPart+=T_RequestHeader.PutDescriptor(Index,Text.length);
  VariablePart+=ASCIIToEBCDIC.Translate(Text);
  Index+=Text.length;
  return FixedPart+VariablePart;
}// of T_GetInformation.Build

//**********************************************************
//             T_RecorderDataRequest                       *
//  This is a HotLine Request received from the            *
//  HotLineAgent and is the Data in a T_DataRq.            *
//**********************************************************

//  T_RecorderDataRequest Properties
function T_RecorderDataRequest(Msg,Index){
  this.Msg=Msg;
  this.Base=Index;
}
//  T_RecorderDataRequest Methods

T_RecorderDataRequest.prototype.Assign = function(Msg,Index){
  this.Msg=Msg;
  this.Base=Index;
  return Index;
}// of T_RecorderDataRequest.prototype.Assign

T_RecorderDataRequest.prototype.RqLength = function(){
  return T_Rq.GetInt32(this.Msg,this.Base);
}// of T_RecorderDataRequest.prototype.RqLength

T_RecorderDataRequest.prototype.Tag = function(){
  return T_Rq.GetInt32(this.Msg,this.Base+4);
}// of T_RecorderDataRequest.prototype.Tag

T_RecorderDataRequest.prototype.RequestHeader = function(){
  var Index,Length;
  Index=T_RecorderDataRequest.FixedPartSize +
        T_RequestHeader.GetDescriptorIndex(this.Msg,this.Base+8);
  Length=T_RequestHeader.GetDescriptorLength(this.Msg,this.Base+8);
  return this.Msg.slice(this.Base+Index,this.Base+Index+Length);
}// of T_RecorderDataRequest.prototype.RequestHeader

T_RecorderDataRequest.prototype.Data = function(){
  var Index,nElements;
  Index=T_RecorderDataRequest.FixedPartSize +
        T_RequestHeader.GetDescriptorIndex(this.Msg,this.Base+12);
  nElements=T_RequestHeader.GetDescriptorLength(this.Msg,this.Base+12);
  return this.Msg.slice(this.Base+Index,this.Base+Index+(nElements*T_B6700.BytesPerWord));
}// of T_RecorderDataRequest.prototype.Data

//  T_RecorderDataRequest Class Items

T_RecorderDataRequest.FixedPartSize = 16;

//**********************************************************
//                T_RecorderData                           *
// This type provides access to the fields of a RECORDER   *
// binary message.                                         *
// These properties are defined,                           *
//    this.Action     (+ or - or *)                        *
//    this.Tag        (the text between the +/-/* and the :*
//    this.TaggedText (the text after the :)               *
//**********************************************************

//  T_RecorderData Properties
function T_RecorderData(Msg){
  if (arguments.length==0)
    this.Msg=''
  else
    this.Msg=Msg;
  this.CheckTag();
}// of T_RecorderData

//  Layout of Recorder Message
T_RecorderData.RcdTimeStamp              = 0;  // TIME(6)
T_RecorderData.RcdCategory               = 1;  // [47:1] Supervisor End of Page
                                               // [46:3] Version (1 Adds AuxLink and Length)
  T_RecorderData.RcdVersion              = 1;  // [43:20] Length of Record in Words
                                               // [9:1]   HotLine FLag
                                               // [7:8]   Category of Message
  T_RecorderData.RcdRecord               = 1;  // Category of Record Verb in ODTS  
  T_RecorderData.RcdRefresh              = 27; // Names have changed
T_RecorderData.RcdLink                   = 2;  // [47:16]  Link to WhenID
                                               // [31:16]  Link to Verb
                                               // [15:16]  Link to Text
T_RecorderData.RcdUser                   = 3;  // Class in RECORD[<class>(<text>) verb
T_RecorderData.RcdTermInfo               = 4;  // Terminal Info
                                               // [47:1] Suppress Term Output
                                               // [46:1] Output To Printer
T_RecorderData.RcdAuxLink                = 5;  // Secondary Link used by Filters
T_RecorderData.RcdVariableLength         = 6;  // Words

T_RecorderData.HeartBeatV                = 48; // ClassID
T_RecorderData.HeartBeatIdV              = 'Heartbeat'; 
T_RecorderData.HeartBeatWhenIdV          = 'HEARTBEAT';
T_RecorderData.HeartBeatVerbV            = 'HOTLINE';

T_RecorderData.NotificationV             = 48; // ClassID
T_RecorderData.NotificationIdV           = 'Notification'; 
T_RecorderData.NotificationWhenIdV       = 'NOTIFICATION';
T_RecorderData.NotificationVerbV         = 'HOTLINE';


//  T_RecorderData Methods

T_RecorderData.prototype.Load = function(Msg){
  this.Msg=Msg;
  this.CheckTag();
  return true;
}// of T_RecorderData.prototype.Load

T_RecorderData.prototype.TimeStamp = function(){
  var DFHTimeStamp = new T_DFHTimeStamp(new T_Word48(this.Msg,T_RecorderData.RcdTimeStamp));
  return DFHTimeStamp.Date;
}// of T_RecorderData.prototype.TimeStamp

T_RecorderData.prototype.Category = function(){
  var Word48=new T_Word48(this.Msg,T_RecorderData.RcdCategory*T_B6700.BytesPerWord);
  return Word48.FieldIsolate(7,8);
}// of T_RecorderData.prototype.Category

T_RecorderData.prototype.LengthInWords = function(){
  var Word48=new T_Word48(this.Msg,T_RecorderData.RcdCategory*T_B6700.BytesPerWord);
  return Word48.FieldIsolate(43,20);
}// of T_RecorderData.prototype.LengthInWords

T_RecorderData.prototype.Text = function(){
  var iStart,iEnd;
  var Word48=new T_Word48(this.Msg,(T_RecorderData.RcdLink*T_B6700.BytesPerWord));
  iStart=Word48.FieldIsolate(15,16)*T_B6700.BytesPerWord;
  if (this.Msg.length-iStart == 0)
   return '';
  iEnd=this.Msg.indexOf('\0',iStart);
  if (iEnd < 0)
    iEnd=this.Msg.length;
  return EBCDICToASCII.Translate(this.Msg.slice(iStart,iEnd));
}// of T_RecorderData.prototype.Text

T_RecorderData.prototype.CheckTag = function(){
  //  Returns the Tag or null. If there is a valid Tag
  //  the other Tag properties are initialized
  var Inspect=this.Text().match(/^(\+|\-|\*)([^:]*)(:)(.*)/);
  if ((Inspect==null) || Empty(Inspect[2])){
    this.Tag=null;
    return;
  };
  this.Action=Inspect[1];
  this.Tag=Inspect[2];
  this.TaggedText=Inspect[4];
}// of T_RecorderData.prototype.CheckTag

T_RecorderData.prototype.ItsTagged = function(){
  return (this.Tag != null);
}// of T_RecorderData.prototype.ItsTagged

T_RecorderData.prototype.ItsXUL = function(){
  //  Returns true if the Text contains <?xul>
  return (this.Text().search(/.*<\?xul>/) >= 0)
}// of T_RecorderData.prototype.Tagged

T_RecorderData.prototype.XUL = function(){
  //  Returns the XUL from the Text.
  var Text=(this.ItsTagged() ? this.TaggedText : this.Text());
  var Inspect=Text.match(/.*<\?xul>(.*)/);
  if (Inspect == null)
    return '';
  return Inspect[1];
}// of T_RecorderData.prototype.XUL

T_RecorderData.prototype.Abstract = function(){
  //  Returns the text before the <?xul>,
  //  not including any Tag, to be used
  //  as the abstract, with the XUL 
  //  presenting the details.
  var Text=(this.ItsTagged() ? this.TaggedText : this.Text());
  var Inspect=Text.match(/(.*)<\?xul>.*/);
  if (Inspect == null)
    return '';
  return Inspect[1];
}// of T_RecorderData.prototype.Abstract

T_RecorderData.prototype.GetSimpleName = function(iStartingBit){
  var iStart,iEnd;
  var Word48=new T_Word48(this.Msg,(T_RecorderData.RcdLink*T_B6700.BytesPerWord));
  iStart=Word48.FieldIsolate(iStartingBit,16)*T_B6700.BytesPerWord;
  if (this.Msg.length-iStart <= 0)
   return '';
  iEnd=iStart+this.Msg.charCodeAt(iStart)+1;
  return EBCDICToASCII.Translate(this.Msg.slice(iStart+1,iEnd));
}// of T_RecorderData.prototype.GetSimpleName

T_RecorderData.prototype.Verb = function(){
  return this.GetSimpleName(31);
}// of T_RecorderData.prototype.Verb

T_RecorderData.prototype.WhenID = function(){
  return this.GetSimpleName(47);
}// of T_RecorderData.prototype.WhenID

T_RecorderData.prototype.ClassID = function(){
  var Word48=new T_Word48(this.Msg,(T_RecorderData.RcdUser*T_B6700.BytesPerWord));
  return Word48.FieldIsolate(31,32);
}// of T_RecorderData.prototype.ClassID

T_RecorderData.prototype.Opal = function(Name){
  //  Verb/Whenid Record[Name(ClassId)]
  return this.Verb() + '/' + this.WhenID() + ' Record[' + Name +
          '(' + this.ClassID() + ')]';
}// of T_RecorderData.prototype.Opal

T_RecorderData.prototype.Hint = function(Name){
  //  Verb/Whenid Record[Name(ClassId)](Text)
  return this.Opal() + '("' + this.Text() + '")';
}// of T_RecorderData.prototype.Hint

//  T_RecorderData Class Items

T_RecorderData.Build = function(ClassID,WhenID,Verb,Text){
  //  Integer ClassID, String WhenID,Verb,Text
  var W=new T_Word48();
  var VariablePart='';
  //  Length
  var LengthInWords=T_RecorderData.RcdVariableLength +
                    T_B6700.WordsIn(WhenID.length+1) +
                    T_B6700.WordsIn(Verb.length+1) +
                    T_B6700.WordsIn(Text.length+1);
  var LengthInBytes=(LengthInWords*T_B6700.BytesPerWord);
  var Index=T_RecorderData.RcdVariableLength;
  //  Word 0 - TimeStamp 
  var FixedPart = T_Word48.ZEROES();
  //  Word 1 - Category Word
  W.Assign(T_RecorderData.RcdRecord);
  W.FieldInsert(T_RecorderData.RcdVersion,46,3);
  FixedPart+=W.ToString();
  //  Simple Form WhenID 
  var iWhenID=T_RecorderData.RcdVariableLength;
  VariablePart+=T_B6700.FillToWordBoundary(T_B6700.SimpleName(ASCIIToEBCDIC.Translate(WhenID)),0);
  //  Simple Form Verb
  var iVerb=iWhenID+T_B6700.WordsIn(WhenID.length+1);
  VariablePart+=T_B6700.FillToWordBoundary(T_B6700.SimpleName(ASCIIToEBCDIC.Translate(Verb)),0);
  //  Text, terminated with a NUL
  var iText=iVerb+T_B6700.WordsIn(Verb.length+1);
  VariablePart+=T_B6700.FillToWordBoundary(ASCIIToEBCDIC.Translate(Text)+String.fromCharCode(0),0);
  // Word 2 - Links
  W.Assign(iText);
  W.FieldInsert(iVerb,31,16);
  W.FieldInsert(iWhenID,47,16);
  FixedPart+=W.ToString();
  //  Word 3 - Recorder Class
  W.Assign(ClassID);
  FixedPart+=W.ToString();
  //  Word 4 - Terminal Info
  FixedPart+=T_Word48.ZEROES();
  //  Word 5 - Auxilliary Link
  FixedPart+=T_Word48.ZEROES();

  return FixedPart+VariablePart;
}// of T_RecorderData.Build

//**********************************************************
//               Network Services                          *
//**********************************************************

var TransportService=Components.classes["@mozilla.org/network/socket-transport-service;1"].
                                        getService(Components.interfaces.nsISocketTransportService);

//**********************************************************
//              T_HotLineListener                          *
//**********************************************************

//  T_HotLineListener Methods

function T_HotLineListener(HotLineSession){
  this.HotLineSession=HotLineSession;
}// of T_HotLineListener

T_HotLineListener.prototype.onStartRequest =  function(request, context){
  this.HotLineSession.DoStartRequest(request,context);
} // of  T_HotLineListener.prototype.onStartRequest

T_HotLineListener.prototype.onStopRequest =  function(request, context, status){
  this.HotLineSession.DoStopRequest(request,context,status);
} // of  T_HotLineListener.prototype.onStopRequest

T_HotLineListener.prototype.onDataAvailable = function(request, context, inputStream, offset, count){
  this.HotLineSession.DoDataAvailable(request,context,inputStream,offset,count);
}// of T_HotLineListener.prototype.onDataAvailable
   

//**********************************************************
//             T_HotLineSession                            *
//  This type manages a connection to an MCP host.         *
//**********************************************************

//  T_HotLineSession Properties        
function T_HotLineSession(HotLineHost){
  this.HotLineHost=HotLineHost;
}// of T_HotLineSession

//  T_HotLineSession Methods

T_HotLineSession.prototype.Display = function (Text){
  this.HotLineHost.Display(Text);
}// of T_HotLineSession.prototype.Display

T_HotLineSession.prototype.RemoteHostId = function(){
  return this.Host + ' on Port ' + this.Port;
}// of T_HotLineSession.prototype.RemoteHostId

T_HotLineSession.prototype.Initialize = function(UserName,Host,Reason){
  //  Host is the IPAddress
  var Result;
  this.Port = 22256;
  this.Host=Host;
  this.Transport=TransportService.createTransport(null,0,this.Host,this.Port,null);
  if (this.Transport == null){
    Reason.Text='Failed to Create Transport to ' + this.RemoteHostId();
    return false;
  };
  // ---------------- Output Stream -----------------------
  this.OutputStream=this.Transport.openOutputStream(0,0,0);
  if (this.OutputStream == null) {
    Reason.Text='Failed to Create Output Stream';
    return false;
  };
  // ---------------- Input Stream ------------------------
  this.InputStream = this.Transport.openInputStream(0,0,0);
  if (this.InputStream == null) {
    Reason.Text='Failed to Create Input Stream';
    return false;
  };
  //  Binary Wrapper
  this.BinaryStream = Components.classes["@mozilla.org/binaryinputstream;1"].
                                createInstance(Components.interfaces.nsIBinaryInputStream);
  if (this.BinaryStream == null){
    Reason.Text='Failed to Create Binary Stream';
    return false;
  };
  this.BinaryStream.setInputStream(this.InputStream);
  //  Dispatcher
  this.Dispatcher = new T_Dispatcher();
  //  Listener
  this.HotLineListener=new T_HotLineListener(this);
  //  Pump
  this.Pump = Components.classes["@mozilla.org/network/input-stream-pump;1"].
                           createInstance(Components.interfaces.nsIInputStreamPump);
  if (this.BinaryStream == null) {
    Reason.Text='Failed to Create Pump';
    return false;
  };
  this.Pump.init(this.InputStream, -1, -1, 0, 0, false);
  this.Pump.asyncRead(this.HotLineListener,null);

  //  The following messages types are loaded into these objects:
  //     Connect Response
  this.ConnectResp = new T_ConnectResp();

  //     T_DataRq Message from MCP
  this.DataRq = new T_DataRq();

  //     T_RecorderDataRequest is the payload of a DataRq.Data
  this.RecorderDataRequest = new T_RecorderDataRequest('',0);

  //     T_RequestHeader is embedded in T_RecorderDataRequest
  this.RequestHeader = new T_RequestHeader(); 

  //     T_RecorderData is payload of RecorderDataRequest.Data
  this.RecorderData = new T_RecorderData();

  //     T_GetTypeNames is the payload of a DataRq.Data
  this.GetTypeNames = new T_GetTypeNames('',0);

  //     T_GetInformation is the payload of a DataRq.Data
  this.GetInformation = new T_GetInformation('',0);

  //     T_NotifyRq Message from MCP
  this.NotifyRq = new T_NotifyRq();


  // ---------------- Connect Request ---------------------
  this.ConnectRq = T_ConnectRq.Build(10,
                                     UserName,
                                     'HOTLINEAGENT',
                                     T_Rq.BYFUNCTIONV,
                                     'COPYWRITEHOTAGENT',
                                     '',
                                     '',
                                     'LINUX');
  this.Display('Connect Session to HotLineAgent at ' + this.RemoteHostId() + ' as ' + UserName);
  //  Send the ConnectRq
  if (!this.Writer(this.ConnectRq,Reason))
    return false;

  return true;
}// of T_HotLineSession.prototype.Initialize

T_HotLineSession.prototype.ShutDown = function(Reason){
  //  Called from T_HotLineHost.ShutDown
  this.InputStream.close();
  this.OutputStream.close();
  return true;
}// of T_HotLineSession.prototype.ShutDown

T_HotLineSession.prototype.Writer = function(Msg,Reason){
  this.OutputStream.write(Msg,Msg.length);
  return true;
}// of T_HotLineSession.prototype.Writer

T_HotLineSession.prototype.SendTypeMask = function(TypeMask,Reason){
  //  TypeMask is a T_Word48 Object
  var SetTypeMaskMsg = 
         T_SetTypeMask.Build(0xCCC,  // Tag
                             T_RequestHeader.Build(T_RequestHeader.SetTypeMaskV,false,false,''), 
                             TypeMask);
  var SetTypeMaskRq = T_DataRq.Build(T_RequestHeader.SetTypeMaskV,SetTypeMaskMsg);
  return this.Writer(SetTypeMaskRq,Reason);
}// of T_HotLineSession.prototype.SendTypeMask

T_HotLineSession.prototype.RequestTypeNames = function(Reason){
  var GetTypeNamesMsg = 
         T_GetTypeNames.Build(0xCCC,  // Tag
                             T_RequestHeader.Build(T_RequestHeader.GetTypeNamesV,false,false,''), 
                             0,'');
  var GetTypeNamesRq = T_DataRq.Build(T_RequestHeader.GetTypeNamesV,GetTypeNamesMsg);
  return this.Writer(GetTypeNamesRq,Reason);
}// of T_HotLineSession.prototype.RequestTypeNames

T_HotLineSession.prototype.RequestInformation = function(Which,Reason){
  var GetInformationMsg = 
         T_GetInformation.Build(0xCCC,  // Tag
                                T_RequestHeader.Build(T_RequestHeader.GetInformationV,false,false,''), 
                                Which,
                                new T_Word48(),
                                '');
  var GetInformationRq = T_DataRq.Build(T_RequestHeader.GetInformationV,GetInformationMsg);
  return this.Writer(GetInformationRq,Reason);
}// of T_HotLineSession.prototype.RequestTypeNames

T_HotLineSession.prototype.DoStartRequest = function(request, context){
  //  Called from T_HotLineListener
  this.HotLineHost.Started(this.Host);
}// of  T_HotLineSession.prototype.DoStartRequest

T_HotLineSession.prototype.DoStopRequest = function(request, context, status){
  //  Called from T_HotLineListener when the connection is closed
  this.InputStream.close();
  this.OutputStream.close();
  this.HotLineHost.Disconnected(status);
}// of  T_HotLineSession.prototype.DoStopRequest

T_HotLineSession.prototype.DoDataAvailable = function(request, context, inputStream, offset, count){
  var Data='';
  var Msg;
  var Result;
  //  Accumulate the Bytes
  for (var j=0;j<count;j++){
    Data+=String.fromCharCode(this.BinaryStream.read8())};
  if (HotLineConstant.Monitoring)
    this.Display('Rcv:' + HexString(Data));
  //  Dispatch Messages
  this.Dispatcher.Receive(Data);
  this.LastReceiveDate=new Date();
  while ((Msg=this.Dispatcher.GetMsg()).length > 0){
    if (HotLineConstant.Monitoring)
      this.Display('Msg:' + HexString(Msg));
    switch (T_Rq.GetMsgType(Msg)){
  case T_Rq.CONNECTRESPONSEV:
        this.ConnectResp = new T_ConnectResp();
        this.ConnectResp.Load(Msg);
        if (!this.ConnectResp.Accepted)
          this.DoRejected(this.ConnectResp.Reason)
        else
          this.DoConnected(this.ConnectResp.Reason);
        break;
  case T_Rq.DATARQV:
        this.DataRq.Load(Msg);
        if (HotLineConstant.Monitoring)
          this.Display('Data:' + HexString(this.DataRq.Data()));
        switch (T_Rq.GetMsgTag(Msg)) {
     case T_RequestHeader.GetTypeNamesV:
           this.GetTypeNames.Assign(this.DataRq.Data(),0);
           this.RequestHeader.Load(this.GetTypeNames.RequestHeader());
           this.DoTypeNames(this.GetTypeNames.TypeNames());
           break;
     case T_RequestHeader.SetTypeMaskV:
           //  This is an outbound request
           break;
     case T_RequestHeader.GetInformationV:
           this.GetInformation.Assign(this.DataRq.Data(),0);
           this.RequestHeader.Load(this.GetInformation.RequestHeader());
           this.DoInformation(this.GetInformation.Which(),
                              this.GetInformation.Data(),
                              this.GetInformation.Text());
           break;
     case T_RequestHeader.RecorderDataV:
           this.RecorderDataRequest.Assign(this.DataRq.Data(),0);
           this.RequestHeader.Load(this.RecorderDataRequest.RequestHeader());
           this.RecorderData.Load(this.RecorderDataRequest.Data());
           switch (this.RecorderData.Category()){
         case T_RecorderData.RcdRecord:
               this.DoRecord(this.RecorderData);
               break;
         case T_RecorderData.RcdRefresh:
               //  The TypeNames have been changed, send a GetTypeNames Request
               var Reason=new T_Reason();
               if (!this.RequestTypeNames(Reason))
                  this.Display(Reason.Text);
               break;
         default:
               this.Display('Invalid Category ' +  this.RecorderData.Category() + ' (' + this.RemoteHostId() + ')');
               break;
           };//  Switch
           break;
     default:
           this.Display('Invalid RqCode ' +  this.RequestHeader.RqCode() + ' (' + this.RemoteHostId() + ') ' +
                        HexString(Msg));
           break;
        }; // Switch
      break;
  case T_Rq.NOTIFYRQV:
        this.NotifyRq.Load(Msg);
        this.DoNotified(this.NotifyRq);
      break;
  default:
      this.Display('Invalid MsgType ' +  T_Rq.GetMsgType(Msg) + ' (' + this.RemoteHostId() + ')');
    };// Switch
  };// While
}// of  T_HotLineSession.prototype.DoDataAvailable

T_HotLineSession.prototype.DoConnected = function(ConnectedText){
  var Reason=new T_Reason();
  if (HotLineConstant.Monitoring)
    this.Display('Connected to ' + this.RemoteHostId() + '(' + ConnectedText + ')');
  this.HotLineHost.Connected(ConnectedText);
}// of T_HotLineSession.prototype.DoConnected

T_HotLineSession.prototype.DoRejected = function(RejectedText){
  if (HotLineConstant.Monitoring)
    this.Display('Connect to ' + this.RemoteHostId() + ' rejected because ' + RejectedText);
  this.HotLineHost.Rejected(RejectedText);
}// of T_HotLineSession.prototype.DoRejected

T_HotLineSession.prototype.DoRecord = function(RecorderData){
  this.HotLineHost.Record(RecorderData);
}// of T_HotLineSession.prototype.DoRecord

T_HotLineSession.prototype.DoNotified = function(NotifyRq){
  this.HotLineHost.Notify(NotifyRq.Text());
}// of T_HotLineSession.prototype.DoNotified

T_HotLineSession.prototype.DoTypeNames = function(TypeNames){
  this.HotLineHost.ReceivedTypeNames(TypeNames);
}// of T_HotLineSession.prototype.DoTypeNames

T_HotLineSession.prototype.DoInformation = function(Which,Data,Text){
  //  T_Word48 Data, String Text
  this.HotLineHost.ReceivedInformation(Which,Data,Text);
}// of T_HotLineSession.prototype.DoInformation

//**********************************************************
//                T_RecorderFile                           *
//  This type is used to manage the Recorder File, which   *
//  is an RDF DataSource.                                  *
//**********************************************************

//  T_RecorderFile Properties
function T_RecorderFile(HotLineHost){
  this.HotLineHost=HotLineHost;
}// of T_RecorderFile

T_RecorderFile.prototype.Display = function (Text){
  this.HotLineHost.Display(Text);
}// of T_RecorderFile.prototype.Display

T_RecorderFile.prototype.RecorderFileName = function (){
  return this.HostRecorderFile.Path.ToDisplay();
}// of T_RecorderFile.prototype.RecorderFileName

//  T_RecorderFile Methods

T_RecorderFile.prototype.Initialize = function (HostName,FileType,Reason){
  //  Opens an existing DataSource, or if not present
  //  creates a new DataSource.
  //  The FileType is used to create the filename, so
  //  that this class can be used for Recorder or other
  //  log files.

  this.HostName=HostName;

  //  Load the Host Recorder RDF

//  this.HostRecorderFile=new T_File(new T_ExtensionDirectory(T_HotLineConstant.ExtensionID,
//                                    "content",
//                                    T_HotLineConstant.LogFileName(HostName)));
  this.HostRecorderFile=new T_File(this.HotLineHost.Configurator.LogFilePath(HostName));
  //  Create the T_RDF with the NameSpace
  this.RDF=new T_RDF(T_HotLineConstant.RDFNameSpace);
  this.Display('Loading ' + this.HostRecorderFile.Path.ToURI() + ' for ' + HostName);
  //  If the file is not resident open it with Wait, otherwise an exception
  //  is caused when an attempt is made to add it to the DataSources.
  if (!this.HostRecorderFile.Resident())
    if (!this.CreateRecorderFile(Reason))
       return false;
  if (!this.RDF.Initialize(this.HostRecorderFile.Path.ToURI(),true,Reason))
    this.Display(Reason.Text);

  this.MsgPredicateURI = this.RDF.PredicateURI('msg');
  this.TypePredicateURI = 'http://www.w3.org/1999/02/22-rdf-syntax-ns#type';

  this.LocalTimePredicateURI = this.RDF.PredicateURI('localtime');
  this.HostNamePredicateURI = this.RDF.PredicateURI('hostname');
  this.TimeStampPredicateURI = this.RDF.PredicateURI('timestamp');
  this.ShortTimePredicateURI = this.RDF.PredicateURI('shorttime');
  this.TextPredicateURI = this.RDF.PredicateURI('text');
  this.VerbPredicateURI = this.RDF.PredicateURI('verb');
  this.WhenIDPredicateURI = this.RDF.PredicateURI('whenid');
  this.ClassIDPredicateURI = this.RDF.PredicateURI('classid');
  this.ClassNamePredicateURI = this.RDF.PredicateURI('classname');
  this.PropertiesPredicateURI = this.RDF.PredicateURI('properties');
  this.DetailsPredicateURI = this.RDF.PredicateURI(T_HotLineConstant.DetailsProperty);

  return true;
}// of T_RecorderFile.prototype.Initialize

T_RecorderFile.prototype.Finalize = function (Reason){
  return this.Flush(Reason);
}// of T_RecorderFile.prototype.Finalize

T_RecorderFile.prototype.Flush = function (Reason){
  this.RDF.Save();
  this.Display('Saved ' + this.HostRecorderFile.Path.ToURI());
  return true;
}// of T_RecorderFile.prototype.Flush

T_RecorderFile.prototype.AddToDataSources = function (ElementName,Rebuild){
  this.RDF.AddToDataSources(ElementName,Rebuild);
}// of T_RecorderFile.prototype.AddToDataSources

T_RecorderFile.prototype.CreateRecorderFile = function (Reason){
  //  If the Recorder File is created, the function returns
  //  true otherwise false and a reason. It does not matter
  //  what is written to the file because when it is flushed
  //  it is overwritten
  if (!this.HostRecorderFile.Open(T_File.ReadAndWrite,true,Reason))
    return false;
  if (!this.HostRecorderFile.Write(' ',Reason))
    return false;
  if (!this.HostRecorderFile.Close(Reason))
    return false;
  return true;
}// of T_RecorderFile.prototype.CreateRecorderFile

T_RecorderFile.prototype.Store = function (ClassName,RecorderData,TimeStamp){
  //  ClassName is by value string.
  //  ReorderData is a T_RecorderData object
  //  TimeStamp is a T_ByReference value returned
  var Properties='';
  var Element=new T_RDFDescription(this.RDF);
  var Attribute,Content;

  Element.Initialize(); // anonymous entry

  Attribute=new T_RDFAttribute(this.RDF,this.LocalTimePredicateURI);

  //  This should be a Date created by SetToDateLiteral
  //  but while the NC:parseDate is applied, the
  //  Date value is peculiar.
  var LogTimeStamp=T_RDF.TimeStamp();
  Attribute.SetToLiteral(LogTimeStamp);
  Element.Insert(Attribute);

  Attribute.SetPredicate(this.HostNamePredicateURI);
  Attribute.SetToLiteral(this.HostName);
  Element.Insert(Attribute);

  var DateTime=RecorderData.TimeStamp();
  
  Attribute.SetPredicate(this.TimeStampPredicateURI);
  Attribute.SetToLiteral(DateTime);
  Element.Insert(Attribute);
  
  Attribute.SetPredicate(this.ShortTimePredicateURI);
  Attribute.SetToLiteral(HotLineConstant.ShortTimeStamp(DateTime.getDay(),
                                                        DateTime.getHours(),
                                                        DateTime.getMinutes()));
  Element.Insert(Attribute);
  
  Attribute.SetPredicate(this.TextPredicateURI);
  if (RecorderData.ItsXUL())
     Content=RecorderData.Abstract()
  else
     if (RecorderData.ItsTagged())
       Content=RecorderData.TaggedText
     else
       Content=RecorderData.Text();
  Attribute.SetToLiteral(Content);
  Element.Insert(Attribute);

  if (RecorderData.ItsXUL()){
    //  Set the Details to the XUL 
    Attribute.SetPredicate(this.DetailsPredicateURI);
    Attribute.SetToLiteral(RecorderData.XUL());
    Element.Insert(Attribute);
    //  Set the 'details' property which is used to
    //  style an image on the text cell to indicate
    //  that there is more information.
    Properties+=' ' + T_HotLineConstant.DetailsProperty;
  };
  
  Attribute.SetPredicate(this.VerbPredicateURI);
  Attribute.SetToLiteral(RecorderData.Verb());
  Element.Insert(Attribute);
  
  Attribute.SetPredicate(this.WhenIDPredicateURI);
  Attribute.SetToLiteral(RecorderData.WhenID());
  Element.Insert(Attribute);
  
  Attribute.SetPredicate(this.ClassIDPredicateURI);
  Attribute.SetToLiteral(RecorderData.ClassID());
  Element.Insert(Attribute);
  
  Attribute.SetPredicate(this.ClassNamePredicateURI);
  Attribute.SetToLiteral(ClassName);
  Element.Insert(Attribute);

  if (!Empty(Properties)){
    Attribute.SetPredicate(this.PropertiesPredicateURI);
    Attribute.SetToLiteral(Properties);
    Element.Insert(Attribute);
  };
  
  Attribute.SetPredicate('http://www.w3.org/1999/02/22-rdf-syntax-ns#type');
  Attribute.SetToResource(this.MsgPredicateURI);
  Element.Insert(Attribute);
  
  TimeStamp.Value=LogTimeStamp;
}// of T_RecorderFile.prototype.Store

T_RecorderFile.prototype.GetRecorderData = function (Resource,Reason) {
  //  Rebuild the RecorderData for the nsIRDFResource
  //  If any error, return null and a Reason.

  var ClassID=new T_ByReference();
  if (!this.RDF.GetLiteral(Resource,this.ClassIDPredicateURI,ClassID,Reason))
    ClassID.Value=0;

  var WhenID=new T_ByReference();
  if (!this.RDF.GetLiteral(Resource,this.WhenIDPredicateURI,WhenID,Reason))
    WhenID.Value='unknown';

  var Verb=new T_ByReference();
  if (!this.RDF.GetLiteral(Resource,this.VerbPredicateURI,Verb,Reason))
    Verb.Value='unknown';

  var Text=new T_ByReference();
  if (!this.RDF.GetLiteral(Resource,this.TextPredicateURI,Text,Reason))
    Text.Value='';

  var XUL=new T_ByReference();
  if (!this.RDF.GetLiteral(Resource,this.DetailsPredicateURI,XUL,Reason))
    XUL.Value='';
  if (XUL.Value.length > 0)
    Text.Value+='<?xul>' + XUL.Value;

  return new T_RecorderData(T_RecorderData.Build(ClassID.Value,WhenID.Value,Verb.Value,Text.Value));
}// of T_RecorderFile.prototype.GetRecorderData

T_RecorderFile.prototype.ExpireRecords = function (HoursToKeep,Reason){
  //  Get a list of all the Resources
  var Count=0;
  var Now=new Date();
  var DataSource=this.RDF.DataSource;
  var Predicate=this.RDF.RDFService.GetResource(this.LocalTimePredicateURI);
  var Resources=DataSource.GetAllResources();
  var Verbose=HotLineConstant.Monitoring;
  if (Verbose)
    this.Display('Discarding Recorder Messages older than ' + HoursToKeep + ' Hours');
  try{
    DataSource.beginUpdateBatch();
    while (Resources.hasMoreElements()){
      var Subject=Resources.getNext();
      if (Subject instanceof Components.interfaces.nsIRDFResource){
        var Target=DataSource.GetTarget(Subject,Predicate,true);
        if ((Target==null) ||
            (!(Target instanceof Components.interfaces.nsIRDFLiteral)))
          this.Display(this.RDF.Triple(Subject.Value,this.LocalTimePredicateURI,Target) +' Not Found')
        else{
          var LogTimeStamp=FormattedYYYYMMDDHHMMSSNNNToDate(Target.Value);
          var AgeInHours=Math.floor((Now.getTime()-LogTimeStamp.getTime())/(1000*60*60));
          if (Verbose)
              this.Display('Entry ' + Subject.Value + ' (' + LogTimeStamp.toLocaleString() + 
                           ') Aged ' + AgeInHours + ' Hours');
          if (AgeInHours > HoursToKeep){
            //  Delete Entry 
            if (!T_RDFDescription.DeleteNode(DataSource,Subject,Reason))
              this.Display(Reason.Text)
            else{
              Count++;
              if (Verbose)
                this.Display('[' + Count + '] Deleted Entry ' + Subject.Value + 
                             ' (' + LogTimeStamp.toLocaleString() + 
                             ') Aged ' + AgeInHours + ' Hours');
            }; 
          };
        }
      };
    };// while
    if (Count > 0){
      if (!this.Flush(Reason))
         return false;
      this.Display('Discarded ' + Count + ' Recorder messages older than ' + HoursToKeep + ' hours');
    };
  }
  finally{
    DataSource.endUpdateBatch();
  }; 
  return true;
}// of T_RecorderFile.prototype.ExpireRecords

//**********************************************************
//                T_Filter                                 *
//  This type build the Table used for filtering the       *
//  messages received from Recorder on the MCP.            *
//  It is instantiated once for Recorder Messages and      *
//  once for Alert Messages.                               *
//                                                         *
//  It contains an Array indexed by the ClassId of the     *
//  message. If the ClassId is to be selected, the         *
//  entry indexed by the ClassId contains the Mnemonic     *
//  for the ClassID, otherwise it contains null.           *
//                                                         *
//  The Table is built from the RDF Configuration File     *
//  for the Host, which contains the TypeNames with        *
//  a predicate for AlertFilter and/or Recorder Filter.    *
//**********************************************************

//  T_Filter  Constructor
function T_Filter(HotLineHost){
  this.HotLineHost=HotLineHost;
  this.Configurator=this.HotLineHost.HotLine.Configurator;
}// of T_Filter

T_Filter.prototype.Initialize = function (FilterName,Reason){
  //  The FilterName is the last level of the Predicate
  //  for the Filter to be created.
  this.FilterName=FilterName;
  return this.Rebuild(Reason);
}// of T_Filter.prototype.Initialize

T_Filter.prototype.Display = function (Text){
  this.HotLineHost.Display(Text);
}// of T_Filter.prototype.Display

T_Filter.prototype.Rebuild = function (Reason){
  this.Table=new Array();
  if (!this.Configurator.BuildFilter(this.HotLineHost.HostName,
                                     this.FilterName,
                                     this.Table,
                                     Reason))
    return false;
  this.Count=0;
  for (var j in this.Table)
    this.Count++;
  if (HotLineConstant.Monitoring)
    this.DumpTable();
  return true;
}// of T_Filter.prototype.Rebuild

T_Filter.prototype.GetTypeName = function (ClassId){
  //  This function searches the Table of TypeNames
  //  for the given ClassId. If the ClassId is
  //  in the Table, the function returns the
  //  associated TypeName, otherwise it returns null.
  var Result;
  if ((ClassId < 0) || (ClassId >= Math.max(this.Table.length,T_B6700.BytesPerWord*8)))
     return null;
  Result=this.Table[ClassId];
  if (Result == undefined) 
    return null;
  return Result;
}// of T_Filter.prototype.GetTypeName

T_Filter.prototype.SetTypeMaskBits = function (TypeMask){
  //  TypeMask is a T_Word48
  for (var j=0;j<this.Table.length;j++)
   if (this.GetTypeName(j) != null)
     TypeMask.SetBit(j);
}// of T_Filter.prototype.SetTypeMaskBits

T_Filter.prototype.DumpTable = function (){
  this.Display('----- ' + this.FilterName + ' Table -----');
  for (var j in this.Table)
   this.Display('['+j+'] ' + this.Table[j]);
  this.Display('----- End of Table -----');
}// of T_Filter.prototype.DumpTable


