/***********************************************************
 *                   MCPSupport                            *
 * This file contains JavaScript types and functions which *
 * support the interface to a Burroughs B6700 type system  *
 * running the MCP Operating System.                       *
 *                                                         *
 * T_B6700  - General constants.                           *
 *                                                         *
 * T_Word48 - A type which represents a B6700 48 Bit Word. *
 *                                                         *
 *                                                         *
 *                                                         *
 * Uses                                                    *
 *   generalsupport.js                                     *
 *                                                         *
 *                      History                            *
 * 16/02/2006  Initial Version                             *
 * 23/05/2006  HotLine Version 4.0                         *
 * 07/06/2006  Add WordsIn                                 *
 * 11/02/2009  Fix FieldIsolate with abs when [32:1] set   *
 ***********************************************************/

//**********************************************************
//                T_B6700                                  *
//**********************************************************

//  T_B6700 Properties

function T_B6700(){
}// of T_B6700

//  T_B6700 Class Items

T_B6700.BytesPerWord = 6;

//  T_Word48 Class Items

T_B6700.WordsIn = function(nBytes){
  //  Integer nBytes
  return RoundUp(nBytes,T_B6700.BytesPerWord);
}//of T_Word48.WordsIn

T_B6700.SimpleName = function(Name){
  //  String Name
  return String.fromCharCode(Name.length) + Name;
}//of T_Word48.SimpleName

T_B6700.FillToWordBoundary = function(ByteString,Byte){
  //  String ByteString, Integer Byte
  var T=(T_B6700.WordsIn(ByteString.length)*T_B6700.BytesPerWord)-ByteString.length;
  for (var j=0;j<T;j++)
    ByteString+=String.fromCharCode(Byte);
  return ByteString;
}//of T_Word48.FillToWordBoundary

//**********************************************************
//                T_Word48                                 *
//**********************************************************

//  T_Word48 Properties

function T_Word48(Msg,Index){
  this.Bits = new Array(48);
  //  If created as new T_Word48(), then set to 4'000000000000',
  //  otherwise initialized from 6 Bytes at Msg[Index]
  if (arguments.length==0)
    for (var j=0;j<48;j++)
      this.Bits[j]=false;
  else  
    for (var iBit=47;iBit>=0;iBit--){
      this.Bits[iBit]=(Msg.charCodeAt(Index+(5-DIV(iBit,8))) & (1 << (iBit % 8))) != 0;
  }
}// of T_Word48

//  T_Word48 Methods

T_Word48.prototype.TestBit= function(iBit){
  return this.Bits[iBit];
}// of T_Word48.prototype.TestBit

T_Word48.prototype.SetBit= function(iBit){
  return this.Bits[iBit]=true;
}// of T_Word48.prototype.SetBit

T_Word48.prototype.ResetBit= function(iBit){
  return this.Bits[iBit]=false;
}// of T_Word48.prototype.ResetBit

T_Word48.prototype.SetAllBits= function(){
  for (var j=0;j<48;j++)
    this.Bits[j]=true;
  return true;
}// of T_Word48.prototype.SetAllBits

T_Word48.prototype.ToString = function(){
  var Byte;
  var Value='';
  //  Convert the Bits to a 6 byte B6700 Word48 String
  for (var j=5;j>=0;j--){
    Byte=0;
    for (var k=7;k>=0;k--)
      if (this.Bits[(j*8)+k])
        Byte+=(1<<k);
    Value+=String.fromCharCode(Byte);
  }
  return Value;
}// of T_Word48.prototype.ToString

T_Word48.prototype.ToHex= function(){
  var b;
  var w,s='';
  //  Return the 6 Byte B6700 Word48 in Hex
  w=this.ToString();
  for (var j=0;j<6;j++){
    b=w.charCodeAt(j) & 0xFF;
    s+=HexDigit(b >> 4) + HexDigit(b & 0xF);
  }
  return s;
}// of T_Word48.prototype.ToHex

T_Word48.prototype.FieldIsolate = function(iStartingBit,NumberOfBits){
  var Value=0;
  var k=NumberOfBits-1;
  //  Returns [iStartingBit:NumberOfBits] as a Number
  if (NumberOfBits > 32)
    throw 'Max Field Isolate is 32 Bits';
  for (var j=iStartingBit;j>(iStartingBit-NumberOfBits);j--){
    if (this.Bits[j])
      Value+=Math.abs(1<<k);
    k--;
  };
  return Value;
}//of T_Word48.prototype.FieldIsolate

T_Word48.prototype.FieldInsert = function(Value,iStartingBit,NumberOfBits){
  var  k=(iStartingBit-NumberOfBits)+1;
  //  Inserts Value at [iStartingBit:NumberOfBits]
  if (NumberOfBits > 32)
    throw 'Max Field Insert is 32 Bits';
  for (var j=0;j<NumberOfBits;j++)
    this.Bits[k+j]=(Value & (1<<j)) != 0;
  return Value;
}//of T_Word48.prototype.FieldInsert

T_Word48.prototype.Clear = function(){
  //  Set all bits off
  for (var j=0;j<48;j++)
    this.Bits[j]=false;
}// of T_Word48.prototype.Clear

T_Word48.prototype.Assign = function(Value){
  //  Assign the given positive Integer32
  this.Clear();
  Value=Math.floor(Math.abs(Value));
  for (var j=0;j<32;j++)
    this.Bits[j]=(Value & (1<<j)) != 0;
}// of T_Word48.prototype.Assign

T_Word48.prototype.ToValue= function(){
  //  Return [31:32] as a positive Integer32
  return this.FieldIsolate(31,32);
}// of T_Word48.prototype.ToValue

//  T_Word48 Class Items

T_Word48.ALLONES = function(){
  return String.fromCharCode(0xFF,0xFF,0xFF,0xFF,0xFF,0xFF);
}//of T_Word48.ALLONES

T_Word48.ZEROES = function(){
  return String.fromCharCode(0x00,0x00,0x00,0x00,0x00,0x00);
}//of T_Word48.ZEROES

T_Word48.ONE = function(){
  return String.fromCharCode(0x00,0x00,0x00,0x00,0x00,0x01);
}//of T_Word48.ONE


//**********************************************************
//                T_TranslateTable                         *
//**********************************************************

var EbcdicTable = [
0x00,0x10,0x20,0x30,0x40,0x50,0x60,0x70,0x80,0x90,0xA0,0xB0,0xC0,0xD0,0xE0,0xF0,
0x01,0x11,0x21,0x31,0x41,0x51,0x61,0x71,0x81,0x91,0xA1,0xB1,0xC1,0xD1,0xE1,0xF1,
0x02,0x12,0x22,0x32,0x42,0x52,0x62,0x72,0x82,0x92,0xA2,0xB2,0xC2,0xD2,0xE2,0xF2,
0x03,0x13,0x23,0x33,0x43,0x53,0x63,0x73,0x83,0x93,0xA3,0xB3,0xC3,0xD3,0xE3,0xF3,
0x04,0x14,0x24,0x34,0x44,0x54,0x64,0x74,0x84,0x94,0xA4,0xB4,0xC4,0xD4,0xE4,0xF4,
0x05,0x15,0x25,0x35,0x45,0x55,0x65,0x75,0x85,0x95,0xA5,0xB5,0xC5,0xD5,0xE5,0xF5,
0x06,0x16,0x26,0x36,0x46,0x56,0x66,0x76,0x86,0x96,0xA6,0xB6,0xC6,0xD6,0xE6,0xF6,
0x07,0x17,0x27,0x37,0x47,0x57,0x67,0x77,0x87,0x97,0xA7,0xB7,0xC7,0xD7,0xE7,0xF7,
0x08,0x18,0x28,0x38,0x48,0x58,0x68,0x78,0x88,0x98,0xA8,0xB8,0xC8,0xD8,0xE8,0xF8,
0x09,0x19,0x29,0x39,0x49,0x59,0x69,0x79,0x89,0x99,0xA9,0xB9,0xC9,0xD9,0xE9,0xF9,
0x0A,0x1A,0x2A,0x3A,0x4A,0x5A,0x6A,0x7A,0x8A,0x9A,0xAA,0xBA,0xCA,0xDA,0xEA,0xFA,
0x0B,0x1B,0x2B,0x3B,0x4B,0x5B,0x6B,0x7B,0x8B,0x9B,0xAB,0xBB,0xCB,0xDB,0xEB,0xFB,
0x0C,0x1C,0x2C,0x3C,0x4C,0x5C,0x6C,0x7C,0x8C,0x9C,0xAC,0xBC,0xCC,0xDC,0xEC,0xFC,
0x0D,0x1D,0x2D,0x3D,0x4D,0x5D,0x6D,0x7D,0x8D,0x9D,0xAD,0xBD,0xCD,0xDD,0xED,0xFD,
0x0E,0x1E,0x2E,0x3E,0x4E,0x5E,0x6E,0x7E,0x8E,0x9E,0xAE,0xBE,0xCE,0xDE,0xEE,0xFE,
0x0F,0x1F,0x2F,0x3F,0x4F,0x5F,0x6F,0x7F,0x8F,0x9F,0xAF,0xBF,0xCF,0xDF,0xEF,0xFF
];
var EbcdicToAsciiTable = [
0x00,0x10,0x00,0x00,0x20,0x26,0x2D,0x00,0x00,0x00,0x00,0x00,0x7B,0x7D,0x5C,0x30,
0x01,0x11,0x00,0x00,0x00,0x00,0x2F,0x00,0x61,0x6A,0x7E,0x00,0x41,0x4A,0x00,0x31,
0x02,0x12,0x00,0x16,0x00,0x00,0x00,0x00,0x62,0x6B,0x73,0x00,0x42,0x4B,0x53,0x32,
0x03,0x13,0x00,0x00,0x00,0x00,0x00,0x00,0x63,0x6C,0x74,0x00,0x43,0x4C,0x54,0x33,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x64,0x6D,0x75,0x00,0x44,0x4D,0x55,0x34,
0x09,0x00,0x0A,0x00,0x00,0x00,0x00,0x00,0x65,0x6E,0x76,0x00,0x45,0x4E,0x56,0x35,
0x00,0x08,0x17,0x00,0x00,0x00,0x00,0x00,0x66,0x6F,0x77,0x00,0x46,0x4F,0x57,0x36,
0x7F,0x00,0x1B,0x04,0x00,0x00,0x00,0x00,0x67,0x70,0x78,0x00,0x47,0x50,0x58,0x37,
0x00,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x68,0x71,0x79,0x00,0x48,0x51,0x59,0x38,
0x00,0x19,0x00,0x00,0x00,0x00,0x00,0x60,0x69,0x72,0x7A,0x00,0x49,0x52,0x5A,0x39,
0x00,0x00,0x00,0x00,0x5B,0x5D,0x7C,0x3A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x0B,0x00,0x00,0x00,0x2E,0x24,0x2C,0x23,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x0C,0x1C,0x00,0x14,0x3C,0x2A,0x25,0x40,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x0D,0x1D,0x05,0x15,0x28,0x29,0x5F,0x27,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x0E,0x1E,0x06,0x00,0x2B,0x3B,0x3E,0x3D,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x0F,0x1F,0x07,0x1A,0x21,0x5E,0x3F,0x22,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
];
var AsciiTable = [0x00,0x10,0x20,0x30,0x40,0x50,0x60,0x70,
                   0x01,0x11,0x21,0x31,0x41,0x51,0x61,0x71,
                   0x02,0x12,0x22,0x32,0x42,0x52,0x62,0x72,
                   0x03,0x13,0x23,0x33,0x43,0x53,0x63,0x73,
                   0x04,0x14,0x24,0x34,0x44,0x54,0x64,0x74,
                   0x05,0x15,0x25,0x35,0x45,0x55,0x65,0x75,
                   0x06,0x16,0x26,0x36,0x46,0x56,0x66,0x76,
                   0x07,0x17,0x27,0x37,0x47,0x57,0x67,0x77,
                   0x08,0x18,0x28,0x38,0x48,0x58,0x68,0x78,
                   0x09,0x19,0x29,0x39,0x49,0x59,0x69,0x79,
                   0x0A,0x1A,0x2A,0x3A,0x4A,0x5A,0x6A,0x7A,
                   0x0B,0x1B,0x2B,0x3B,0x4B,0x5B,0x6B,0x7B,
                   0x0C,0x1C,0x2C,0x3C,0x4C,0x5C,0x6C,0x7C,
                   0x0D,0x1D,0x2D,0x3D,0x4D,0x5D,0x6D,0x7D,
                   0x0E,0x1E,0x2E,0x3E,0x4E,0x5E,0x6E,0x7E,
                   0x0F,0x1F,0x2F,0x3F,0x4F,0x5F,0x6F,0x7F];
var AsciiToEbcdicTable = [0x00,0x10,0x40,0xF0,0x7C,0xD7,0x79,0x97,
                           0x01,0x11,0x4F,0xF1,0xC1,0xD8,0x81,0x98,
                           0x02,0x12,0x7F,0xF2,0xC2,0xD9,0x82,0x99,
                           0x03,0x13,0x7B,0xF3,0xC3,0xE2,0x83,0xA2,
                           0x37,0x3C,0x5B,0xF4,0xC4,0xE3,0x84,0xA3,
                           0x2D,0x3D,0x6C,0xF5,0xC5,0xE4,0x85,0xA4,
                           0x2E,0x32,0x50,0xF6,0xC6,0xE5,0x86,0xA5,
                           0x2F,0x26,0x7D,0xF7,0xC7,0xE6,0x87,0xA6,
                           0x16,0x18,0x4D,0xF8,0xC8,0xE7,0x88,0xA7,
                           0x05,0x19,0x5D,0xF9,0xC9,0xE8,0x89,0xA8,
                           0x25,0x3F,0x5C,0x7A,0xD1,0xE9,0x91,0xA9,
                           0x0B,0x27,0x4E,0x5E,0xD2,0x4A,0x92,0xC0,
                           0x0C,0x1C,0x6B,0x4C,0xD3,0xE0,0x93,0x6A,
                           0x0D,0x1D,0x60,0x7E,0xD4,0x5A,0x94,0xD0,
                           0x0E,0x1E,0x4B,0x6E,0xD5,0x5F,0x95,0xA1,
                           0x0F,0x1F,0x61,0x6F,0xD6,0x6D,0x96,0x07];

//  T_TranslateTable Properties
function T_TranslateTable(Source,Destination){
  this.Table = new Array(0xFF);
  for (var j=0;j<Math.min(Source.length,Destination.length);j++){
    this.Table[Source[j]]=Destination[j];
  };
}
//  T_TranslateTable Methods

T_TranslateTable.prototype.Translate = function(Source){
  var Result = '';
  for (var j=0;j<Source.length;j++){
    Result+=String.fromCharCode(this.Table[Source.charCodeAt(j)]);
  };
  return Result;
}
//  T_Translatetable Class Items

//  Intrinsic Translate Tables

var ASCIIToEBCDIC = new T_TranslateTable(AsciiTable,AsciiToEbcdicTable);
var EBCDICToASCII = new T_TranslateTable(EbcdicTable,EbcdicToAsciiTable);

//**********************************************************
//                T_DFHTimeStamp (Time(6) TimeStamp)       *
//**********************************************************

function IsLeapYear(Year){
  return (Year % 4 == 0) && ((Year % 100 != 0) || (Year % 400 == 0));
}// of IsLeapYear

function ConvertTime6ToDate(TimeStamp){
   //  TimeStamp is a T_Word48
   var TM,JD,Hr,Min,Sec,Yr,Mnth,DayOfYr,X,Y;
   TM=TimeStamp.FieldIsolate(31,32)*16.0*(2.4*Math.pow(10,-6));

   Hr=DIV(Math.round(TM),3600);
   Min=DIV(Math.round(TM),60) % 60;
   Sec=Math.round(TM) % 60;

   JD=TimeStamp.FieldIsolate(47,16)+70000;
   Yr=DIV(JD,1000);
   DayOfYr=JD % 1000;
   X=DayOfYr+91;
   if (IsLeapYear(Yr+1900))
     Y=1
   else
     Y=0;
   if ((X-150) > Y)
     X=(X+2-Y);
   Y=DIV(X*100,3055);
   Mnth=((Y-3) % 12)+1;
   X=X-DIV(Y*3055,100);
   Yr+=1900;
   return new Date(Yr,Mnth-1,X,Hr,Min,Sec,0);
}// of ConvertTime6ToDate

//  T_DFHTimeStamp Properties
function T_DFHTimeStamp(TimeStamp){
  this.TimeStamp=TimeStamp;
  this.Date=ConvertTime6ToDate(TimeStamp);
}// of T_DFHTimeStamp

