/***********************************************************
 *                   Preferences                           *
 * This file contains JavaScript types and functions which *
 * provide information about preferences.                  *
 *                                                         *
 * Uses                                                    *
 *   generalsupport.js                                     *
 *                                                         *
 *                      History                            *
 * 08/03/2006  Initial Version                             *
 ***********************************************************/


function T_Preferences(Branch,Default){
  //  Optional String Branch. If specified, it is the branch
  //  of preferences to be referenced.
  //  Optional Boolean Default. If set, the default branch is
  //  selected
  //  If the Branch does not end with a '.', then a '.'
  //  is appended to the Branch
  if (((arguments.length > 0) && (Branch.length > 0)) || Default){
    if (Branch[Branch.length-1] != '.')
       Branch+='.';
    var Service = Components.classes["@mozilla.org/preferences-service;1"].
                                   getService(Components.interfaces.nsIPrefService);
    if (Default)
      this.PrefService=Service.getDefaultBranch(Branch)
    else
      this.PrefService=Service.getBranch(Branch);
  }
  else  // Root Branch
    this.PrefService = Components.classes["@mozilla.org/preferences-service;1"].
                                     getService(Components.interfaces.nsIPrefBranch);

}// of T_Preferences

//  T_Preferences Methods

T_Preferences.prototype.List = function(){
  //  Returns an array of the Preference Names
  var Preferences=this.PrefService.getChildList('',{value:0});
  return Preferences;
}// of T_Preferences.prototype.List

T_Preferences.prototype.Exists = function(Name){
  return this.PrefService.prefHasUserValue(Name);  
}// of  T_Preferences.prototype.Exists

T_Preferences.prototype.Type = function(Name){
  switch (this.PrefService.getPrefType(Name)){
 case this.PrefService.PREF_BOOL:
    return 'boolean';
 case this.PrefService.PREF_INT:
    return 'integer';
 case this.PrefService.PREF_STRING:
    return 'string';
 case this.PrefService.PREF_INVALID:
 default:
    return 'undefined';
  };
}// of  T_Preferences.prototype.Type


T_Preferences.prototype.GetBoolean = function(Name){
  try{
    return this.PrefService.getBoolPref(Name);
  }
  catch(e){
    return false;
  };
}// of  T_Preferences.prototype.GetBoolean

T_Preferences.prototype.SetBoolean = function(Name,Value){
  this.PrefService.setBoolPref(Name,Value);
}// of  T_Preferences.prototype.SetBoolean

T_Preferences.prototype.GetInteger = function(Name){
  try{
    return this.PrefService.getIntPref(Name);
  }
  catch(e){
    return false;
  };
}// of  T_Preferences.prototype.GetInteger

T_Preferences.prototype.SetInteger = function(Name,Value){
  this.PrefService.setIntPref(Name,Value);
}// of  T_Preferences.prototype.SetInteger

T_Preferences.prototype.GetString = function(Name){
  try{
    return this.PrefService.getCharPref(Name);
  }
  catch(e){
    return false;
  };
}// of  T_Preferences.prototype.GetString

T_Preferences.prototype.SetString = function(Name,Value){
  this.PrefService.setCharPref(Name,Value);
}// of  T_Preferences.prototype.SetString


T_Preferences.prototype.GetFile = function(Name){
  try{
    return this.PrefService.getComplexValue(Name, Components.interfaces.nsILocalFile);
  }
  catch(e){
    return false;
  };
}// of  T_Preferences.prototype.GetFile

T_Preferences.prototype.SetFile = function(Name,File){
  //  String Name, nsIFile File
  this.PrefService.setComplexValue(Name, Components.interfaces.nsILocalFile, File);
}// of  T_Preferences.prototype.SetFile


